@@ -17,46 +17,52 @@
  */
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <time.h>
 
 #include <gtk/gtk.h>
 
+#include "gimphelp.h"
+
 #include "libgimp/gimpfeatures.h"
 
-#include "about_dialog.h"
-#include "interface.h"
-
 #include "config.h"
+#include "libgimp/gimpenv.h"
+#include "libgimp/gimpintl.h"
+#include "libgimp/gimpmath.h"
+
+#include "about_dialog.h"
 
 #define ANIMATION_STEPS 16
-#define ANIMATION_SIZE 2
+#define ANIMATION_SIZE   2
 
-static int  about_dialog_load_logo (GtkWidget *window);
-static void about_dialog_destroy (void);
-static void about_dialog_unmap (void);
-static int  about_dialog_logo_expose (GtkWidget *widget, GdkEventExpose *event);
-static int  about_dialog_button (GtkWidget *widget, GdkEventButton *event);
-static int  about_dialog_timer (gpointer data);
+static gint  about_dialog_load_logo   (GtkWidget *window);
+static void  about_dialog_destroy     (void);
+static void  about_dialog_unmap       (void);
+static gint  about_dialog_logo_expose (GtkWidget *widget, GdkEventExpose *event);
+static gint  about_dialog_button      (GtkWidget *widget, GdkEventButton *event);
+static gint  about_dialog_timer       (gpointer data);
 
 
-static GtkWidget *about_dialog = NULL;
-static GtkWidget *logo_area = NULL;
-static GtkWidget *scroll_area = NULL;
-static GdkPixmap *logo_pixmap = NULL;
+static GtkWidget *about_dialog  = NULL;
+static GtkWidget *logo_area     = NULL;
+static GtkWidget *scroll_area   = NULL;
+static GdkPixmap *logo_pixmap   = NULL;
 static GdkPixmap *scroll_pixmap = NULL;
-static unsigned char *dissolve_map = NULL;
-static int dissolve_width;
-static int dissolve_height;
-static int logo_width = 0;
-static int logo_height = 0;
-static int do_animation = 0;
-static int do_scrolling = 0;
-static int scroll_state = 0;
-static int frame = 0;
-static int offset = 0;
-static int timer = 0;
+static guchar    *dissolve_map  = NULL;
+static gint       dissolve_width;
+static gint       dissolve_height;
+static gint       logo_width  = 0;
+static gint       logo_height = 0;
+static gboolean   do_animation = FALSE;
+static gboolean   do_scrolling = FALSE;
+static gint       scroll_state = 0;
+static gint       frame  = 0;
+static gint       offset = 0;
+static gint       timer  = 0;
 
-static char *scroll_text[] =
+/* See the end of the list for how to add names with Non-ASCII characters */
+static gchar *scroll_text[] =
 {
   "Lauri Alanko",
   "Shawn Amundson",
@@ -66,50 +72,79 @@ static char *scroll_text[] =
   "Marc Bless",
   "Edward Blevins",
   "Roberto Boyd",
+  "Stanislav Brabec",
+  "Simon Budig",
   "Seth Burgess",
   "Brent Burton",
   "Francisco Bustamante",
   "Ed Connel",
+  "Jay Cox",
+  "Kenneth Christiansen",
   "Andreas Dilger",
+  "Austin Donnelly",
+  "Scott Draves",
   "Misha Dynin",
+  "Daniel Egger",
   "Larry Ewing",
+  "Nick Fetchak",
+  "Valek Filippov",
   "David Forsyth",
   "Jim Geuther",
   "Scott Goehring",
   "Heiko Goller",
   "Michael Hammel",
+  "James Henstridge",
   "Christoph Hoegl",
+  "Wolfgang Hofer",
   "Jan Hubicka",
   "Simon Janes",
   "Tim Janik",
   "Tuomas Kuosmanen",
-  "Peter Kirchgessner", 
+  "Peter Kirchgessner",
+  "Karin Kylander",
+  "Olof S Kylander",
   "Nick Lamb",
   "Karl LaRocca",
+  "Chris Lahey",
   "Jens Lautenbacher",
   "Laramie Leavitt",
   "Elliot Lee",
+  "Marc Lehmann",
   "Raph Levien",
+  "Wing Tung Leung",
   "Adrian Likins",
+  "Tor Lillqvist",
   "Ingo Luetkebohle",
   "Josh MacDonald",
   "Ed Mackey",
+  "Daniele Medri",
+  "Vidar Madsen",
   "Marcelo Malheiros",
   "Ian Main",
+  "Kjartan Maraas",
+  "Kelly Martin",
   "Torsten Martinsen",
   "Federico Mena",
+  "David Monniaux",
   "Adam D. Moss",
+  "Sung-Hyun Nam",
   "Shuji Narazaki",
+  "Michael Natterer",
   "Sven Neumann",
   "Stephen Robert Norris",
   "Erik Nygren",
+  "Balazs Nagy",
   "Miles O'Neal",
+  "Garry R. Osgood",
   "Jay Painter",
+  "Asbjorn Pettersen",
   "Mike Phillips",
   "Raphael Quinet",
+  "Vincent Renardias",
   "James Robinson",
   "Mike Schaeffer",
   "Tracy Scott",
+  "Aaron Sherman",
   "Manish Singh",
   "Nathan Summers",
   "Mike Sweet",
@@ -121,16 +156,24 @@ static char *scroll_text[] =
   "James Wang",
   "Kris Wehner",
   "Matthew Wilson",
+  "Shirasaki Yasuhiro",
+#ifndef GDK_USE_UTF8_MBS
+  "Ville Hautamki",
+  "Tomas gren",
+#else  /* Win32 GDK uses UTF-8 */
+  "Ville Hautamäki",
+  "Tomas Ögren",
+#endif
 };
-static int nscroll_texts = sizeof (scroll_text) / sizeof (scroll_text[0]);
-static int scroll_text_widths[100] = { 0 };
-static int cur_scroll_text = 0;
-static int cur_scroll_index = 0;
+static gint nscroll_texts = sizeof (scroll_text) / sizeof (scroll_text[0]);
+static gint scroll_text_widths[100] = { 0 };
+static gint cur_scroll_text = 0;
+static gint cur_scroll_index; 
 
-static int shuffle_array[ sizeof(scroll_text) / sizeof(scroll_text[0]) ];
+static gint shuffle_array[ sizeof(scroll_text) / sizeof(scroll_text[0]) ];
 
 void
-about_dialog_create (int timeout)
+about_dialog_create (gint timeout)
 {
   GtkStyle *style;
   GtkWidget *vbox;
@@ -139,20 +182,29 @@ about_dialog_create (int timeout)
   GtkWidget *alignment;
   gint max_width;
   gint i;
+  gchar *label_text;
 
   if (!about_dialog)
     {
       about_dialog = gtk_window_new (GTK_WINDOW_DIALOG);
       gtk_window_set_wmclass (GTK_WINDOW (about_dialog), "about_dialog", "Gimp");
-      gtk_window_set_title (GTK_WINDOW (about_dialog), "About the GIMP");
+      gtk_window_set_title (GTK_WINDOW (about_dialog), _("About the GIMP"));
       gtk_window_set_policy (GTK_WINDOW (about_dialog), FALSE, FALSE, FALSE);
-      gtk_window_position (GTK_WINDOW (about_dialog), GTK_WIN_POS_CENTER);
+      gtk_window_set_position (GTK_WINDOW (about_dialog), GTK_WIN_POS_CENTER);
+
+      gimp_help_connect_help_accel (about_dialog, gimp_standard_help_func,
+				    "dialogs/about.html");
+
       gtk_signal_connect (GTK_OBJECT (about_dialog), "destroy",
-			  (GtkSignalFunc) about_dialog_destroy, NULL);
+			  GTK_SIGNAL_FUNC (about_dialog_destroy),
+			  NULL);
       gtk_signal_connect (GTK_OBJECT (about_dialog), "unmap_event",
-			  (GtkSignalFunc) about_dialog_unmap, NULL);
+			  GTK_SIGNAL_FUNC (about_dialog_unmap),
+			  NULL);
       gtk_signal_connect (GTK_OBJECT (about_dialog), "button_press_event",
-			  (GtkSignalFunc) about_dialog_button, NULL);
+			  GTK_SIGNAL_FUNC (about_dialog_button),
+			  NULL);
+
       gtk_widget_set_events (about_dialog, GDK_BUTTON_PRESS_MASK);
 
       if (!about_dialog_load_logo (about_dialog))
@@ -163,20 +215,22 @@ about_dialog_create (int timeout)
 	}
 
       vbox = gtk_vbox_new (FALSE, 1);
-      gtk_container_border_width (GTK_CONTAINER (vbox), 1);
+      gtk_container_set_border_width (GTK_CONTAINER (vbox), 1);
       gtk_container_add (GTK_CONTAINER (about_dialog), vbox);
       gtk_widget_show (vbox);
 
       aboutframe = gtk_frame_new (NULL);
       gtk_frame_set_shadow_type (GTK_FRAME (aboutframe), GTK_SHADOW_IN);
-      gtk_container_border_width (GTK_CONTAINER (aboutframe), 0);
+      gtk_container_set_border_width (GTK_CONTAINER (aboutframe), 0);
       gtk_box_pack_start (GTK_BOX (vbox), aboutframe, TRUE, TRUE, 0);
       gtk_widget_show (aboutframe);
 
       logo_area = gtk_drawing_area_new ();
       gtk_signal_connect (GTK_OBJECT (logo_area), "expose_event",
-			  (GtkSignalFunc) about_dialog_logo_expose, NULL);
-      gtk_drawing_area_size (GTK_DRAWING_AREA (logo_area), logo_width, logo_height);
+			  GTK_SIGNAL_FUNC (about_dialog_logo_expose),
+			  NULL);
+      gtk_drawing_area_size (GTK_DRAWING_AREA (logo_area),
+			     logo_width, logo_height);
       gtk_widget_set_events (logo_area, GDK_EXPOSURE_MASK);
       gtk_container_add (GTK_CONTAINER (aboutframe), logo_area);
       gtk_widget_show (logo_area);
@@ -184,30 +238,34 @@ about_dialog_create (int timeout)
       gtk_widget_realize (logo_area);
       gdk_window_set_background (logo_area->window, &logo_area->style->black);
 
-
       style = gtk_style_new ();
       gdk_font_unref (style->font);
-      style->font = gdk_font_load ("-Adobe-Helvetica-Medium-R-Normal--*-140-*-*-*-*-*-*");
+      style->font = 
+	gdk_font_load (_("-*-helvetica-medium-r-normal--*-140-*-*-*-*-*-*"));
       gtk_widget_push_style (style);
+      gtk_style_unref (style);
 
-      label = gtk_label_new ("Version " GIMP_VERSION " brought to you by");
+      label_text = g_strdup_printf (_("Version %s brought to you by"),
+				    GIMP_VERSION);
+      label = gtk_label_new (label_text);
+      g_free (label_text);
+      label_text = NULL;
       gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 0);
       gtk_widget_show (label);
 
-      label = gtk_label_new ("Spencer Kimball and Peter Mattis");
+      label = gtk_label_new ("Spencer Kimball & Peter Mattis");
       gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 0);
       gtk_widget_show (label);
 
       gtk_widget_pop_style ();
 
-
       alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
       gtk_box_pack_start (GTK_BOX (vbox), alignment, FALSE, TRUE, 0);
       gtk_widget_show (alignment);
 
       aboutframe = gtk_frame_new (NULL);
       gtk_frame_set_shadow_type (GTK_FRAME (aboutframe), GTK_SHADOW_IN);
-      gtk_container_border_width (GTK_CONTAINER (aboutframe), 0);
+      gtk_container_set_border_width (GTK_CONTAINER (aboutframe), 0);
       gtk_container_add (GTK_CONTAINER (alignment), aboutframe);
       gtk_widget_show (aboutframe);
 
@@ -227,7 +285,7 @@ about_dialog_create (int timeout)
       gtk_container_add (GTK_CONTAINER (aboutframe), scroll_area);
       gtk_widget_show (scroll_area);
 
-      label = gtk_label_new ("Please visit http://www.gimp.org/ for more info");
+      label = gtk_label_new (_("Please visit http://www.gimp.org/ for more info"));
       gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 0);
       gtk_widget_show (label);
 
@@ -252,47 +310,49 @@ about_dialog_create (int timeout)
 
       for (i = 0; i < nscroll_texts; i++) 
 	{
-	  int j, k;
+	  int j;
 	  j = rand() % nscroll_texts;
-	  k = rand() % nscroll_texts;
-	  if (j != k) 
+	  if (i != j) 
 	    {
 	      int t;
 	      t = shuffle_array[j];
-	      shuffle_array[j] = shuffle_array[k];
-	      shuffle_array[k] = t;
+	      shuffle_array[j] = shuffle_array[i];
+	      shuffle_array[i] = t;
 	    }
 	}
+      cur_scroll_text = rand() % nscroll_texts;
     }
   else 
     {
-      gdk_window_raise(about_dialog->window);
+      gdk_window_raise (about_dialog->window);
     }
 }
 
 
-static int
+static gint
 about_dialog_load_logo (GtkWidget *window)
 {
   GtkWidget *preview;
   GdkGC *gc;
-  char buf[1024];
-  unsigned char *pixelrow;
+  gchar buf[1024];
+  guchar *pixelrow;
   FILE *fp;
-  int count;
-  int i, j, k;
+  gint count;
+  gint i, j, k;
 
   if (logo_pixmap)
     return TRUE;
 
-  sprintf (buf, "%s/gimp_logo.ppm", DATADIR);
+  g_snprintf (buf, sizeof(buf), "%s" G_DIR_SEPARATOR_S "gimp_logo.ppm",
+	      gimp_data_directory ());
 
   fp = fopen (buf, "rb");
   if (!fp)
     return 0;
 
   fgets (buf, 1024, fp);
-  if (strcmp (buf, "P6\n") != 0)
+
+  if (strncmp (buf, "P6", 2) != 0)
     {
       fclose (fp);
       return 0;
@@ -303,7 +363,7 @@ about_dialog_load_logo (GtkWidget *window)
   sscanf (buf, "%d %d", &logo_width, &logo_height);
 
   fgets (buf, 1024, fp);
-  if (strcmp (buf, "255\n") != 0)
+  if (strncmp (buf, "255", 3) != 0)
     {
       fclose (fp);
       return 0;
@@ -341,8 +401,10 @@ about_dialog_load_logo (GtkWidget *window)
 
   fclose (fp);
 
-  dissolve_width = (logo_width / ANIMATION_SIZE)+(logo_width % ANIMATION_SIZE ==0 ?0 : 1);
-  dissolve_height = (logo_height / ANIMATION_SIZE)+(logo_height % ANIMATION_SIZE ==0 ?0 : 1);
+  dissolve_width =
+    (logo_width / ANIMATION_SIZE) + (logo_width % ANIMATION_SIZE == 0 ? 0 : 1);
+  dissolve_height =
+    (logo_height / ANIMATION_SIZE) + (logo_height % ANIMATION_SIZE == 0 ? 0 : 1);
 
   dissolve_map = g_new (guchar, dissolve_width * dissolve_height);
 
@@ -356,14 +418,14 @@ about_dialog_load_logo (GtkWidget *window)
 }
 
 static void
-about_dialog_destroy ()
+about_dialog_destroy (void)
 {
   about_dialog = NULL;
   about_dialog_unmap ();
 }
 
 static void
-about_dialog_unmap ()
+about_dialog_unmap (void)
 {
   if (timer)
     {
@@ -372,7 +434,7 @@ about_dialog_unmap ()
     }
 }
 
-static int
+static gint
 about_dialog_logo_expose (GtkWidget      *widget,
 			  GdkEventExpose *event)
 {
@@ -404,7 +466,7 @@ about_dialog_logo_expose (GtkWidget      *widget,
   return FALSE;
 }
 
-static int
+static gint
 about_dialog_button (GtkWidget      *widget,
 		     GdkEventButton *event)
 {
@@ -418,7 +480,7 @@ about_dialog_button (GtkWidget      *widget,
   return FALSE;
 }
 
-static int
+static gint
 about_dialog_timer (gpointer data)
 {
   gint i, j, k;
@@ -437,7 +499,8 @@ about_dialog_timer (gpointer data)
 		  gdk_draw_pixmap (logo_area->window,
 				   logo_area->style->black_gc,
 				   logo_pixmap,
-				   j * ANIMATION_SIZE, i * ANIMATION_SIZE, j * ANIMATION_SIZE, i * ANIMATION_SIZE,
+				   j * ANIMATION_SIZE, i * ANIMATION_SIZE,
+				   j * ANIMATION_SIZE, i * ANIMATION_SIZE,
 				   ANIMATION_SIZE, ANIMATION_SIZE);
 		}
 
@@ -463,7 +526,6 @@ about_dialog_timer (gpointer data)
 					scroll_area->allocation.width,
 					scroll_area->allocation.height,
 					-1);
-  
 
       switch (scroll_state)
 	{
@@ -479,7 +541,8 @@ about_dialog_timer (gpointer data)
 	  break;
 	}
 
-      if (offset > (scroll_text_widths[cur_scroll_text] + scroll_area->allocation.width))
+      if (offset > (scroll_text_widths[cur_scroll_text] +
+		    scroll_area->allocation.width))
 	{
 	  scroll_state = 0;
 	  cur_scroll_index += 1;
@@ -511,10 +574,12 @@ about_dialog_timer (gpointer data)
       offset += 15;
       if (scroll_state == 0)
 	{
-	  if (offset > ((scroll_area->allocation.width + scroll_text_widths[cur_scroll_text]) / 2))
+	  if (offset > ((scroll_area->allocation.width +
+			 scroll_text_widths[cur_scroll_text]) / 2))
 	    {
 	      scroll_state = 1;
-	      offset = (scroll_area->allocation.width + scroll_text_widths[cur_scroll_text]) / 2;
+	      offset = (scroll_area->allocation.width +
+			scroll_text_widths[cur_scroll_text]) / 2;
 	    }
 	}
     }
