@@ -18,9 +18,7 @@
 #ifndef __BUILDMENU_H__
 #define __BUILDMENU_H__
 
-#define PULLDOWN   0
-#define POPUP      1
-#define OPTION     2
+#include <gtk/gtk.h>
 
 /* Structures */
 
@@ -31,9 +29,9 @@ typedef void (*MenuItemCallback) (GtkWidget *widget,
 
 struct _MenuItem
 {
-  char *label;
-  char  accelerator_key;
-  int   accelerator_mods;
+  gchar *label;
+  guint  accelerator_key;
+  guint  accelerator_mods;
   MenuItemCallback callback;
   gpointer user_data;
   MenuItem *subitems;
