@@ -13,28 +13,33 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 #include <stdlib.h>
-#include <stdio.h>
 #include <string.h>
+
 #include "appenv.h"
-#include "actionarea.h"
 #include "colormaps.h"
 #include "cursorutil.h"
 #include "fileops.h"
 #include "gdisplay_ops.h"
-#include "general.h"
 #include "gimage.h"
+#include "gimpui.h"
 #include "gximage.h"
-#include "interface.h"
 #include "menus.h"
 #include "scale.h"
+#include "gimprc.h"
 
+#include "config.h"
+#include "libgimp/gimpintl.h"
 
-static void gdisplay_close_warning_callback (GtkWidget *, gpointer);
-static void gdisplay_cancel_warning_callback (GtkWidget *, gpointer);
-static void gdisplay_close_warning_dialog   (char *, GDisplay *);
+static void gdisplay_close_warning_callback  (GtkWidget *widget,
+					      gboolean   close,
+					      gpointer   data);
+static void gdisplay_close_warning_dialog    (gchar     *image_name,
+					      GDisplay  *gdisp);
+
+static GtkWidget *warning_dialog = NULL;
 
 /*
  *  This file is for operations on the gdisplay object
@@ -65,6 +70,34 @@ gdisplay_color_pixel (GDisplay *gdisp)
 }
 
 
+void
+gdisplay_xserver_resolution (gdouble *xres,
+			     gdouble *yres)
+{
+  gint width, height;
+  gint widthMM, heightMM;
+
+  width  = gdk_screen_width ();
+  height = gdk_screen_height ();
+
+  widthMM  = gdk_screen_width_mm ();
+  heightMM = gdk_screen_height_mm ();
+
+  /*
+   * From xdpyinfo.c:
+   *
+   * there are 2.54 centimeters to an inch; so there are 25.4 millimeters.
+   *
+   *     dpi = N pixels / (M millimeters / (25.4 millimeters / 1 inch))
+   *         = N pixels / (M inch / 25.4)
+   *         = N * 25.4 pixels / M inch
+   */
+
+  *xres = (width  * 25.4) / ((gdouble) widthMM);
+  *yres = (height * 25.4) / ((gdouble) heightMM);
+}
+
+
 void
 gdisplay_new_view (GDisplay *gdisp)
 {
@@ -74,30 +107,40 @@ gdisplay_new_view (GDisplay *gdisp)
   if (gdisp->gimage)
     {
       new_gdisp = gdisplay_new (gdisp->gimage, gdisp->scale);
-      new_gdisp->offset_x = gdisp->offset_x;
-      new_gdisp->offset_y = gdisp->offset_y;
     }
 }
 
 
 void
 gdisplay_close_window (GDisplay *gdisp,
-		       int       kill_it)
+		       gboolean  kill_it)
 {
   /*  If the image has been modified, give the user a chance to save
    *  it before nuking it--this only applies if its the last view
-   *  to an image canvas.  (a gimage with ref_count = 1)
+   *  to an image canvas.  (a gimage with disp_count = 1)
    */
-  if (!kill_it && (gdisp->gimage->ref_count == 1) && (gdisp->gimage->dirty > 0))
-    gdisplay_close_warning_dialog (prune_filename (gimage_filename (gdisp->gimage)), gdisp);
+  if (! kill_it &&
+      (gdisp->gimage->disp_count == 1) &&
+      gdisp->gimage->dirty &&
+      confirm_on_close)
+    {
+      gdisplay_close_warning_dialog
+	(g_basename (gimage_filename (gdisp->gimage)), gdisp);
+    }
   else
-    gtk_widget_destroy (gdisp->shell);
+    {
+      gtk_widget_destroy (gdisp->shell);
+    }
 }
 
 
 void
 gdisplay_shrink_wrap (GDisplay *gdisp)
 {
+  /* FIXME: Still not perfect - have to hide and show the window to 
+   * get it to work.  Adding a queue_resize to the shell doesn't 
+   * seem to help. Anyone have any good ideas here? 
+   */
   gint x, y;
   gint disp_width, disp_height;
   gint width, height;
@@ -105,13 +148,13 @@ gdisplay_shrink_wrap (GDisplay *gdisp)
   gint shell_width, shell_height;
   gint max_auto_width, max_auto_height;
   gint border_x, border_y;
-  int s_width, s_height;
+  gint s_width, s_height;
 
   s_width = gdk_screen_width ();
   s_height = gdk_screen_height ();
 
-  width = SCALE (gdisp, gdisp->gimage->width);
-  height = SCALE (gdisp, gdisp->gimage->height);
+  width = SCALEX (gdisp, gdisp->gimage->width);
+  height = SCALEY (gdisp, gdisp->gimage->height);
 
   disp_width = gdisp->disp_width;
   disp_height = gdisp->disp_height;
@@ -128,21 +171,35 @@ gdisplay_shrink_wrap (GDisplay *gdisp)
   /*  If 1) the projected width & height are smaller than screen size, &
    *     2) the current display size isn't already the desired size, expand
    */
-  if (((width + border_x) < s_width && (height + border_y) < s_height) &&
+  if (((width + border_x) < s_width || (height + border_y) < s_height) &&
       (width != disp_width || height != disp_height))
     {
-      gtk_widget_set_usize (gdisp->canvas,
-			    width, height);
+      width = ((width + border_x) < s_width) ? width : max_auto_width;
+      height = ((height + border_y) < s_height) ? height : max_auto_height;
 
-      gtk_widget_show (gdisp->canvas);
+      if (width < gdisp->statusarea->requisition.width) 
+        { 
+          width = gdisp->statusarea->requisition.width; 
+        }
+      /* I don't know why, but if I don't hide the window, it doesn't work */
+      gtk_widget_hide(gdisp->shell); 
+      gtk_drawing_area_size(GTK_DRAWING_AREA(gdisp->canvas),
+	                    width, height);
+      gtk_widget_show(gdisp->shell); 
+
+      /*printf("1w:%d/%d d:%d/%d s:%d/%d b:%d/%d\n",
+	     width, height,
+	     disp_width, disp_height,
+	     shell_width, shell_height,
+	     border_x, border_y);fflush(stdout);*/
 
       gdk_window_get_origin (gdisp->shell->window, &shell_x, &shell_y);
 
       shell_width = width + border_x;
       shell_height = height + border_y;
 
-      x = HIGHPASS (shell_x, BOUNDS (s_width - shell_width, 0, s_width));
-      y = HIGHPASS (shell_y, BOUNDS (s_height - shell_height, 0, s_height));
+      x = MIN (shell_x, CLAMP (s_width - shell_width, border_x, s_width));
+      y = MIN (shell_y, CLAMP (s_height - shell_height, border_y, s_height));
 
       if (x != shell_x || y != shell_y)
 	gdk_window_move (gdisp->shell->window, x, y);
@@ -155,20 +212,35 @@ gdisplay_shrink_wrap (GDisplay *gdisp)
    *  3/4 of the screen size, expand automagically
    */
   else if ((width > disp_width || height > disp_height) &&
-	   (disp_width < max_auto_width && disp_height < max_auto_height))
+	   (disp_width < max_auto_width || disp_height < max_auto_height))
     {
-      gtk_widget_set_usize (gdisp->canvas,
-			    max_auto_width, max_auto_height);
+      max_auto_width = MIN (max_auto_width, width);
+      max_auto_height = MIN (max_auto_height, height);
+      
+      if (width < gdisp->statusarea->requisition.width) 
+        { 
+          width = gdisp->statusarea->requisition.width; 
+        }
 
-      gtk_widget_show (gdisp->canvas);
+      /* I don't know why, but if I don't hide the window, it doesn't work */
+      gtk_widget_hide(gdisp->shell); 
+      gtk_drawing_area_size(GTK_DRAWING_AREA(gdisp->canvas),
+	                    width, height);
+      gtk_widget_show(gdisp->shell); 
+
+      /*printf("2w:%d/%d d:%d/%d s:%d/%d b:%d/%d\n",
+	     width, height,
+	     disp_width, disp_height,
+	     shell_width, shell_height,
+	     border_x, border_y);fflush(stdout);*/
 
       gdk_window_get_origin (gdisp->shell->window, &shell_x, &shell_y);
 
       shell_width = width + border_x;
       shell_height = height + border_y;
 
-      x = HIGHPASS (shell_x, BOUNDS (s_width - shell_width, 0, s_width));
-      y = HIGHPASS (shell_y, BOUNDS (s_height - shell_height, 0, s_height));
+      x = MIN (shell_x, CLAMP (s_width - shell_width, border_x, s_width));
+      y = MIN (shell_y, CLAMP (s_height - shell_height, border_y, s_height));
 
       if (x != shell_x || y != shell_y)
 	gdk_window_move (gdisp->shell->window, x, y);
@@ -194,17 +266,17 @@ gdisplay_shrink_wrap (GDisplay *gdisp)
 }
 
 
-int
+gint
 gdisplay_resize_image (GDisplay *gdisp)
 {
-  int sx, sy;
-  int width, height;
+  gint sx, sy;
+  gint width, height;
 
   /*  Calculate the width and height of the new canvas  */
-  sx = SCALE (gdisp, gdisp->gimage->width);
-  sy = SCALE (gdisp, gdisp->gimage->height);
-  width = HIGHPASS (sx, gdisp->disp_width);
-  height = HIGHPASS (sy, gdisp->disp_height);
+  sx = SCALEX (gdisp, gdisp->gimage->width);
+  sy = SCALEY (gdisp, gdisp->gimage->height);
+  width = MIN (sx, gdisp->disp_width);
+  height = MIN (sy, gdisp->disp_height);
 
   /* if the new dimensions of the ximage are different than the old...resize */
   if (width != gdisp->disp_width || height != gdisp->disp_height)
@@ -237,68 +309,58 @@ gdisplay_resize_image (GDisplay *gdisp)
  ********************************************************/
 
 static void
-gdisplay_close_warning_callback (GtkWidget *w,
-				 gpointer   client_data)
+gdisplay_close_warning_callback (GtkWidget *widget,
+				 gboolean   close,
+				 gpointer   data)
 {
   GDisplay *gdisp;
-  GtkWidget *mbox;
+
+  gdisp = (GDisplay *) data;
 
   menus_set_sensitive ("<Image>/File/Close", TRUE);
-  mbox = (GtkWidget *) client_data;
-  gdisp = (GDisplay *) gtk_object_get_user_data (GTK_OBJECT (mbox));
-  gtk_widget_destroy (gdisp->shell);
-  gtk_widget_destroy (mbox);
+
+  if (close)
+    gtk_widget_destroy (gdisp->shell);
 }
 
-
 static void
-gdisplay_cancel_warning_callback (GtkWidget *w,
-				  gpointer   client_data)
-{
-  GtkWidget *mbox;
-
-  menus_set_sensitive ("<Image>/File/Close", TRUE);
-  mbox = (GtkWidget *) client_data;
-  gtk_widget_destroy (mbox);
-}
-
-
-static void
-gdisplay_close_warning_dialog (char     *image_name,
+gdisplay_close_warning_dialog (gchar    *image_name,
 			       GDisplay *gdisp)
 {
-  static ActionAreaItem mbox_action_items[2] =
-  {
-    { "Close", gdisplay_close_warning_callback, NULL, NULL },
-    { "Cancel", gdisplay_cancel_warning_callback, NULL, NULL }
-  };
   GtkWidget *mbox;
-  GtkWidget *vbox;
-  GtkWidget *label;
-  char *warning_buf;
+  gchar     *warning_buf;
+
+  /* FIXUP this will raise any prexsisting close dialogs, which can be a
+     a bit confusing if you tried to close a new window because you had
+     forgotten the old dialog was still around */
+  /* If a warning dialog already exists raise the window and get out */
+  if (warning_dialog != NULL)
+    {
+      gdk_window_raise (warning_dialog->window);
+      return;
+    }
 
   menus_set_sensitive ("<Image>/File/Close", FALSE);
 
-  mbox = gtk_dialog_new ();
-  gtk_window_set_title (GTK_WINDOW (mbox), image_name);
-  gtk_window_position (GTK_WINDOW (mbox), GTK_WIN_POS_MOUSE);
-  gtk_object_set_user_data (GTK_OBJECT (mbox), gdisp);
+  warning_buf =
+    g_strdup_printf (_("Changes were made to %s.\nClose anyway?"), image_name);
 
-  vbox = gtk_vbox_new (FALSE, 1);
-  gtk_container_border_width (GTK_CONTAINER (vbox), 1);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (mbox)->vbox), vbox, TRUE, TRUE, 0);
-  gtk_widget_show (vbox);
+  warning_dialog = mbox =
+    gimp_query_boolean_box (image_name,
+			    gimp_standard_help_func,
+			    "dialogs/really_close.html",
+			    TRUE,
+			    warning_buf,
+			    _("Close"), _("Cancel"),
+			    NULL, NULL,
+			    gdisplay_close_warning_callback,
+			    gdisp);
 
-  warning_buf = (char *) g_malloc (strlen (image_name) + 50);
-  sprintf (warning_buf, "Changes made to %s.  Close anyway?", image_name);
-  label = gtk_label_new (warning_buf);
-  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, FALSE, 0);
-  gtk_widget_show (label);
   g_free (warning_buf);
 
-  mbox_action_items[0].user_data = mbox;
-  mbox_action_items[1].user_data = mbox;
-  build_action_area (GTK_DIALOG (mbox), mbox_action_items, 2, 0);
+  gtk_signal_connect (GTK_OBJECT (mbox), "destroy",
+		      GTK_SIGNAL_FUNC (gtk_widget_destroyed),
+		      &warning_dialog);
 
   gtk_widget_show (mbox);
 }
