@@ -17,32 +17,41 @@
  */
 #include <stdlib.h>
 #include "appenv.h"
-#include "actionarea.h"
 #include "drawable.h"
+#include "image_new.h"
 #include "floating_sel.h"
 #include "gdisplay.h"
 #include "gimage.h"
 #include "gimage_mask.h"
-#include "general.h"
+#include "gimpui.h"
 #include "global_edit.h"
-#include "interface.h"
 #include "layer.h"
 #include "paint_funcs.h"
 #include "tools.h"
 #include "undo.h"
 
+#include "config.h"
+#include "libgimp/gimpintl.h"
+
 #include "tile_manager_pvt.h"
 #include "drawable_pvt.h"
 
+typedef enum
+{
+  PASTE,
+  PASTE_INTO,
+  PASTE_AS_NEW
+} PasteAction;  
 
 /*  The named paste dialog  */
 typedef struct _PasteNamedDlg PasteNamedDlg;
+
 struct _PasteNamedDlg
 {
   GtkWidget   *shell;
   GtkWidget   *list;
-  int          paste_into;
   GDisplay    *gdisp;
+  PasteAction  action;
 };
 
 /*  The named buffer structure...  */
@@ -50,42 +59,42 @@ typedef struct _named_buffer NamedBuffer;
 
 struct _named_buffer
 {
-  TileManager * buf;
-  char *        name;
+  TileManager *buf;
+  gchar       *name;
 };
 
 
 /*  The named buffer list  */
-GSList * named_buffers = NULL;
+static GSList *named_buffers = NULL;
 
 /*  The global edit buffer  */
-TileManager * global_buf = NULL;
+TileManager   *global_buf = NULL;
 
 
 /*  Crop the buffer to the size of pixels with non-zero transparency */
 
 TileManager *
 crop_buffer (TileManager *tiles,
-	     int          border)
+	     gboolean     border)
 {
-  PixelRegion PR;
+  PixelRegion  PR;
   TileManager *new_tiles;
-  int bytes, alpha;
-  unsigned char * data;
-  int empty;
-  int x1, y1, x2, y2;
-  int x, y;
-  int ex, ey;
-  int found;
-  void * pr;
-  unsigned char black[MAX_CHANNELS] = { 0, 0, 0, 0 };
+  gint         bytes, alpha;
+  guchar      *data;
+  gint         empty;
+  gint         x1, y1, x2, y2;
+  gint         x, y;
+  gint         ex, ey;
+  gint         found;
+  void        *pr;
+  guchar       black[MAX_CHANNELS] = { 0, 0, 0, 0 };
 
-  bytes = tiles->levels[0].bpp;
+  bytes = tiles->bpp;
   alpha = bytes - 1;
 
   /*  go through and calculate the bounds  */
-  x1 = tiles->levels[0].width;
-  y1 = tiles->levels[0].height;
+  x1 = tiles->width;
+  y1 = tiles->height;
   x2 = 0;
   y2 = 0;
 
@@ -118,17 +127,17 @@ crop_buffer (TileManager *tiles,
 	}
     }
 
-  x2 = BOUNDS (x2 + 1, 0, tiles->levels[0].width);
-  y2 = BOUNDS (y2 + 1, 0, tiles->levels[0].height);
+  x2 = CLAMP (x2 + 1, 0, tiles->width);
+  y2 = CLAMP (y2 + 1, 0, tiles->height);
 
-  empty = (x1 == tiles->levels[0].width && y1 == tiles->levels[0].height);
+  empty = (x1 == tiles->width && y1 == tiles->height);
 
   /*  If there are no visible pixels, return NULL */
   if (empty)
     new_tiles = NULL;
   /*  If no cropping, return original buffer  */
-  else if (x1 == 0 && y1 == 0 && x2 == tiles->levels[0].width &&
-	   y2 == tiles->levels[0].height && border == 0)
+  else if (x1 == 0 && y1 == 0 && x2 == tiles->width &&
+	   y2 == tiles->height && border == 0)
     new_tiles = tiles;
   /*  Otherwise, crop the original area  */
   else
@@ -166,12 +175,12 @@ crop_buffer (TileManager *tiles,
 }
 
 TileManager *
-edit_cut (GImage *gimage,
+edit_cut (GImage       *gimage,
 	  GimpDrawable *drawable)
 {
   TileManager *cut;
   TileManager *cropped_cut;
-  int empty;
+  gint         empty;
 
   if (!gimage || drawable == NULL)
     return NULL;
@@ -198,6 +207,10 @@ edit_cut (GImage *gimage,
   else
     cropped_cut = NULL;
 
+  if (cut)
+    image_new_reset_current_cut_buffer ();
+
+
   /*  end the group undo  */
   undo_push_group_end (gimage);
 
@@ -216,12 +229,12 @@ edit_cut (GImage *gimage,
 }
 
 TileManager *
-edit_copy (GImage *gimage,
+edit_copy (GImage       *gimage,
 	   GimpDrawable *drawable)
 {
-  TileManager * copy;
-  TileManager * cropped_copy;
-  int empty;
+  TileManager *copy;
+  TileManager *cropped_copy;
+  gint         empty;
 
   if (!gimage || drawable == NULL)
     return NULL;
@@ -245,6 +258,10 @@ edit_copy (GImage *gimage,
   else
     cropped_copy = NULL;
 
+  if(copy)
+    image_new_reset_current_cut_buffer();
+
+
   if (cropped_copy)
     {
       /*  Free the old global edit buffer  */
@@ -259,32 +276,54 @@ edit_copy (GImage *gimage,
     return NULL;
 }
 
-int
-edit_paste (GImage      *gimage,
+GimpLayer *
+edit_paste (GImage       *gimage,
 	    GimpDrawable *drawable,
-	    TileManager *paste,
-	    int          paste_into)
+	    TileManager  *paste,
+	    gboolean      paste_into)
 {
-  Layer * float_layer;
-  int x1, y1, x2, y2;
-  int cx, cy;
+  Layer *layer;
+  gint   x1, y1, x2, y2;
+  gint   cx, cy;
 
-  /*  Make a new floating layer  */
-  float_layer = layer_from_tiles (gimage, drawable, paste, "Pasted Layer", OPAQUE_OPACITY, NORMAL);
+  /*  Make a new layer: iff drawable == NULL,
+   *  user is pasting into an empty display.
+   */
 
-  if (float_layer)
+  if (drawable != NULL)
+    layer = layer_new_from_tiles (gimage,
+				  gimp_drawable_type_with_alpha (drawable),
+				  paste, 
+				  _("Pasted Layer"),
+				  OPAQUE_OPACITY, NORMAL_MODE);
+  else
+    layer = layer_new_from_tiles (gimage,
+				  gimp_image_base_type_with_alpha (gimage),
+				  paste, 
+				  _("Pasted Layer"),
+				  OPAQUE_OPACITY, NORMAL_MODE);
+ 
+  if (layer)
     {
       /*  Start a group undo  */
       undo_push_group_start (gimage, EDIT_PASTE_UNDO);
 
       /*  Set the offsets to the center of the image  */
-      drawable_offsets ( (drawable), &cx, &cy);
-      drawable_mask_bounds ( (drawable), &x1, &y1, &x2, &y2);
-      cx += (x1 + x2) >> 1;
-      cy += (y1 + y2) >> 1;
+      if (drawable != NULL)
+	{
+	  drawable_offsets (drawable, &cx, &cy);
+	  drawable_mask_bounds (drawable, &x1, &y1, &x2, &y2);
+	  cx += (x1 + x2) >> 1;
+	  cy += (y1 + y2) >> 1;
+	}
+      else
+	{
+	  cx = gimage->width >> 1;
+	  cy = gimage->height >> 1;
+	}
 
-      GIMP_DRAWABLE(float_layer)->offset_x = cx - (GIMP_DRAWABLE(float_layer)->width >> 1);
-      GIMP_DRAWABLE(float_layer)->offset_y = cy - (GIMP_DRAWABLE(float_layer)->height >> 1);
+      GIMP_DRAWABLE (layer)->offset_x = cx - (GIMP_DRAWABLE (layer)->width >> 1);
+      GIMP_DRAWABLE (layer)->offset_y = cy - (GIMP_DRAWABLE (layer)->height >> 1);
 
       /*  If there is a selection mask clear it--
        *  this might not always be desired, but in general,
@@ -293,26 +332,74 @@ edit_paste (GImage      *gimage,
       if (! gimage_mask_is_empty (gimage) && !paste_into)
 	channel_clear (gimage_get_mask (gimage));
 
-      /*  add a new floating selection  */
-      floating_sel_attach (float_layer, drawable);
-
-      /*  end the group undo  */
+      /*  if there's a drawable, add a new floating selection  */
+      if (drawable != NULL)
+	{
+	  floating_sel_attach (layer, drawable);
+	}
+      else
+	{
+	  gimp_drawable_set_gimage (GIMP_DRAWABLE (layer), gimage);
+	  gimage_add_layer (gimage, layer, 0);
+	}
+      
+     /*  end the group undo  */
       undo_push_group_end (gimage);
 
-      return GIMP_DRAWABLE(float_layer)->ID;
+      return layer;
     }
-  else
-    return 0;
+
+  return NULL;
 }
 
-int
-edit_clear (GImage *gimage,
+gboolean
+edit_paste_as_new (GImage      *invoke,
+		   TileManager *paste)
+{
+  GImage   *gimage;
+  Layer    *layer;
+  GDisplay *gdisp;
+
+  if (!global_buf)
+    return FALSE;
+
+  /*  create a new image  (always of type RGB)  */
+  gimage = gimage_new (paste->width, paste->height, RGB);
+  gimage_disable_undo (gimage);
+  gimp_image_set_resolution (gimage, invoke->xresolution, invoke->yresolution);
+  gimp_image_set_unit (gimage, invoke->unit);
+  
+  layer = layer_new_from_tiles (gimage,
+				gimp_image_base_type_with_alpha (gimage),
+				paste, 
+				_("Pasted Layer"),
+				OPAQUE_OPACITY, NORMAL_MODE);
+
+  if (layer)
+    {
+      /*  add the new layer to the image  */
+      gimp_drawable_set_gimage (GIMP_DRAWABLE (layer), gimage);
+      gimage_add_layer (gimage, layer, 0);
+
+      gimage_enable_undo (gimage);
+
+      gdisp = gdisplay_new (gimage, 0x0101);
+      gimp_context_set_display (gimp_context_get_user (), gdisp);
+
+      return TRUE;
+    }
+
+  return FALSE;			       
+}
+
+gboolean
+edit_clear (GImage       *gimage,
 	    GimpDrawable *drawable)
 {
   TileManager *buf_tiles;
-  PixelRegion bufPR;
-  int x1, y1, x2, y2;
-  unsigned char col[MAX_CHANNELS];
+  PixelRegion  bufPR;
+  gint         x1, y1, x2, y2;
+  guchar       col[MAX_CHANNELS];
 
   if (!gimage || drawable == NULL)
     return FALSE;
@@ -324,7 +411,7 @@ edit_clear (GImage *gimage,
   drawable_mask_bounds (drawable, &x1, &y1, &x2, &y2);
 
   if (!(x2 - x1) || !(y2 - y1))
-    return FALSE;
+    return TRUE;  /*  nothing to do, but the clear succeded  */
 
   buf_tiles = tile_manager_new ((x2 - x1), (y2 - y1), drawable_bytes (drawable));
   pixel_region_init (&bufPR, buf_tiles, 0, 0, (x2 - x1), (y2 - y1), TRUE);
@@ -343,14 +430,14 @@ edit_clear (GImage *gimage,
   return TRUE;
 }
 
-int
-edit_fill (GImage *gimage,
+gboolean
+edit_fill (GImage       *gimage,
 	   GimpDrawable *drawable)
 {
   TileManager *buf_tiles;
-  PixelRegion bufPR;
-  int x1, y1, x2, y2;
-  unsigned char col[MAX_CHANNELS];
+  PixelRegion  bufPR;
+  gint         x1, y1, x2, y2;
+  guchar       col[MAX_CHANNELS];
 
   if (!gimage || drawable == NULL)
     return FALSE;
@@ -362,8 +449,8 @@ edit_fill (GImage *gimage,
   drawable_mask_bounds (drawable, &x1, &y1, &x2, &y2);
 
   if (!(x2 - x1) || !(y2 - y1))
-    return FALSE;
-
+    return TRUE;  /*  nothing to do, but the fill succeded  */
+ 
   buf_tiles = tile_manager_new ((x2 - x1), (y2 - y1), drawable_bytes (drawable));
   pixel_region_init (&bufPR, buf_tiles, 0, 0, (x2 - x1), (y2 - y1), TRUE);
   color_region (&bufPR, col);
@@ -381,60 +468,62 @@ edit_fill (GImage *gimage,
   return TRUE;
 }
 
-int
-global_edit_cut (void *gdisp_ptr)
+gboolean
+global_edit_cut (GDisplay *gdisp)
 {
-  GDisplay *gdisp;
-
   /*  stop any active tool  */
-  gdisp = (GDisplay *) gdisp_ptr;
-  active_tool_control (HALT, gdisp_ptr);
+  active_tool_control (HALT, gdisp);
 
   if (!edit_cut (gdisp->gimage, gimage_active_drawable (gdisp->gimage)))
     return FALSE;
-  else
-    {
-      /*  flush the display  */
-      gdisplays_flush ();
-      return TRUE;
-    }
+
+  /*  flush the display  */
+  gdisplays_flush ();
+
+  return TRUE;
 }
 
-int
-global_edit_copy (void *gdisp_ptr)
+gboolean
+global_edit_copy (GDisplay *gdisp)
 {
-  GDisplay *gdisp;
-
-  gdisp = (GDisplay *) gdisp_ptr;
-
   if (!edit_copy (gdisp->gimage, gimage_active_drawable (gdisp->gimage)))
     return FALSE;
-  else
-    return TRUE;
+
+  return TRUE;
 }
 
-int
-global_edit_paste (void *gdisp_ptr,
-		   int   paste_into)
+gboolean
+global_edit_paste (GDisplay *gdisp,
+		   gboolean  paste_into)
 {
-  GDisplay *gdisp;
+  /*  stop any active tool  */
+  active_tool_control (HALT, gdisp);
+
+  if (!edit_paste (gdisp->gimage, gimage_active_drawable (gdisp->gimage), 
+		   global_buf, paste_into))
+    return FALSE;
+
+  /*  flush the display  */
+  gdisplays_update_title (gdisp->gimage);
+  gdisplays_flush ();
+
+  return TRUE;
+}
+
+gboolean
+global_edit_paste_as_new (GDisplay *gdisp)
+{
+  if (!global_buf)
+    return FALSE;
 
   /*  stop any active tool  */
-  gdisp = (GDisplay *) gdisp_ptr;
-  active_tool_control (HALT, gdisp_ptr);
+  active_tool_control (HALT, gdisp);
 
-  if (!edit_paste (gdisp->gimage, gimage_active_drawable (gdisp->gimage), global_buf, paste_into))
-    return FALSE;
-  else
-    {
-      /*  flush the display  */
-      gdisplays_flush ();
-      return TRUE;
-    }
+  return edit_paste_as_new (gdisp->gimage, global_buf);
 }
 
 void
-global_edit_free ()
+global_edit_free (void)
 {
   if (global_buf)
     tile_manager_destroy (global_buf);
@@ -448,52 +537,70 @@ global_edit_free ()
 static void
 set_list_of_named_buffers (GtkWidget *list_widget)
 {
-  GSList *list;
+  GSList      *list;
   NamedBuffer *nb;
-  GtkWidget *list_item;
+  GtkWidget   *list_item;
 
   gtk_list_clear_items (GTK_LIST (list_widget), 0, -1);
-  list = named_buffers;
 
-  while (list)
+  for (list = named_buffers; list; list = g_slist_next (list))
     {
       nb = (NamedBuffer *) list->data;
-      list = g_slist_next (list);
 
       list_item = gtk_list_item_new_with_label (nb->name);
       gtk_container_add (GTK_CONTAINER (list_widget), list_item);
-      gtk_widget_show (list_item);
       gtk_object_set_user_data (GTK_OBJECT (list_item), (gpointer) nb);
+      gtk_widget_show (list_item);
     }
 }
 
 static void
-named_buffer_paste_foreach (GtkWidget *w,
-			    gpointer   client_data)
+named_buffer_paste_foreach (GtkWidget *widget,
+			    gpointer   data)
 {
   PasteNamedDlg *pn_dlg;
-  NamedBuffer *nb;
+  NamedBuffer   *nb;
 
-  if (w->state == GTK_STATE_SELECTED)
+  if (widget->state == GTK_STATE_SELECTED)
     {
-      pn_dlg = (PasteNamedDlg *) client_data;
-      nb = (NamedBuffer *) gtk_object_get_user_data (GTK_OBJECT (w));
-      edit_paste (pn_dlg->gdisp->gimage,
-		  gimage_active_drawable (pn_dlg->gdisp->gimage),
-		  nb->buf, pn_dlg->paste_into);
+      pn_dlg = (PasteNamedDlg *) data;
+      nb     = (NamedBuffer *) gtk_object_get_user_data (GTK_OBJECT (widget));
+
+      switch (pn_dlg->action)
+	{
+	case PASTE:
+	  edit_paste (pn_dlg->gdisp->gimage,
+		      gimage_active_drawable (pn_dlg->gdisp->gimage),
+		      nb->buf, FALSE);
+	  break;
+
+	case PASTE_INTO:
+	  edit_paste (pn_dlg->gdisp->gimage,
+		      gimage_active_drawable (pn_dlg->gdisp->gimage),
+		      nb->buf, TRUE);
+	  break;
+
+	case PASTE_AS_NEW:
+	  edit_paste_as_new (pn_dlg->gdisp->gimage, nb->buf);
+	  break;
+
+	default:
+	  break;
+	}
     }
 }
 
 static void
-named_buffer_paste_callback (GtkWidget *w,
-			     gpointer   client_data)
+named_buffer_paste_callback (GtkWidget *widget,
+			     gpointer   data)
 {
   PasteNamedDlg *pn_dlg;
 
-  pn_dlg = (PasteNamedDlg *) client_data;
+  pn_dlg = (PasteNamedDlg *) data;
 
-  gtk_container_foreach ((GtkContainer*) pn_dlg->list,
-			 named_buffer_paste_foreach, client_data);
+  pn_dlg->action = PASTE_INTO;
+  gtk_container_foreach ((GtkContainer *) pn_dlg->list,
+			 named_buffer_paste_foreach, data);
 
   /*  Destroy the box  */
   gtk_widget_destroy (pn_dlg->shell);
@@ -505,16 +612,59 @@ named_buffer_paste_callback (GtkWidget *w,
 }
 
 static void
-named_buffer_delete_foreach (GtkWidget *w,
-			     gpointer   client_data)
+named_buffer_paste_into_callback (GtkWidget *widget,
+				  gpointer   data)
 {
   PasteNamedDlg *pn_dlg;
-  NamedBuffer * nb;
 
-  if (w->state == GTK_STATE_SELECTED)
+  pn_dlg = (PasteNamedDlg *) data;
+
+  pn_dlg->action = PASTE_INTO;
+  gtk_container_foreach ((GtkContainer *) pn_dlg->list,
+			 named_buffer_paste_foreach, data);
+
+  /*  Destroy the box  */
+  gtk_widget_destroy (pn_dlg->shell);
+
+  g_free (pn_dlg);
+      
+  /*  flush the display  */
+  gdisplays_flush ();
+}
+
+static void
+named_buffer_paste_as_new_callback (GtkWidget *widget,
+				    gpointer   data)
+{
+  PasteNamedDlg *pn_dlg;
+
+  pn_dlg = (PasteNamedDlg *) data;
+
+  pn_dlg->action = PASTE_AS_NEW;
+  gtk_container_foreach ((GtkContainer *) pn_dlg->list,
+			 named_buffer_paste_foreach, data);
+
+  /*  Destroy the box  */
+  gtk_widget_destroy (pn_dlg->shell);
+
+  g_free (pn_dlg);
+      
+  /*  flush the display  */
+  gdisplays_flush ();
+}
+
+static void
+named_buffer_delete_foreach (GtkWidget *widget,
+			     gpointer   data)
+{
+  PasteNamedDlg *pn_dlg;
+  NamedBuffer   *nb;
+
+  if (widget->state == GTK_STATE_SELECTED)
     {
-      pn_dlg = (PasteNamedDlg *) client_data;
-      nb = (NamedBuffer *) gtk_object_get_user_data (GTK_OBJECT (w));
+      pn_dlg = (PasteNamedDlg *) data;
+      nb     = (NamedBuffer *) gtk_object_get_user_data (GTK_OBJECT (widget));
+
       named_buffers = g_slist_remove (named_buffers, (void *) nb);
       g_free (nb->name);
       tile_manager_destroy (nb->buf);
@@ -523,24 +673,25 @@ named_buffer_delete_foreach (GtkWidget *w,
 }
 
 static void
-named_buffer_delete_callback (GtkWidget *w,
-			      gpointer   client_data)
+named_buffer_delete_callback (GtkWidget *widget,
+			      gpointer   data)
 {
   PasteNamedDlg *pn_dlg;
 
-  pn_dlg = (PasteNamedDlg *) client_data;
+  pn_dlg = (PasteNamedDlg *) data;
+
   gtk_container_foreach ((GtkContainer*) pn_dlg->list,
-			 named_buffer_delete_foreach, client_data);
+			 named_buffer_delete_foreach, data);
   set_list_of_named_buffers (pn_dlg->list);
 }
 
 static void
-named_buffer_cancel_callback (GtkWidget *w,
-			      gpointer   client_data)
+named_buffer_cancel_callback (GtkWidget *widget,
+			      gpointer   data)
 {
   PasteNamedDlg *pn_dlg;
 
-  pn_dlg = (PasteNamedDlg *) client_data;
+  pn_dlg = (PasteNamedDlg *) data;
 
   /*  Destroy the box  */
   gtk_widget_destroy (pn_dlg->shell);
@@ -548,63 +699,54 @@ named_buffer_cancel_callback (GtkWidget *w,
   g_free (pn_dlg);
 }
 
-static gint
-named_buffer_dialog_delete_callback (GtkWidget *w,
-				     GdkEvent  *e,
-				     gpointer   client_data)
-{
-  named_buffer_cancel_callback (w, client_data);
-
-  return TRUE;
-}
-
-static void
-named_buffer_paste_into_update (GtkWidget *w,
-				gpointer   client_data)
-{
-  PasteNamedDlg *pn_dlg;
-
-  pn_dlg = (PasteNamedDlg *) client_data;
-
-  if (GTK_TOGGLE_BUTTON (w)->active)
-    pn_dlg->paste_into = FALSE;
-  else
-    pn_dlg->paste_into = TRUE;
-}
-
 static void
 paste_named_buffer (GDisplay *gdisp)
 {
-  static ActionAreaItem action_items[3] =
-  {
-    { "Paste", named_buffer_paste_callback, NULL, NULL },
-    { "Delete", named_buffer_delete_callback, NULL, NULL },
-    { "Cancel", named_buffer_cancel_callback, NULL, NULL }
-  };
   PasteNamedDlg *pn_dlg;
   GtkWidget *vbox;
   GtkWidget *label;
-  GtkWidget *paste_into;
   GtkWidget *listbox;
+  GtkWidget *bbox;
+  GtkWidget *button;
+  gint i;
 
-  pn_dlg = (PasteNamedDlg *) g_malloc (sizeof (PasteNamedDlg));
+  static gchar *paste_action_labels[] =
+  {
+    N_("Paste"),
+    N_("Paste Into"),
+    N_("Paste as New"),
+  };
+
+  static GtkSignalFunc paste_action_functions[] =
+  {
+    named_buffer_paste_callback,
+    named_buffer_paste_into_callback,
+    named_buffer_paste_as_new_callback,
+  };
+
+  pn_dlg = g_new (PasteNamedDlg, 1);
   pn_dlg->gdisp = gdisp;
 
-  pn_dlg->shell = gtk_dialog_new ();
-  gtk_window_set_wmclass (GTK_WINDOW (pn_dlg->shell), "paste_named_buffer", "Gimp");
-  gtk_window_set_title (GTK_WINDOW (pn_dlg->shell), "Paste Named Buffer");
-  gtk_window_position (GTK_WINDOW (pn_dlg->shell), GTK_WIN_POS_MOUSE);
+  pn_dlg->shell =
+    gimp_dialog_new (_("Paste Named Buffer"), "paste_named_buffer",
+		     gimp_standard_help_func,
+		     "dialogs/paste_named.html",
+		     GTK_WIN_POS_MOUSE,
+		     FALSE, TRUE, FALSE,
 
-  gtk_signal_connect (GTK_OBJECT (pn_dlg->shell), "delete_event",
-		      GTK_SIGNAL_FUNC (named_buffer_dialog_delete_callback),
-		      pn_dlg);
+		     _("Delete"), named_buffer_delete_callback,
+		     pn_dlg, NULL, NULL, FALSE, FALSE,
+		     _("Cancel"), named_buffer_cancel_callback,
+		     pn_dlg, NULL, NULL, TRUE, TRUE,
 
+		     NULL);
+		     
   vbox = gtk_vbox_new (FALSE, 1);
-  gtk_container_border_width (GTK_CONTAINER (vbox), 1);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (pn_dlg->shell)->vbox), vbox, TRUE, TRUE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (vbox), 1);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (pn_dlg->shell)->vbox), vbox);
   gtk_widget_show (vbox);
 
-  label = gtk_label_new ("Select a buffer to paste:");
+  label = gtk_label_new (_("Select a buffer to paste:"));
   gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, FALSE, 0);
   gtk_widget_show (label);
 
@@ -618,28 +760,32 @@ paste_named_buffer (GDisplay *gdisp)
 
   pn_dlg->list = gtk_list_new ();
   gtk_list_set_selection_mode (GTK_LIST (pn_dlg->list), GTK_SELECTION_BROWSE);
-  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (listbox), pn_dlg->list);
+  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (listbox),
+					 pn_dlg->list);
   set_list_of_named_buffers (pn_dlg->list);
   gtk_widget_show (pn_dlg->list);
 
-  paste_into = gtk_check_button_new_with_label ("Replace Current Selection");
-  gtk_box_pack_start (GTK_BOX (vbox), paste_into, FALSE, FALSE, 0);
-  gtk_signal_connect (GTK_OBJECT (paste_into), "toggled",
-		      (GtkSignalFunc) named_buffer_paste_into_update,
-		      pn_dlg);
-  gtk_widget_show (paste_into);
-
-  action_items[0].user_data = pn_dlg;
-  action_items[1].user_data = pn_dlg;
-  action_items[2].user_data = pn_dlg;
-  build_action_area (GTK_DIALOG (pn_dlg->shell), action_items, 3, 0);
+  bbox = gtk_hbutton_box_new ();
+  gtk_container_set_border_width (GTK_CONTAINER (bbox), 6);
+  gtk_button_box_set_spacing (GTK_BUTTON_BOX (bbox), 2);
+  gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 0);
+  for (i = 0; i < 3; i++)
+    {
+      button = gtk_button_new_with_label (gettext (paste_action_labels[i]));
+      gtk_container_add (GTK_CONTAINER (bbox), button);
+      gtk_signal_connect (GTK_OBJECT (button), "clicked",
+			  (GtkSignalFunc) paste_action_functions[i],
+			  pn_dlg);
+      gtk_widget_show (button);
+    }
+  gtk_widget_show (bbox);
 
   gtk_widget_show (pn_dlg->shell);
 }
 
 static void
 new_named_buffer (TileManager *tiles,
-		  char        *name)
+		  gchar       *name)
 {
   PixelRegion srcPR, destPR;
   NamedBuffer *nb;
@@ -648,9 +794,9 @@ new_named_buffer (TileManager *tiles,
 
   nb = (NamedBuffer *) g_malloc (sizeof (NamedBuffer));
 
-  nb->buf = tile_manager_new (tiles->levels[0].width, tiles->levels[0].height, tiles->levels[0].bpp);
-  pixel_region_init (&srcPR, tiles, 0, 0, tiles->levels[0].width, tiles->levels[0].height, FALSE);
-  pixel_region_init (&destPR, nb->buf, 0, 0, tiles->levels[0].width, tiles->levels[0].height, TRUE);
+  nb->buf = tile_manager_new (tiles->width, tiles->height, tiles->bpp);
+  pixel_region_init (&srcPR, tiles, 0, 0, tiles->width, tiles->height, FALSE);
+  pixel_region_init (&destPR, nb->buf, 0, 0, tiles->width, tiles->height, TRUE);
   copy_region (&srcPR, &destPR);
 
   nb->name = g_strdup ((char *) name);
@@ -658,16 +804,14 @@ new_named_buffer (TileManager *tiles,
 }
 
 static void
-cut_named_buffer_callback (GtkWidget *w,
-			   gpointer   client_data,
-			   gpointer   call_data)
+cut_named_buffer_callback (GtkWidget *widget,
+			   gchar     *name,
+			   gpointer   data)
 {
   TileManager *new_tiles;
-  GDisplay *gdisp;
-  char *name;
+  GDisplay    *gdisp;
 
-  gdisp = (GDisplay *) client_data;
-  name = g_strdup ((char *) call_data);
+  gdisp = (GDisplay *) data;
   
   new_tiles = edit_cut (gdisp->gimage, gimage_active_drawable (gdisp->gimage));
   if (new_tiles) 
@@ -675,74 +819,81 @@ cut_named_buffer_callback (GtkWidget *w,
   gdisplays_flush ();
 }
 
-int
-named_edit_cut (void *gdisp_ptr)
+gboolean
+named_edit_cut (GDisplay *gdisp)
 {
-  GDisplay *gdisp;
+  GtkWidget *qbox;
 
   /*  stop any active tool  */
-  gdisp = (GDisplay *) gdisp_ptr;
-  active_tool_control (HALT, gdisp_ptr);
+  active_tool_control (HALT, gdisp);
+
+  qbox = gimp_query_string_box (_("Cut Named"),
+				gimp_standard_help_func,
+				"dialogs/cut_named.html",
+				_("Enter a name for this buffer"),
+				NULL,
+				GTK_OBJECT (gdisp->gimage), "destroy",
+				cut_named_buffer_callback, gdisp);
+  gtk_widget_show (qbox);
 
-  query_string_box ("Cut Named", "Enter a name for this buffer", NULL,
-		    cut_named_buffer_callback, gdisp);
   return TRUE;
 }
 
 static void
-copy_named_buffer_callback (GtkWidget *w,
-			    gpointer   client_data,
-			    gpointer   call_data)
+copy_named_buffer_callback (GtkWidget *widget,
+			    gchar     *name,
+			    gpointer   data)
 {
   TileManager *new_tiles;
-  GDisplay *gdisp;
-  char *name;
+  GDisplay    *gdisp;
 
-  gdisp = (GDisplay *) client_data;
-  name = g_strdup ((char *) call_data);
+  gdisp = (GDisplay *) data;
   
   new_tiles = edit_copy (gdisp->gimage, gimage_active_drawable (gdisp->gimage));
   if (new_tiles) 
     new_named_buffer (new_tiles, name);
 }
 
-int
-named_edit_copy (void *gdisp_ptr)
+gboolean
+named_edit_copy (GDisplay *gdisp)
 {
-  GDisplay *gdisp;
+  GtkWidget *qbox;
+
+  qbox = gimp_query_string_box (_("Copy Named"),
+				gimp_standard_help_func,
+				"dialogs/copy_named.html",
+				_("Enter a name for this buffer"),
+				NULL,
+				GTK_OBJECT (gdisp->gimage), "destroy",
+				copy_named_buffer_callback, gdisp);
+  gtk_widget_show (qbox);
 
-  gdisp = (GDisplay *) gdisp_ptr;
-  
-  query_string_box ("Copy Named", "Enter a name for this buffer", NULL,
-		    copy_named_buffer_callback, gdisp);
   return TRUE;
 }
 
-int
-named_edit_paste (void *gdisp_ptr)
+gboolean
+named_edit_paste (GDisplay *gdisp)
 {
-  paste_named_buffer ((GDisplay *) gdisp_ptr);
+  paste_named_buffer (gdisp);
 
-  gdisplays_flush();
+  gdisplays_flush ();
 
   return TRUE;
 }
 
 void
-named_buffers_free ()
+named_buffers_free (void)
 {
-  GSList *list;
-  NamedBuffer * nb;
+  GSList      *list;
+  NamedBuffer *nb;
 
-  list = named_buffers;
-
-  while (list)
+  for (list = named_buffers; list; list = g_slist_next (list))
     {
       nb = (NamedBuffer *) list->data;
+
       tile_manager_destroy (nb->buf);
       g_free (nb->name);
       g_free (nb);
-      list = g_slist_next (list);
     }
 
   g_slist_free (named_buffers);
