@@ -16,35 +16,35 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 #include <stdlib.h>
+
 #include "appenv.h"
-#include "errors.h"
 #include "gdisplay.h"
 #include "gdisplay_ops.h"
 #include "gimprc.h"
-#include "info_window.h"
+#include "nav_window.h"
 #include "scale.h"
 #include "tools.h"
 
 void
 bounds_checking (GDisplay *gdisp)
 {
-  int sx, sy;
+  gint sx, sy;
 
-  sx = SCALE(gdisp, gdisp->gimage->width);
-  sy = SCALE(gdisp, gdisp->gimage->height);
+  sx = SCALEX (gdisp, gdisp->gimage->width);
+  sy = SCALEY (gdisp, gdisp->gimage->height);
 
-  gdisp->offset_x = BOUNDS (gdisp->offset_x, 0,
-			    LOWPASS (sx - gdisp->disp_width));
+  gdisp->offset_x = CLAMP (gdisp->offset_x, 0,
+			   LOWPASS (sx - gdisp->disp_width));
 
-  gdisp->offset_y = BOUNDS (gdisp->offset_y, 0,
-			    LOWPASS (sy - gdisp->disp_height));
+  gdisp->offset_y = CLAMP (gdisp->offset_y, 0,
+			   LOWPASS (sy - gdisp->disp_height));
 }
 
 
 void
 resize_display (GDisplay *gdisp,
-		int       resize_window,
-		int       redisplay)
+		gint      resize_window,
+		gint      redisplay)
 {
   /* freeze the active tool */
   active_tool_control (PAUSE, (void *) gdisp);
@@ -59,6 +59,8 @@ resize_display (GDisplay *gdisp,
     {
       gdisplay_expose_full (gdisp);
       gdisplays_flush ();
+      /* title may have changed if it includes the zoom ratio */
+      gdisplay_update_title (gdisp);
     }
 
   /* re-enable the active tool */
@@ -87,17 +89,18 @@ shrink_wrap_display (GDisplay *gdisp)
 
 void
 change_scale (GDisplay *gdisp,
-	      int       dir)
+	      gint      dir)
 {
-  unsigned char scalesrc, scaledest;
-  double offset_x, offset_y;
-  long sx, sy;
+  guchar scalesrc, scaledest;
+  gdouble offset_x, offset_y;
+  glong sx, sy;
 
-  scalesrc = SCALESRC(gdisp);
-  scaledest = SCALEDEST(gdisp);
+  /* user zoom control, so resolution versions not needed -- austin */
+  scalesrc = SCALESRC (gdisp);
+  scaledest = SCALEDEST (gdisp);
 
-  offset_x = gdisp->offset_x + (gdisp->disp_width/2.0);
-  offset_y = gdisp->offset_y + (gdisp->disp_height/2.0);
+  offset_x = gdisp->offset_x + (gdisp->disp_width / 2.0);
+  offset_y = gdisp->offset_y + (gdisp->disp_height / 2.0);
 
   offset_x *= ((double) scalesrc / (double) scaledest);
   offset_y *= ((double) scalesrc / (double) scaledest);
@@ -110,16 +113,16 @@ change_scale (GDisplay *gdisp,
       else
 	if (scaledest < 0x10)
 	  scaledest++;
-
       break;
+
     case ZOOMOUT :
       if (scaledest > 1)
 	scaledest--;
       else
 	if (scalesrc < 0x10)
 	  scalesrc++;
-
       break;
+
     default :
       scalesrc = dir%100;
       if (scalesrc < 1)
@@ -152,34 +155,56 @@ change_scale (GDisplay *gdisp,
 
       /*  resize the image  */
       resize_display (gdisp, allow_resize_windows, TRUE);
-
     }
 }
 
 
+/* scale image coord to realworld units (cm, inches, pixels) */
+/* 27/Feb/1999 I tried inlining this, but the result was slightly
+ * slower (poorer cache locality, probably) -- austin */
+static gdouble
+img2real (GDisplay *gdisp,
+	  gboolean  xdir,
+	  gdouble   a)
+{
+  gdouble res;
+
+  if (gdisp->dot_for_dot)
+    return a;
+
+  if (xdir)
+    res = gdisp->gimage->xresolution;
+  else
+    res = gdisp->gimage->yresolution;
+
+  return a * gimp_unit_get_factor (gdisp->gimage->unit) / res;
+}
+
+
 void
 setup_scale (GDisplay *gdisp)
 {
   GtkRuler *hruler;
   GtkRuler *vruler;
   gfloat sx, sy;
-  gfloat step;
+  gfloat stepx, stepy;
 
-  sx = SCALE(gdisp, gdisp->gimage->width);
-  sy = SCALE(gdisp, gdisp->gimage->height);
-  step = SCALE(gdisp, 1);
+  sx = SCALEX (gdisp, gdisp->gimage->width);
+  sy = SCALEY (gdisp, gdisp->gimage->height);
+  stepx = SCALEFACTOR_X (gdisp);
+  stepy = SCALEFACTOR_Y (gdisp);
 
   gdisp->hsbdata->value = gdisp->offset_x;
   gdisp->hsbdata->upper = sx;
   gdisp->hsbdata->page_size = MIN (sx, gdisp->disp_width);
   gdisp->hsbdata->page_increment = (gdisp->disp_width / 2);
-  gdisp->hsbdata->step_increment = step;
+  gdisp->hsbdata->step_increment = stepx;
 
   gdisp->vsbdata->value = gdisp->offset_y;
   gdisp->vsbdata->upper = sy;
   gdisp->vsbdata->page_size = MIN (sy, gdisp->disp_height);
   gdisp->vsbdata->page_increment = (gdisp->disp_height / 2);
-  gdisp->vsbdata->step_increment = step;
+  gdisp->vsbdata->step_increment = stepy;
 
   gtk_signal_emit_by_name (GTK_OBJECT (gdisp->hsbdata), "changed");
   gtk_signal_emit_by_name (GTK_OBJECT (gdisp->vsbdata), "changed");
@@ -188,39 +213,51 @@ setup_scale (GDisplay *gdisp)
   vruler = GTK_RULER (gdisp->vrule);
 
   hruler->lower = 0;
-  hruler->upper = UNSCALE (gdisp, gdisp->disp_width);
-  hruler->max_size = MAXIMUM (gdisp->gimage->width, gdisp->gimage->height);
+  hruler->upper = img2real (gdisp, TRUE, FUNSCALEX (gdisp, gdisp->disp_width));
+  hruler->max_size = img2real (gdisp, TRUE, MAX (gdisp->gimage->width,
+						 gdisp->gimage->height));
 
   vruler->lower = 0;
-  vruler->upper = UNSCALE (gdisp, gdisp->disp_height);
-  vruler->max_size = MAXIMUM (gdisp->gimage->width, gdisp->gimage->height);
+  vruler->upper = img2real(gdisp, FALSE, FUNSCALEY(gdisp, gdisp->disp_height));
+  vruler->max_size = img2real (gdisp, FALSE, MAX (gdisp->gimage->width,
+						  gdisp->gimage->height));
 
   if (sx < gdisp->disp_width)
     {
       gdisp->disp_xoffset = (gdisp->disp_width - sx) / 2;
-      hruler->lower -= UNSCALE (gdisp, (double) gdisp->disp_xoffset);
-      hruler->upper -= UNSCALE (gdisp, (double) gdisp->disp_xoffset);
+      hruler->lower -= img2real(gdisp, TRUE,
+				FUNSCALEX (gdisp, (double) gdisp->disp_xoffset));
+      hruler->upper -= img2real(gdisp, TRUE,
+				FUNSCALEX (gdisp, (double) gdisp->disp_xoffset));
     }
   else
     {
       gdisp->disp_xoffset = 0;
-      hruler->lower += UNSCALE (gdisp, (double) gdisp->offset_x);
-      hruler->upper += UNSCALE (gdisp, (double) gdisp->offset_x);
+      hruler->lower += img2real (gdisp, TRUE,
+				 FUNSCALEX (gdisp, (double) gdisp->offset_x));
+      hruler->upper += img2real (gdisp, TRUE,
+				 FUNSCALEX (gdisp, (double) gdisp->offset_x));
     }
 
   if (sy < gdisp->disp_height)
     {
       gdisp->disp_yoffset = (gdisp->disp_height - sy) / 2;
-      vruler->lower -= UNSCALE (gdisp, (double) gdisp->disp_yoffset);
-      vruler->upper -= UNSCALE (gdisp, (double) gdisp->disp_yoffset);
+      vruler->lower -= img2real(gdisp, FALSE,
+				FUNSCALEY (gdisp, (double) gdisp->disp_yoffset));
+      vruler->upper -= img2real(gdisp, FALSE,
+				FUNSCALEY (gdisp, (double) gdisp->disp_yoffset));
     }
   else
     {
       gdisp->disp_yoffset = 0;
-      vruler->lower += UNSCALE (gdisp, (double) gdisp->offset_y);
-      vruler->upper += UNSCALE (gdisp, (double) gdisp->offset_y);
+      vruler->lower += img2real (gdisp, FALSE,
+				 FUNSCALEY (gdisp, (double) gdisp->offset_y));
+      vruler->upper += img2real (gdisp, FALSE,
+				 FUNSCALEY (gdisp, (double) gdisp->offset_y));
     }
 
-  gtk_widget_draw (GTK_WIDGET (hruler), NULL);
-  gtk_widget_draw (GTK_WIDGET (vruler), NULL);
+  gtk_widget_queue_draw (GTK_WIDGET (hruler));
+  gtk_widget_queue_draw (GTK_WIDGET (vruler));
+
+  nav_window_update_window_marker (gdisp->window_nav_dialog);
 }
