@@ -13,16 +13,25 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 #ifndef __SHEAR_TOOL_H__
 #define __SHEAR_TOOL_H__
 
 #include "tools.h"
+#include "transform_core.h"
 
-void *        shear_tool_transform (Tool *, XtPointer, int);
-Tool *        tools_new_shear_tool    ();
-void          tools_free_shear_tool   (Tool *);
+TileManager * shear_tool_transform (Tool           *tool,
+				    gpointer        gdisp_ptr,
+				    TransformState  state);
+TileManager * shear_tool_shear     (GimpImage      *gimage,
+				    GimpDrawable   *drawable,
+				    GDisplay       *gdisp,
+				    TileManager    *float_tiles,
+				    gboolean        interpolation,
+				    GimpMatrix      matrix);
 
+Tool * tools_new_shear_tool  (void);
+void   tools_free_shear_tool (Tool *matrix);
 
 #endif  /*  __SHEAR_TOOL_H__  */
