@@ -13,14 +13,29 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 #ifndef __TEXT_TOOL_H__
 #define __TEXT_TOOL_H__
 
 #include "tools.h"
+#include "layerF.h"
+#include "gimpimageF.h"
+#include "gimpdrawableF.h"
 
-Tool *   tools_new_text (void);
-void     tools_free_text (Tool *);
+typedef enum {
+  PIXELS,
+  POINTS
+} SizeType;
+
+#define SUPERSAMPLE  3
+
+/*  text functions  */
+Tool * tools_new_text  (void);
+void   tools_free_text (Tool *);
+
+int         text_get_extents (char *, char *, int *, int *, int *, int *);
+GimpLayer * text_render      (GimpImage *, GimpDrawable *, int, int, char *,
+			      char *, int, int);
 
 #endif /* __TEXT_TOOL_H__ */
