@@ -2,7 +2,7 @@
  * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
@@ -11,13 +11,14 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 #include "config.h"
 
+#include <locale.h>
 #include <errno.h>
 #include <signal.h>
 #include <stdarg.h>
@@ -25,9 +26,18 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
+#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
+#endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+
+#include "parasiteP.h"
+#include "gimpenv.h"
 
 #ifdef HAVE_IPC_H
 #include <sys/ipc.h>
@@ -41,6 +51,15 @@
 #include <sys/select.h>
 #endif
 
+#if defined(G_OS_WIN32) || defined(G_WITH_CYGWIN)
+#  define STRICT
+#  include <windows.h>
+#  undef RGB
+#endif
+#ifdef __EMX__
+#  include <fcntl.h>
+#endif
+
 #include "gimp.h"
 #include "gimpprotocol.h"
 #include "gimpwire.h"
@@ -50,30 +69,42 @@
 
 void gimp_extension_process (guint timeout);
 void gimp_extension_ack     (void);
+void gimp_read_expect_msg(WireMessage *msg, int type);
 
+
+#ifndef G_OS_WIN32
 static RETSIGTYPE gimp_signal        (int signum);
-static int        gimp_write         (int fd, guint8 *buf, gulong count);
-static int        gimp_flush         (int fd);
+#endif
+static int        gimp_write         (GIOChannel *channel , guint8 *buf, gulong count);
+static int        gimp_flush         (GIOChannel *channel );
 static void       gimp_loop          (void);
 static void       gimp_config        (GPConfig *config);
 static void       gimp_proc_run      (GPProcRun *proc_run);
 static void       gimp_temp_proc_run (GPProcRun *proc_run);
 static void       gimp_message_func  (char *str);
+static void       gimp_process_message(WireMessage *msg);
+static void       gimp_close         (void);
 
 
-int _readfd = 0;
-int _writefd = 0;
+GIOChannel *_readchannel = NULL;
+GIOChannel *_writechannel = NULL;
+
 int _shm_ID = -1;
 guchar *_shm_addr = NULL;
 
-const guint gimp_major_version = GIMP_MAJOR_VERSION;
-const guint gimp_minor_version = GIMP_MINOR_VERSION;
-const guint gimp_micro_version = GIMP_MICRO_VERSION;
+guint gimp_major_version = GIMP_MAJOR_VERSION;
+guint gimp_minor_version = GIMP_MINOR_VERSION;
+guint gimp_micro_version = GIMP_MICRO_VERSION;
+
+#if defined(G_OS_WIN32) || defined(G_WITH_CYGWIN)
+static HANDLE shm_handle;
+#endif
 
 static gdouble _gamma_val;
 static gint _install_cmap;
 static gint _use_xshm;
 static guchar _color_cube[4];
+static gint _gdisp_ID = -1;
 
 static char *progname = NULL;
 static guint8 write_buffer[WRITE_BUFFER_SIZE];
@@ -81,13 +112,73 @@ static guint write_buffer_index = 0;
 
 static GHashTable *temp_proc_ht = NULL;
 
+#ifdef G_OS_WIN32
+static GPlugInInfo *PLUG_IN_INFO_PTR;
+#define PLUG_IN_INFO (*PLUG_IN_INFO_PTR)
+void
+set_gimp_PLUG_IN_INFO_PTR(GPlugInInfo *p)
+{
+  PLUG_IN_INFO_PTR = p;
+}
+#else
+#ifndef __EMX__
 extern GPlugInInfo PLUG_IN_INFO;
+#else
+static GPlugInInfo PLUG_IN_INFO;
+void set_gimp_PLUG_IN_INFO(const GPlugInInfo *p)
+{
+  PLUG_IN_INFO = *p;
+}
+#endif
+#endif
 
 
 int
 gimp_main (int   argc,
 	   char *argv[])
 {
+#ifdef G_OS_WIN32
+  char *peer, *peer_fd;
+  guint32 thread;
+  int i, j, k;
+#endif
+
+  setlocale (LC_NUMERIC, "C");
+
+#ifdef G_OS_WIN32
+  /* Check for exe file name with spaces in the path having been split up
+   * by buggy NT C runtime, or something. I don't know why this happens
+   * on NT (including w2k), but not on w95/98.
+   */
+
+  for (i = 1; i < argc; i++)
+    {
+      k = strlen (argv[i]);
+      if (k > 10)
+	if (g_strcasecmp (argv[i] + k - 4, ".exe") == 0)
+	  {
+	    /* Found the end of the executable name, most probably.
+	     * Splice the parts of the name back together.
+	     */
+	    GString *s;
+
+	    s = g_string_new (argv[0]);
+	    for (j = 1; j <= i; j++)
+	      {
+		s = g_string_append_c (s, ' ');
+		s = g_string_append (s, argv[j]);
+	      }
+	    argv[0] = s->str;
+	    /* Move rest of argv down */
+	    for (j = 1; j < argc - i; j++)
+	      argv[j] = argv[j + i];
+	    argv[argc - i] = NULL;
+	    argc -= i;
+	    break;
+	  }
+    }    
+#endif
+
   if ((argc < 4) || (strcmp (argv[1], "-gimp") != 0))
     {
       g_print ("%s is a gimp plug-in and must be run by the gimp to be used\n", argv[0]);
@@ -96,17 +187,49 @@ gimp_main (int   argc,
 
   progname = argv[0];
 
+#ifndef G_OS_WIN32
+  /* No use catching these on Win32, the user won't get any meaningful
+   * stack trace from glib anyhow. It's better to let Windows inform
+   * about the program error, and offer debugging if the plug-in
+   * has been built with MSVC, and the user has MSVC installed.
+   */
+#ifdef SIGHUP
   signal (SIGHUP, gimp_signal);
+#endif
   signal (SIGINT, gimp_signal);
+#ifdef SIGQUIT
   signal (SIGQUIT, gimp_signal);
+#endif
+#ifdef SIGBUS
   signal (SIGBUS, gimp_signal);
+#endif
+#ifdef SIGSEGV
   signal (SIGSEGV, gimp_signal);
+#endif
+#ifdef SIGPIPE
   signal (SIGPIPE, gimp_signal);
+#endif
   signal (SIGTERM, gimp_signal);
+#ifdef SIGFPE
   signal (SIGFPE, gimp_signal);
+#endif
+#endif
 
-  _readfd = atoi (argv[2]);
-  _writefd = atoi (argv[3]);
+#ifndef G_OS_WIN32
+  _readchannel = g_io_channel_unix_new (atoi (argv[2]));
+  _writechannel = g_io_channel_unix_new (atoi (argv[3]));
+#ifdef __EMX__
+  setmode(g_io_channel_unix_get_fd(_readchannel), O_BINARY);
+  setmode(g_io_channel_unix_get_fd(_writechannel), O_BINARY);
+#endif
+#else
+  g_assert (PLUG_IN_INFO_PTR != NULL);
+  _readchannel = g_io_channel_win32_new_pipe (atoi (argv[2]));
+  peer = strchr (argv[3], ':') + 1;
+  peer_fd = strchr (peer, ':') + 1;
+  _writechannel = g_io_channel_win32_new_pipe_with_wakeups
+    (atoi (argv[3]), atoi (peer), atoi (peer_fd));
+#endif
 
   gp_init ();
   wire_set_writer (gimp_write);
@@ -116,11 +239,18 @@ gimp_main (int   argc,
     {
       if (PLUG_IN_INFO.query_proc)
 	(* PLUG_IN_INFO.query_proc) ();
-      gimp_quit ();
+      gimp_close ();
       return 0;
     }
 
-  g_set_message_handler (&gimp_message_func);
+#ifdef G_OS_WIN32
+  /* Tell the GIMP our thread id */
+  thread = GetCurrentThreadId ();
+  wire_write_int32 (_writechannel, &thread, 1);
+  wire_flush (_writechannel);
+#endif
+
+  g_set_message_handler ((GPrintFunc) gimp_message_func);
 
   temp_proc_ht = g_hash_table_new (&g_str_hash, &g_str_equal);
 
@@ -128,18 +258,28 @@ gimp_main (int   argc,
   return 0;
 }
 
-void
-gimp_quit ()
+static void
+gimp_close (void)
 {
   if (PLUG_IN_INFO.quit_proc)
     (* PLUG_IN_INFO.quit_proc) ();
 
+#if defined(G_OS_WIN32) || defined(G_WITH_CYGWIN)
+  CloseHandle (shm_handle);
+#else
 #ifdef HAVE_SHM_H
   if ((_shm_ID != -1) && _shm_addr)
     shmdt ((char*) _shm_addr);
+#endif
 #endif
 
-  gp_quit_write (_writefd);
+  gp_quit_write (_writechannel);
+}
+
+void
+gimp_quit (void)
+{
+  gimp_close ();
   exit (0);
 }
 
@@ -161,6 +301,28 @@ gimp_set_data (gchar *  id,
   gimp_destroy_params (return_vals, nreturn_vals);
 }
 
+guint32
+gimp_get_data_size (gchar *  id)
+{
+  GParam *return_vals;
+  int nreturn_vals;
+  guint32 length;
+
+  return_vals = gimp_run_procedure ("gimp_procedural_db_get_data_size",
+                                    &nreturn_vals,
+                                    PARAM_STRING, id,
+                                    PARAM_END);
+
+  if (return_vals[0].data.d_status == STATUS_SUCCESS)
+      length= return_vals[1].data.d_int32;
+  else
+      length= 0;
+
+  gimp_destroy_params (return_vals, nreturn_vals);
+  return length;
+}
+
+
 void
 gimp_get_data (gchar *  id,
 	       gpointer data)
@@ -196,6 +358,7 @@ gimp_progress_init (char *message)
   return_vals = gimp_run_procedure ("gimp_progress_init",
 				    &nreturn_vals,
 				    PARAM_STRING, message,
+				    PARAM_INT32, _gdisp_ID,
 				    PARAM_END);
 
   gimp_destroy_params (return_vals, nreturn_vals);
@@ -215,9 +378,15 @@ gimp_progress_update (gdouble percentage)
   gimp_destroy_params (return_vals, nreturn_vals);
 }
 
+gint32
+gimp_default_display (void)
+{
+  return _gdisp_ID;
+}
+
 
 void
-gimp_message (char *message)
+gimp_message (const gchar *message)
 {
   GParam *return_vals;
   int nreturn_vals;
@@ -385,7 +554,7 @@ gimp_query_images (int *nimages)
   int nreturn_vals;
   gint32 *images;
 
-  return_vals = gimp_run_procedure ("gimp_list_images",
+  return_vals = gimp_run_procedure ("gimp_image_list",
 				    &nreturn_vals,
 				    PARAM_END);
 
@@ -433,7 +602,7 @@ gimp_install_procedure (char     *name,
   proc_install.params = (GPParamDef*) params;
   proc_install.return_vals = (GPParamDef*) return_vals;
 
-  if (!gp_proc_install_write (_writefd, &proc_install))
+  if (!gp_proc_install_write (_writechannel, &proc_install))
     gimp_quit ();
 }
 
@@ -458,19 +627,26 @@ gimp_install_temp_proc (char     *name,
 			  nparams, nreturn_vals, params, return_vals);
 
   /*  Insert the temp proc run function into the hash table  */
-  g_hash_table_insert (temp_proc_ht, (gpointer) name, (gpointer) run_proc);
+  g_hash_table_insert (temp_proc_ht, g_strdup (name), (gpointer) run_proc);
 }
 
 void
 gimp_uninstall_temp_proc (char *name)
 {
   GPProcUninstall proc_uninstall;
-
+  gpointer hash_name;
+  gboolean found;
   proc_uninstall.name = name;
 
-  if (!gp_proc_uninstall_write (_writefd, &proc_uninstall))
+  if (!gp_proc_uninstall_write (_writechannel, &proc_uninstall))
     gimp_quit ();
-  g_hash_table_remove (temp_proc_ht, (gpointer) name);
+  
+  found = g_hash_table_lookup_extended (temp_proc_ht, name, &hash_name, NULL);
+  if (found)
+  {
+    g_free (hash_name);
+    g_hash_table_remove (temp_proc_ht, (gpointer) name);
+  }
 }
 
 void
@@ -596,6 +772,9 @@ gimp_run_procedure (char *name,
         case PARAM_COLOR:
           (void) va_arg (args, guchar*);
           break;
+        case PARAM_PARASITE:
+          (void) va_arg (args, Parasite*);
+          break;
         case PARAM_REGION:
           break;
 	case PARAM_END:
@@ -680,6 +859,22 @@ gimp_run_procedure (char *name,
         case PARAM_PATH:
 	  proc_run.params[i].data.d_path = va_arg (args, gint32);
           break;
+        case PARAM_PARASITE:
+	{
+	  Parasite *p = va_arg (args, Parasite*);
+	  if (p == NULL)
+	  {
+	    proc_run.params[i].data.d_parasite.name = NULL;
+	    proc_run.params[i].data.d_parasite.data = NULL;
+	  }
+	  else
+	  {
+	    proc_run.params[i].data.d_parasite.name    = p->name;
+	    proc_run.params[i].data.d_parasite.flags   = p->flags;
+	    proc_run.params[i].data.d_parasite.size    = p->size;
+	    proc_run.params[i].data.d_parasite.data    = p->data;
+	  }
+	} break;
         case PARAM_STATUS:
 	  proc_run.params[i].data.d_status = va_arg (args, gint32);
           break;
@@ -690,13 +885,10 @@ gimp_run_procedure (char *name,
 
   va_end (args);
 
-  if (!gp_proc_run_write (_writefd, &proc_run))
-    gimp_quit ();
-  if (!wire_read_msg (_readfd, &msg))
+  if (!gp_proc_run_write (_writechannel, &proc_run))
     gimp_quit ();
 
-  if (msg.type != GP_PROC_RETURN)
-    g_error ("unexpected message: %d\n", msg.type);
+  gimp_read_expect_msg(&msg,GP_PROC_RETURN);
 
   proc_return = msg.data;
   *nreturn_vals = proc_return->nparams;
@@ -721,6 +913,30 @@ gimp_run_procedure (char *name,
   return return_vals;
 }
 
+void
+gimp_read_expect_msg(WireMessage *msg, int type)
+{
+  while(1)
+    {
+      if (!wire_read_msg (_readchannel, msg))
+	gimp_quit ();
+      
+      if (msg->type != type)
+	{
+	  if(msg->type == GP_TEMP_PROC_RUN)
+	    {
+	      gimp_process_message(msg);
+	      continue;
+	    }
+	  else
+	    g_error ("unexpected message: %d\n", msg->type);
+	}
+      else
+	break;
+    }
+}
+
+
 GParam*
 gimp_run_procedure2 (char   *name,
 		     int    *nreturn_vals,
@@ -736,14 +952,11 @@ gimp_run_procedure2 (char   *name,
   proc_run.nparams = nparams;
   proc_run.params = (GPParam *) params;
 
-  if (!gp_proc_run_write (_writefd, &proc_run))
-    gimp_quit ();
-  if (!wire_read_msg (_readfd, &msg))
+  if (!gp_proc_run_write (_writechannel, &proc_run))
     gimp_quit ();
 
-  if (msg.type != GP_PROC_RETURN)
-    g_error ("unexpected message: %d\n", msg.type);
-
+  gimp_read_expect_msg(&msg,GP_PROC_RETURN);
+  
   proc_return = msg.data;
   *nreturn_vals = proc_return->nparams;
   return_vals = (GParam*) proc_return->params;
@@ -760,6 +973,7 @@ gimp_run_procedure2 (char   *name,
       break;
     }
 
+  g_free (proc_return->name);
   g_free (proc_return);
 
   return return_vals;
@@ -774,6 +988,19 @@ gimp_destroy_params (GParam *params,
   _gp_params_destroy ((GPParam*) params, nparams);
 }
 
+void
+gimp_destroy_paramdefs (GParamDef *paramdefs,
+			int        nparams)
+{
+  while (nparams--)
+    {
+      g_free (paramdefs[nparams].name);
+      g_free (paramdefs[nparams].description);
+    }
+  
+  g_free (paramdefs);
+}
+
 gdouble
 gimp_gamma ()
 {
@@ -798,25 +1025,58 @@ gimp_color_cube ()
   return _color_cube;
 }
 
-gchar*
-gimp_gtkrc ()
+static void
+gimp_process_message(WireMessage *msg)
 {
-  static char filename[MAXPATHLEN];
-  char *home_dir;
+  switch (msg->type)
+    {
+    case GP_QUIT:
+      gimp_quit ();
+      break;
+    case GP_CONFIG:
+      gimp_config (msg->data);
+      break;
+    case GP_TILE_REQ:
+    case GP_TILE_ACK:
+    case GP_TILE_DATA:
+      g_warning ("unexpected tile message received (should not happen)\n");
+      break;
+    case GP_PROC_RUN:
+      g_warning ("unexpected proc run message received (should not happen)\n");
+      break;
+    case GP_PROC_RETURN:
+      g_warning ("unexpected proc return message received (should not happen)\n");
+      break;
+    case GP_TEMP_PROC_RUN:
+      gimp_temp_proc_run (msg->data);
+      break;
+    case GP_TEMP_PROC_RETURN:
+      g_warning ("unexpected temp proc return message received (should not happen)\n");
+      break;
+    case GP_PROC_INSTALL:
+      g_warning ("unexpected proc install message received (should not happen)\n");
+      break;
+    }
+}
 
-  home_dir = getenv ("HOME");
-  if (!home_dir)
-    return NULL;
+static void 
+gimp_single_message()
+{
+  WireMessage msg;
 
-  sprintf (filename, "%s/%s/gtkrc", home_dir, GIMPDIR);
+  /* Run a temp function */
+  if (!wire_read_msg (_readchannel, &msg))
+    gimp_quit ();
 
-  return filename;
+  gimp_process_message(&msg);
+  
+  wire_destroy (&msg);
 }
 
 void
 gimp_extension_process (guint timeout)
 {
-  WireMessage msg;
+#ifndef G_OS_WIN32
   fd_set readfds;
   int select_val;
   struct timeval tv;
@@ -825,67 +1085,65 @@ gimp_extension_process (guint timeout)
   if (timeout)
     {
       tv.tv_sec = timeout / 1000;
-      tv.tv_usec = timeout % 1000;
+      tv.tv_usec = (timeout % 1000) * 1000;
       tvp = &tv;
     }
   else
     tvp = NULL;
 
   FD_ZERO (&readfds);
-  FD_SET (_readfd, &readfds);
+  FD_SET (g_io_channel_unix_get_fd (_readchannel), &readfds);
 
   if ((select_val = select (FD_SETSIZE, &readfds, NULL, NULL, tvp)) > 0)
     {
-      if (!wire_read_msg (_readfd, &msg))
-	gimp_quit ();
-
-      switch (msg.type)
-	{
-	case GP_QUIT:
-	  gimp_quit ();
-	  break;
-	case GP_CONFIG:
-	  gimp_config (msg.data);
-	  break;
-	case GP_TILE_REQ:
-	case GP_TILE_ACK:
-	case GP_TILE_DATA:
-	  g_warning ("unexpected tile message received (should not happen)\n");
-	  break;
-	case GP_PROC_RUN:
-	  g_warning ("unexpected proc run message received (should not happen)\n");
-	  break;
-	case GP_PROC_RETURN:
-	  g_warning ("unexpected proc return message received (should not happen)\n");
-	  break;
-	case GP_TEMP_PROC_RUN:
-	  gimp_temp_proc_run (msg.data);
-	  break;
-	case GP_TEMP_PROC_RETURN:
-	  g_warning ("unexpected temp proc return message received (should not happen)\n");
-	  break;
-	case GP_PROC_INSTALL:
-	  g_warning ("unexpected proc install message received (should not happen)\n");
-	  break;
-	}
-
-      wire_destroy (&msg);
+      gimp_single_message();
     }
   else if (select_val == -1)
     {
       perror ("gimp_process");
       gimp_quit ();
     }
+#else
+  /* ??? */
+  MSG msg;
+  UINT timer;
+  static UINT message = 0;
+
+  if (message == 0)
+    message = RegisterWindowMessage ("g-pipe-readable");
+  if (timeout > 0)
+    timer = SetTimer (NULL, 0, timeout, NULL);
+  WaitMessage ();
+  if (timeout > 0)
+    KillTimer (NULL, timer);
+
+  if (PeekMessage (&msg, (HWND) -1, message, message, PM_NOREMOVE))
+    gimp_single_message ();
+#endif
 }
 
 void
-gimp_extension_ack ()
+gimp_extension_ack (void)
 {
   /*  Send an extension initialization acknowledgement  */
-  if (! gp_extension_ack_write (_writefd))
+  if (! gp_extension_ack_write (_writechannel))
     gimp_quit ();
 }
 
+void
+gimp_run_temp (void)
+{
+  gimp_single_message();
+}
+
+void
+gimp_request_wakeups (void)
+{
+  if (!gp_request_wakeups_write (_writechannel))
+    gimp_quit ();
+}
+
+#ifndef G_OS_WIN32
 static RETSIGTYPE
 gimp_signal (int signum)
 {
@@ -893,16 +1151,24 @@ gimp_signal (int signum)
 
   if (caught_fatal_sig)
     kill (getpid (), signum);
+
   caught_fatal_sig = 1;
 
   fprintf (stderr, "\n%s: %s caught\n", progname, g_strsignal (signum));
 
   switch (signum)
     {
+#ifdef SIGBUS
     case SIGBUS:
+#endif
+#ifdef SIGSEGV
     case SIGSEGV:
+#endif
+#ifdef SIGFPE
     case SIGFPE:
-      g_debug (progname);
+#endif
+    case 123456:		/* Must have some case value... */
+      g_on_error_query (progname);
       break;
     default:
       break;
@@ -910,9 +1176,10 @@ gimp_signal (int signum)
 
   gimp_quit ();
 }
+#endif
 
 static int
-gimp_write (int fd, guint8 *buf, gulong count)
+gimp_write (GIOChannel *channel, guint8 *buf, gulong count)
 {
   gulong bytes;
 
@@ -923,7 +1190,7 @@ gimp_write (int fd, guint8 *buf, gulong count)
 	  bytes = WRITE_BUFFER_SIZE - write_buffer_index;
 	  memcpy (&write_buffer[write_buffer_index], buf, bytes);
 	  write_buffer_index += bytes;
-	  if (!wire_flush (fd))
+	  if (!wire_flush (channel))
 	    return FALSE;
 	}
       else
@@ -941,10 +1208,11 @@ gimp_write (int fd, guint8 *buf, gulong count)
 }
 
 static int
-gimp_flush (int fd)
+gimp_flush (GIOChannel *channel)
 {
-  int count;
-  int bytes;
+  GIOError error;
+  guint count;
+  guint bytes;
 
   if (write_buffer_index > 0)
     {
@@ -952,10 +1220,13 @@ gimp_flush (int fd)
       while (count != write_buffer_index)
         {
 	  do {
-	    bytes = write (fd, &write_buffer[count], (write_buffer_index - count));
-	  } while ((bytes == -1) && (errno == EAGAIN));
+	    bytes = 0;
+	    error = g_io_channel_write (channel, &write_buffer[count],
+					(write_buffer_index - count),
+					&bytes);
+	  } while (error == G_IO_ERROR_AGAIN);
 
-	  if (bytes == -1)
+	  if (error != G_IO_ERROR_NONE)
 	    return FALSE;
 
           count += bytes;
@@ -974,14 +1245,17 @@ gimp_loop ()
 
   while (1)
     {
-      if (!wire_read_msg (_readfd, &msg))
-	gimp_quit ();
+      if (!wire_read_msg (_readchannel, &msg))
+        {
+	  gimp_close ();
+          return;
+        }
 
       switch (msg.type)
 	{
 	case GP_QUIT:
-	  gimp_quit ();
-	  break;
+	  gimp_close ();
+	  return;
 	case GP_CONFIG:
 	  gimp_config (msg.data);
 	  break;
@@ -992,8 +1266,8 @@ gimp_loop ()
 	  break;
 	case GP_PROC_RUN:
 	  gimp_proc_run (msg.data);
-	  gimp_quit ();
-	  break;
+	  gimp_close ();
+          return;
 	case GP_PROC_RETURN:
 	  g_warning ("unexpected proc return message received (should not happen)\n");
 	  break;
@@ -1041,16 +1315,52 @@ gimp_config (GPConfig *config)
   _color_cube[1] = config->color_cube[1];
   _color_cube[2] = config->color_cube[2];
   _color_cube[3] = config->color_cube[3];
+  _gdisp_ID = config->gdisp_ID;
 
-#ifdef HAVE_SHM_H
   if (_shm_ID != -1)
     {
+#if defined(G_OS_WIN32) || defined(G_WITH_CYGWIN)
+      /*
+       * Use Win32 shared memory mechanisms for
+       * transfering tile data
+       */
+      char fileMapName[128];
+      int tileByteSize = _gimp_tile_width * _gimp_tile_height * 4;
+      
+      /* From the id, derive the file map name */
+      sprintf(fileMapName, "GIMP%d.SHM", _shm_ID);
+
+      /* Open the file mapping */
+      shm_handle = OpenFileMapping (FILE_MAP_ALL_ACCESS,
+				    0, fileMapName);
+      if (shm_handle)
+	{
+	  /* Map the shared memory into our address space for use */
+	  _shm_addr = (guchar *) MapViewOfFile (shm_handle,
+						FILE_MAP_ALL_ACCESS,
+						0, 0, tileByteSize);
+
+	  /* Verify that we mapped our view */
+	  if (!_shm_addr)
+	    {
+	      g_warning ("MapViewOfFile error: %d... disabling shared memory transport",
+			 GetLastError());
+	    }
+	}
+      else
+	{
+	  g_warning ("OpenFileMapping error: %d... disabling shared memory transport",
+		     GetLastError());
+	}
+#else
+#ifdef HAVE_SHM_H
       _shm_addr = (guchar*) shmat (_shm_ID, 0, 0);
 
       if (_shm_addr == (guchar*) -1)
 	g_error ("could not attach to gimp shared memory segment\n");
-    }
 #endif
+#endif
+    }
 }
 
 static void
@@ -1072,7 +1382,7 @@ gimp_proc_run (GPProcRun *proc_run)
       proc_return.nparams = nreturn_vals;
       proc_return.params = (GPParam*) return_vals;
 
-      if (!gp_proc_return_write (_writefd, &proc_return))
+      if (!gp_proc_return_write (_writechannel, &proc_return))
 	gimp_quit ();
     }
 }
@@ -1080,7 +1390,6 @@ gimp_proc_run (GPProcRun *proc_run)
 static void
 gimp_temp_proc_run (GPProcRun *proc_run)
 {
-  GPProcReturn proc_return;
   GParam *return_vals;
   int nreturn_vals;
   GRunProc run_proc;
@@ -1095,11 +1404,12 @@ gimp_temp_proc_run (GPProcRun *proc_run)
 		    &nreturn_vals,
 		    &return_vals);
 
-      proc_return.name = proc_run->name;
-      proc_return.nparams = nreturn_vals;
-      proc_return.params = (GPParam*) return_vals;
+      /* No longer a return message */
+/*       proc_return.name = proc_run->name; */
+/*       proc_return.nparams = nreturn_vals; */
+/*       proc_return.params = (GPParam*) return_vals; */
 
-      if (!gp_temp_proc_return_write (_writefd, &proc_return))
-	gimp_quit ();
+/*       if (!gp_temp_proc_return_write (_writechannel, &proc_return)) */
+/* 	gimp_quit (); */
     }
 }
