@@ -17,14 +17,16 @@
  */
 
 
-/*
+/* 
    dbbrowser_utils.c
-   0.08  26th sept 97  by Thomas NOEL <thomas@minet.net>
+   0.08  26th sept 97  by Thomas NOEL <thomas@minet.net> 
+
+   98/12/13  Sven Neumann <sven@gimp.org> : added help display
 */
 
-#include <stdlib.h>
-
+#include "config.h"
 #include "dbbrowser_utils.h"
+#include "libgimp/stdplugins-intl.h"
 
 GList *proc_table;
 
@@ -43,50 +45,50 @@ gimp_db_browser(void (* apply_callback) ( gchar     *selected_proc_name,
 					  GParamDef *selected_return_vals ) )
   /* create the dialog box */
   /* console_entry != NULL => called from the script-fu-console */
-
+ 
 {
-
+  
   dbbrowser_t* dbbrowser;
-
+  
   GtkWidget *button;
   GtkWidget *hbox,*searchhbox,*vbox;
   GtkWidget *label;
 
   dbbrowser = (gpointer)malloc(sizeof(dbbrowser_t));
-
+  
   dbbrowser->apply_callback = apply_callback;
-
+  
   /* the dialog box */
 
   dbbrowser->dlg = gtk_dialog_new ();
-
-  gtk_window_set_title (GTK_WINDOW (dbbrowser->dlg), "DB Browser (init)");
+  
+  gtk_window_set_title (GTK_WINDOW (dbbrowser->dlg), _("DB Browser (init)"));
   gtk_window_position (GTK_WINDOW (dbbrowser->dlg), GTK_WIN_POS_MOUSE);
   gtk_signal_connect (GTK_OBJECT (dbbrowser->dlg), "destroy",
                       (GtkSignalFunc) dialog_close_callback,
                       dbbrowser);
-
+  
   /* hbox : left=list ; right=description */
-
+  
   hbox = gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dbbrowser->dlg)->vbox),
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dbbrowser->dlg)->vbox), 
 		      hbox, TRUE, TRUE, 0);
   gtk_widget_show (hbox);
-
+  
   /* left = vbox : the list and the search entry */
-
+  
   vbox = gtk_vbox_new( FALSE, 0 );
-  gtk_container_border_width (GTK_CONTAINER (vbox), 3);
-  gtk_box_pack_start (GTK_BOX (hbox),
+  gtk_container_border_width (GTK_CONTAINER (vbox), 3); 
+  gtk_box_pack_start (GTK_BOX (hbox), 
 		      vbox, FALSE, TRUE, 0);
   gtk_widget_show(vbox);
-
+  
   /* list : list in a scrolled_win */
-
+  
   dbbrowser->clist = gtk_clist_new(1);
   dbbrowser->scrolled_win = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (dbbrowser->scrolled_win),
-                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_clist_set_selection_mode (GTK_CLIST (dbbrowser->clist),
 			        GTK_SELECTION_BROWSE);
 
@@ -106,14 +108,14 @@ gimp_db_browser(void (* apply_callback) ( gchar     *selected_proc_name,
 		      searchhbox, FALSE, TRUE, 0);
   gtk_widget_show(searchhbox);
 
-  label = gtk_label_new("Search :");
+  label = gtk_label_new (_("Search :"));
   gtk_misc_set_alignment( GTK_MISC(label), 0.0, 0.5);
-  gtk_box_pack_start (GTK_BOX (searchhbox),
+  gtk_box_pack_start (GTK_BOX (searchhbox), 
 		      label, TRUE, TRUE, 0);
   gtk_widget_show(label);
 
   dbbrowser->search_entry = gtk_entry_new();
-  gtk_box_pack_start (GTK_BOX (searchhbox),
+  gtk_box_pack_start (GTK_BOX (searchhbox), 
 		      dbbrowser->search_entry, TRUE, TRUE, 0);
   gtk_widget_show(dbbrowser->search_entry);
 
@@ -121,10 +123,10 @@ gimp_db_browser(void (* apply_callback) ( gchar     *selected_proc_name,
 
   dbbrowser->descr_scroll = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (dbbrowser->descr_scroll),
-				  GTK_POLICY_ALWAYS,
+				  GTK_POLICY_ALWAYS, 
 				  GTK_POLICY_ALWAYS
 				  );
-  gtk_box_pack_start (GTK_BOX (hbox),
+  gtk_box_pack_start (GTK_BOX (hbox), 
 		      dbbrowser->descr_scroll, TRUE, TRUE, 0);
   gtk_widget_set_usize (dbbrowser->descr_scroll, DBL_WIDTH - DBL_LIST_WIDTH, 0);
   gtk_widget_show (dbbrowser->descr_scroll);
@@ -133,44 +135,44 @@ gimp_db_browser(void (* apply_callback) ( gchar     *selected_proc_name,
 
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG(dbbrowser->dlg)->action_area), 0);
 
-  dbbrowser->name_button = gtk_button_new_with_label ("Search by name");
+  dbbrowser->name_button = gtk_button_new_with_label (_("Search by name"));
   GTK_WIDGET_SET_FLAGS (dbbrowser->name_button, GTK_CAN_DEFAULT);
   gtk_signal_connect (GTK_OBJECT (dbbrowser->name_button), "clicked",
                       (GtkSignalFunc) dialog_search_callback, dbbrowser);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dbbrowser->dlg)->action_area),
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dbbrowser->dlg)->action_area), 
 		    dbbrowser->name_button , TRUE, TRUE, 0);
   gtk_widget_show(dbbrowser->name_button);
 
-  dbbrowser->blurb_button = gtk_button_new_with_label ("Search by blurb");
+  dbbrowser->blurb_button = gtk_button_new_with_label (_("Search by blurb"));
   GTK_WIDGET_SET_FLAGS (dbbrowser->blurb_button, GTK_CAN_DEFAULT);
   gtk_signal_connect (GTK_OBJECT (dbbrowser->blurb_button), "clicked",
                       (GtkSignalFunc) dialog_search_callback, dbbrowser);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dbbrowser->dlg)->action_area),
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dbbrowser->dlg)->action_area), 
 		    dbbrowser->blurb_button , TRUE, TRUE, 0);
   gtk_widget_show(dbbrowser->blurb_button);
 
   if (apply_callback) {
-    button = gtk_button_new_with_label ("Apply");
+    button = gtk_button_new_with_label (_("Apply"));
     GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
     gtk_signal_connect (GTK_OBJECT (button), "clicked",
 			(GtkSignalFunc) dialog_apply_callback, dbbrowser );
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dbbrowser->dlg)->action_area),
+    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dbbrowser->dlg)->action_area), 
 			button, TRUE, TRUE, 0);
     gtk_widget_show (button);
   }
 
-  button = gtk_button_new_with_label ("Close");
+  button = gtk_button_new_with_label (_("Close"));
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      (GtkSignalFunc) dialog_close_callback, dbbrowser);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dbbrowser->dlg)->action_area),
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dbbrowser->dlg)->action_area), 
 		      button, TRUE, TRUE, 0);
   gtk_widget_show (button);
 
 
   /* now build the list */
 
-  gtk_widget_show (dbbrowser->clist);
+  gtk_widget_show (dbbrowser->clist); 
   gtk_widget_show (dbbrowser->dlg);
 
   /* initialize the "return" value (for "apply") */
@@ -198,8 +200,8 @@ gimp_db_browser(void (* apply_callback) ( gchar     *selected_proc_name,
 
 static gint
 procedure_select_callback (GtkWidget *widget,
-			   gint row,
-			   gint column,
+			   gint row, 
+			   gint column, 
 			   GdkEventButton * bevent,
 			   gpointer data)
 {
@@ -215,21 +217,25 @@ procedure_select_callback (GtkWidget *widget,
   return FALSE;
 }
 
-static void
-dialog_select (dbbrowser_t *dbbrowser,
+static void 
+dialog_select (dbbrowser_t *dbbrowser, 
 	       gchar       *proc_name)
   /* update the description box (right) */
 {
-  GtkWidget* label, *old_table;
+  GtkWidget *label;
+  GtkWidget *old_table;
+  GtkWidget *help;
+  GtkWidget *text = NULL;
+  GtkWidget *vscrollbar;
   gint i,row=0;
-
-  if (dbbrowser->selected_proc_name)
+  
+  if (dbbrowser->selected_proc_name) 
     g_free(dbbrowser->selected_proc_name);
   dbbrowser->selected_proc_name = g_strdup(proc_name);
 
   if (dbbrowser->selected_scheme_proc_name)
     g_free(dbbrowser->selected_scheme_proc_name);
-  dbbrowser->selected_scheme_proc_name =
+  dbbrowser->selected_scheme_proc_name = 
     g_strdup(proc_name);
   convert_string(dbbrowser->selected_scheme_proc_name);
 
@@ -241,22 +247,22 @@ dialog_select (dbbrowser_t *dbbrowser,
   if (dbbrowser->selected_params) g_free(dbbrowser->selected_params);
   if (dbbrowser->selected_return_vals) g_free(dbbrowser->selected_return_vals);
 
-  gimp_query_procedure (proc_name,
-			&(dbbrowser->selected_proc_blurb),
-			&(dbbrowser->selected_proc_help),
+  gimp_query_procedure (proc_name, 
+			&(dbbrowser->selected_proc_blurb), 
+			&(dbbrowser->selected_proc_help), 
 			&(dbbrowser->selected_proc_author),
-			&(dbbrowser->selected_proc_copyright),
-			&(dbbrowser->selected_proc_date),
-			&(dbbrowser->selected_proc_type),
+			&(dbbrowser->selected_proc_copyright), 
+			&(dbbrowser->selected_proc_date), 
+			&(dbbrowser->selected_proc_type), 
 			&(dbbrowser->selected_nparams),
-			&(dbbrowser->selected_nreturn_vals),
-			&(dbbrowser->selected_params),
+			&(dbbrowser->selected_nreturn_vals), 
+			&(dbbrowser->selected_params), 
 			&(dbbrowser->selected_return_vals));
 
   /* save the "old" table */
   old_table = dbbrowser->descr_table;
 
-  dbbrowser->descr_table = gtk_table_new(
+  dbbrowser->descr_table = gtk_table_new( 
        10 + dbbrowser->selected_nparams + dbbrowser->selected_nreturn_vals ,
        5 , FALSE );
 
@@ -264,8 +270,8 @@ dialog_select (dbbrowser_t *dbbrowser,
 
   /* show the name */
 
-  label = gtk_label_new("Name :");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+  label = gtk_label_new(_("Name :"));
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5); 
   gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
 		    0, 1, row, row+1, GTK_FILL, GTK_FILL, 3, 6);
   gtk_widget_show(label);
@@ -280,8 +286,8 @@ dialog_select (dbbrowser_t *dbbrowser,
 
   /* show the description */
 
-  label = gtk_label_new("Blurb :");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+  label = gtk_label_new(_("Blurb :"));
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5); 
   gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
 		    0, 1, row, row+1, GTK_FILL, GTK_FILL, 3, 0);
   gtk_widget_show(label);
@@ -300,19 +306,19 @@ dialog_select (dbbrowser_t *dbbrowser,
   row++;
 
   /* in parameters */
-  if (dbbrowser->selected_nparams)
+  if (dbbrowser->selected_nparams) 
     {
-      label = gtk_label_new("In :");
-      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+      label = gtk_label_new(_("In :"));
+      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5); 
       gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
-	   0, 1, row, row+(dbbrowser->selected_nparams),
+	   0, 1, row, row+(dbbrowser->selected_nparams), 
 	   GTK_FILL, GTK_FILL, 3, 0);
       gtk_widget_show(label);
-      for (i=0;i<(dbbrowser->selected_nparams);i++)
+      for (i=0;i<(dbbrowser->selected_nparams);i++) 
 	{
 	  /* name */
 	  label = gtk_label_new((dbbrowser->selected_params[i]).name);
-	  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5); 
 	  gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
 			    1, 2, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
 	  gtk_widget_show(label);
@@ -330,12 +336,12 @@ dialog_select (dbbrowser_t *dbbrowser,
 	  gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
 			    3, 4, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
 	  gtk_widget_show(label);
-
+	  
 	  row++;
 	}
     }
-
-  if ((dbbrowser->selected_nparams) &&
+  
+  if ((dbbrowser->selected_nparams) && 
       (dbbrowser->selected_nreturn_vals)) {
     label = gtk_hseparator_new(); /* ok, not really a label ... :) */
     gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
@@ -349,18 +355,18 @@ dialog_select (dbbrowser_t *dbbrowser,
   if (dbbrowser->selected_nreturn_vals)
     {
       label = gtk_label_new("Out :");
-      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5); 
       gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
-			0, 1, row, row+(dbbrowser->selected_nreturn_vals),
+			0, 1, row, row+(dbbrowser->selected_nreturn_vals), 
 			GTK_FILL, GTK_FILL, 3, 0);
       gtk_widget_show(label);
-      for (i=0;i<(dbbrowser->selected_nreturn_vals);i++)
+      for (i=0;i<(dbbrowser->selected_nreturn_vals);i++) 
 	{
 	  /* name */
 	  label = gtk_label_new((dbbrowser->selected_return_vals[i]).name);
-	  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5); 
 	  gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
-			    1, 2, row, row+1,
+			    1, 2, row, row+1, 
 			    GTK_FILL, GTK_FILL, 0, 0);
 	  gtk_widget_show(label);
 
@@ -368,15 +374,15 @@ dialog_select (dbbrowser_t *dbbrowser,
 	  label = gtk_label_new(GParamType2char((dbbrowser->selected_return_vals[i]).type));
 	  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	  gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
-			    2, 3, row, row+1,
+			    2, 3, row, row+1, 
 			    GTK_FILL, GTK_FILL, 0, 0);
 	  gtk_widget_show(label);
 
 	  /* description */
 	  label = gtk_label_new((dbbrowser->selected_return_vals[i]).description);
-	  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5); 
 	  gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
-			    3, 4, row, row+1,
+			    3, 4, row, row+1, 
 			    GTK_FILL, GTK_FILL, 0, 0);
 	  gtk_widget_show(label);
 	  row++;
@@ -384,9 +390,7 @@ dialog_select (dbbrowser_t *dbbrowser,
 	}
     }
 
-  /* show the author & the copyright */
-
-  if ((dbbrowser->selected_nparams) ||
+  if ((dbbrowser->selected_nparams) || 
       (dbbrowser->selected_nreturn_vals)) {
     label = gtk_hseparator_new(); /* ok, not really a label ... :) */
     gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
@@ -396,10 +400,51 @@ dialog_select (dbbrowser_t *dbbrowser,
     row++;
   }
 
-  label = gtk_label_new("Author :");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+  /* show the help */
+  if ((dbbrowser->selected_proc_help) && (strlen(dbbrowser->selected_proc_help) > 1))
+    {
+      label = gtk_label_new(_("Help :"));
+      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5); 
+      gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
+			0, 1, row, row+1, 
+			GTK_FILL, GTK_FILL, 3, 0);
+      gtk_widget_show(label);
+      
+      help = gtk_table_new (2, 2, FALSE);
+      gtk_table_set_row_spacing (GTK_TABLE (help), 0, 2);
+      gtk_table_set_col_spacing (GTK_TABLE (help), 0, 2);
+      gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), help,
+			1, 4, row, row+1, GTK_FILL, GTK_FILL, 3, 0);
+      gtk_widget_show (help);
+      row++;
+      
+      text = gtk_text_new (NULL, NULL);
+      gtk_text_set_editable (GTK_TEXT (text), FALSE);
+      gtk_text_set_word_wrap(GTK_TEXT(text), TRUE);
+      gtk_widget_set_usize (text, -1, 60);
+      gtk_table_attach (GTK_TABLE (help), text, 0, 1, 0, 1,
+			GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+			GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
+      gtk_widget_show (text);
+      
+      vscrollbar = gtk_vscrollbar_new (GTK_TEXT (text)->vadj);
+      gtk_table_attach (GTK_TABLE (help), vscrollbar, 1, 2, 0, 1,
+			GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
+      gtk_widget_show (vscrollbar);
+            
+      label = gtk_hseparator_new(); /* ok, not really a label ... :) */
+      gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
+			0, 4, row, row+1, GTK_FILL, GTK_FILL, 3, 6);
+      gtk_widget_show(label);
+      row++;
+    }
+
+  /* show the author & the copyright */
+
+  label = gtk_label_new(_("Author :"));
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5); 
   gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
-		    0, 1, row, row+1,
+		    0, 1, row, row+1, 
 		    GTK_FILL, GTK_FILL, 3, 0);
   gtk_widget_show(label);
 
@@ -410,11 +455,11 @@ dialog_select (dbbrowser_t *dbbrowser,
 		    GTK_FILL, GTK_FILL, 0, 0);
   gtk_widget_show(label);
   row++;
-
-  label = gtk_label_new("Date :");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+  
+  label = gtk_label_new(_("Date :"));
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5); 
   gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
-		    0, 1, row, row+1,
+		    0, 1, row, row+1, 
 		    GTK_FILL, GTK_FILL, 3, 0);
   gtk_widget_show(label);
 
@@ -426,10 +471,10 @@ dialog_select (dbbrowser_t *dbbrowser,
   gtk_widget_show(label);
   row++;
 
-  label = gtk_label_new("Copyright :");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+  label = gtk_label_new(_("Copyright :"));
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5); 
   gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
-		    0, 1, row, row+1,
+		    0, 1, row, row+1, 
 		    GTK_FILL, GTK_FILL, 3, 0);
   gtk_widget_show(label);
 
@@ -441,26 +486,26 @@ dialog_select (dbbrowser_t *dbbrowser,
   gtk_widget_show(label);
   row++;
 
-  /*
-  label = gtk_label_new("Help :");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-  gtk_table_attach (GTK_TABLE (dbbrowser->descr_table), label,
-		    0, 1, row, row+1,
-		    GTK_FILL, GTK_FILL, 3, 0);
-  gtk_widget_show(label);
+  if (old_table)
+    gtk_widget_destroy(old_table);
 
-  TODO: Add help */
-
-  if (old_table) gtk_widget_destroy(old_table);
-
-  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (dbbrowser->descr_scroll),
+  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (dbbrowser->descr_scroll), 
 					 dbbrowser->descr_table);
 
+  /* now after the table is added to a window add the text */
+  if (text != NULL)
+    {
+      gtk_widget_realize (text);
+      gtk_text_freeze (GTK_TEXT (text));
+      gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL,
+		       dbbrowser->selected_proc_help, -1);
+      gtk_text_thaw (GTK_TEXT (text));
+    }
   gtk_widget_show(dbbrowser->descr_table);
 }
 
 static void
-dialog_close_callback (GtkWidget *widget,
+dialog_close_callback (GtkWidget *widget, 
 		       gpointer   data)
      /* end of the dialog */
 {
@@ -477,8 +522,8 @@ dialog_close_callback (GtkWidget *widget,
   }
 }
 
-static void
-dialog_apply_callback (GtkWidget *widget,
+static void 
+dialog_apply_callback (GtkWidget *widget, 
 		       gpointer   data)
      /* end of the dialog */
 {
@@ -498,8 +543,8 @@ dialog_apply_callback (GtkWidget *widget,
 			       dbbrowser->selected_return_vals );
 }
 
-static void
-dialog_search_callback (GtkWidget *widget,
+static void 
+dialog_search_callback (GtkWidget *widget, 
 			gpointer   data)
      /* search in the whole db */
 {
@@ -515,10 +560,10 @@ dialog_search_callback (GtkWidget *widget,
 
   /* search */
 
-  if ( widget == (dbbrowser->name_button) )
+  if ( widget == (dbbrowser->name_button) ) 
     {
-      gtk_window_set_title (GTK_WINDOW (dbbrowser->dlg),
-			    "DB Browser (by name - please wait)");
+      gtk_window_set_title (GTK_WINDOW (dbbrowser->dlg), 
+			    _("DB Browser (by name - please wait)"));
 
       query = g_string_new ("");
       query_text = gtk_entry_get_text(GTK_ENTRY(dbbrowser->search_entry));
@@ -534,24 +579,24 @@ dialog_search_callback (GtkWidget *widget,
 	}
 
       gimp_query_database (query->str,
-			   ".*", ".*", ".*", ".*", ".*", ".*",
+			   ".*", ".*", ".*", ".*", ".*", ".*", 
 			   &num_procs, &proc_list);
 
       g_string_free (query, TRUE);
     }
   else if ( widget == (dbbrowser->blurb_button) )
     {
-      gtk_window_set_title (GTK_WINDOW (dbbrowser->dlg),
-			    "DB Browser (by blurb - please wait)");
-      gimp_query_database (".*",
+      gtk_window_set_title (GTK_WINDOW (dbbrowser->dlg), 
+			    _("DB Browser (by blurb - please wait)"));
+      gimp_query_database (".*", 
 			   gtk_entry_get_text( GTK_ENTRY(dbbrowser->search_entry) ),
 			   ".*", ".*", ".*", ".*", ".*",
 			   &num_procs, &proc_list);
     }
   else {
-      gtk_window_set_title (GTK_WINDOW (dbbrowser->dlg),
-			    "DB Browser (please wait)");
-      gimp_query_database (".*", ".*", ".*", ".*", ".*", ".*", ".*",
+      gtk_window_set_title (GTK_WINDOW (dbbrowser->dlg), 
+			    _("DB Browser (please wait)"));
+      gimp_query_database (".*", ".*", ".*", ".*", ".*", ".*", ".*", 
 			   &num_procs, &proc_list);
   }
 
@@ -567,7 +612,7 @@ dialog_search_callback (GtkWidget *widget,
     gtk_clist_insert (GTK_CLIST (GTK_CLIST(dbbrowser->clist)), j,
                       &label);
     func_name = g_strdup (proc_list[i]);
-
+    
     gtk_clist_set_row_data_full(GTK_CLIST(dbbrowser->clist), j,
                                 func_name, g_free);
   }
@@ -581,7 +626,7 @@ dialog_search_callback (GtkWidget *widget,
   if (num_procs != 0) {
     gchar *insert_name, *label_name;
     int i,j,savej;
-
+    
     for (i = 0; i < num_procs ; i++) {
 
       insert_name=g_strdup(proc_list[0]); savej=0;
@@ -592,7 +637,7 @@ dialog_search_callback (GtkWidget *widget,
 	  savej=j;
 	}
       }
-
+  
       proc_list[savej][0]='\255';
 
       label_name = g_strdup( insert_name );
@@ -605,22 +650,21 @@ dialog_search_callback (GtkWidget *widget,
     }
   }
   */
-
+  
   if ( dbbrowser->clist ) {
     ;
   }
-
+  
   g_free( proc_list );
 
-  gtk_window_set_title (GTK_WINDOW (dbbrowser->dlg),
-			"DB Browser");
+  gtk_window_set_title (GTK_WINDOW (dbbrowser->dlg), _("DB Browser"));
   gtk_clist_thaw(GTK_CLIST(dbbrowser->clist));
 
 }
 
 /* utils ... */
 
-static void
+static void 
 convert_string (char *str)
 {
   while (*str)
@@ -630,7 +674,7 @@ convert_string (char *str)
     }
 }
 
-static char*
+static char* 
 GParamType2char(GParamType t)
 {
   switch (t) {
@@ -654,6 +698,7 @@ GParamType2char(GParamType t)
   case PARAM_SELECTION: return "SELECTION";
   case PARAM_BOUNDARY: return "BOUNDARY";
   case PARAM_PATH: return "PATH";
+  case PARAM_PARASITE: return "PARASITE";
   case PARAM_STATUS: return "STATUS";
   case PARAM_END: return "END";
   default: return "UNKNOWN?";
