@@ -20,18 +20,20 @@
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <stdio.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <errno.h>
 #include <string.h>
 #include <time.h>
 #include "gtk/gtk.h"
 #include "libgimp/gimp.h"
-#include "libgimp/gimpmenu.h"
-#include "libgimp/gimpwire.h"
-
-#include "config.h"
+#include "libgimp/gimpui.h"
+#include "libgimp/stdplugins-intl.h"
 
 #include "megawidget.h"
 
@@ -42,23 +44,22 @@
 #define variation_same   (-2)
 
 /* Declare local functions. */
-static void query(void);
-static void run(char *name,
-		int nparams,
-		GParam * param,
-		int *nreturn_vals,
-		GParam ** return_vals);
-static gint dialog();
+static void query             (void);
+static void run               (char    *name,
+		               int      nparams,
+		               GParam  *param,
+		               int     *nreturn_vals,
+			       GParam **return_vals);
+static void doit              (GDrawable *drawable);
 
-static void doit(GDrawable * drawable);
-
-static void set_flame_preview();
-static void load_callback(GtkWidget * widget, gpointer data);
-static void store_callback(GtkWidget * widget, gpointer data);
-static void set_edit_preview();
-static void menu_cb(GtkWidget * widget, gpointer data);
-static void my_mw_update_cb(gpointer data);
-static void init_mutants();
+static gint dialog            (void);
+static void set_flame_preview (void);
+static void load_callback     (GtkWidget *widget, gpointer data);
+static void save_callback     (GtkWidget *widget, gpointer data);
+static void set_edit_preview  (void);
+static void menu_cb           (GtkWidget *widget, gpointer data);
+static void my_mw_update_cb   (gpointer data);
+static void init_mutants      (void);
 
 char buffer[10000];
 GtkWidget *cmap_preview;
@@ -66,7 +67,7 @@ GtkWidget *flame_preview;
 int preview_width, preview_height;
 GtkWidget *dlg;
 static GtkWidget *file_dlg = 0;
-static int load_store;
+static int load_save;
 
 GtkWidget *edit_dlg = 0;
 
@@ -109,7 +110,8 @@ frame_spec f = {0.0, &config.cp, 1, 0.0};
 MAIN()
 
 
-static void query()
+static void 
+query (void)
 {
   static GParamDef args[] =
   {
@@ -121,20 +123,23 @@ static void query()
   static int nargs = sizeof(args) / sizeof(args[0]);
   static int nreturn_vals = 0;
 
+  INIT_I18N();
   gimp_install_procedure("plug_in_flame",
-			 "cosmic recursive fractal flames",
-			 "use Smooth Palette to make colormaps",
+			 "Creates cosmic recursive fractal flames",
+			 "Creates cosmic recursive fractal flames",
 			 "Scott Draves",
 			 "Scott Draves",
 			 "1997",
-			 "<Image>/Filters/Render/Flame",
+			 N_("<Image>/Filters/Render/Nature/Flame..."),
 			 "RGB*",
 			 PROC_PLUG_IN,
 			 nargs, nreturn_vals,
 			 args, return_vals);
 }
 
-static void maybe_init_cp() {
+static void 
+maybe_init_cp (void) 
+{
   if (0 == config.cp.spatial_oversample) {
     config.randomize = 0;
     config.variation = variation_same;
@@ -159,24 +164,29 @@ static void maybe_init_cp() {
   }
 }
 
-static void run(char *name, int n_params, GParam * param, int *nreturn_vals,
-		GParam ** return_vals)
+static void 
+run (char    *name, 
+     int      n_params, 
+     GParam  *param, 
+     int     *nreturn_vals,
+     GParam **return_vals)
 {
   static GParam values[1];
-  GDrawable *drawable;
+  GDrawable *drawable = NULL;
   GRunModeType run_mode;
   GStatusType status = STATUS_SUCCESS;
 
   *nreturn_vals = 1;
   *return_vals = values;
 
-  SRAND_FUNC (time(0));
+  SRAND_FUNC (time(NULL));
 
   run_mode = param[0].data.d_int32;
   
   if (run_mode == RUN_NONINTERACTIVE) {
     status = STATUS_CALLING_ERROR;
   } else {
+    INIT_I18N_UI();
     gimp_get_data("plug_in_flame", &config);
     /* XXX i tried using the init routine, but it didn't work. */
     mw_update_cb = my_mw_update_cb;
@@ -196,8 +206,8 @@ static void run(char *name, int n_params, GParam * param, int *nreturn_vals,
 
   if (status == STATUS_SUCCESS) {
 
-    if (gimp_drawable_color(drawable->id)) {
-      gimp_progress_init("Drawing Flame...");
+    if (gimp_drawable_is_rgb(drawable->id)) {
+      gimp_progress_init( _("Drawing Flame..."));
       gimp_tile_cache_ntiles(2 * (drawable->width / gimp_tile_width() + 1));
 
       doit(drawable);
@@ -217,7 +227,8 @@ static void run(char *name, int n_params, GParam * param, int *nreturn_vals,
 }
 
 static void
-drawable_to_cmap(control_point *cp) {
+drawable_to_cmap (control_point *cp) 
+{
   int i, j;
   GPixelRgn pr;
   GDrawable *d;
@@ -236,11 +247,11 @@ drawable_to_cmap(control_point *cp) {
     for (i = 0; i < 256; i++)
       for (j = 0; j < 3; j++)
 	cp->cmap[i][j] = g[i*4 + j];
-    free(g);
+    g_free(g);
   } else {
     d = gimp_drawable_get(config.cmap_drawable);
-    indexed = gimp_drawable_indexed(config.cmap_drawable);
-    p = (guchar *) malloc(d->bpp);
+    indexed = gimp_drawable_is_indexed(config.cmap_drawable);
+    p = (guchar *) g_malloc(d->bpp);
     gimp_pixel_rgn_init(&pr, d, 0, 0,
 			d->width, d->height, FALSE, FALSE);
     for (i = 0; i < 256; i++) {
@@ -250,11 +261,12 @@ drawable_to_cmap(control_point *cp) {
 	cp->cmap[i][j] =
 	  (d->bpp >= 3) ? (p[j] / 255.0) : (p[0]/255.0);
     }
-    free(p);
+    g_free(p);
   }
 }
 
-static void doit(GDrawable * drawable)
+static void 
+doit (GDrawable * drawable)
 {
   gint width, height;
   guchar *tmp;
@@ -269,11 +281,7 @@ static void doit(GDrawable * drawable)
     return;
   }
 
-  tmp = (guchar *) malloc(width * height * 4);
-  if (tmp == NULL) {
-    fprintf(stderr, "cannot malloc %d bytes.\n", width * height * bytes);
-    return;
-  }
+  tmp = (guchar *) g_malloc(width * height * 4);
 
   /* render */
   config.cp.width = width;
@@ -293,11 +301,8 @@ static void doit(GDrawable * drawable)
   } else if (3 == bytes) {
     int i, j;
     GPixelRgn src_pr, dst_pr;
-    guchar *sl = (guchar *) malloc(3 * width);
-    if (sl == NULL) {
-      fprintf(stderr, "cannot malloc %d bytes.\n", width * 3);
-      return;
-    }
+    guchar *sl = (guchar *) g_malloc(3 * width);
+
     gimp_pixel_rgn_init(&src_pr, drawable,
 			0, 0, width, height, FALSE, FALSE);
     gimp_pixel_rgn_init(&dst_pr, drawable,
@@ -318,22 +323,26 @@ static void doit(GDrawable * drawable)
       }
       gimp_pixel_rgn_set_rect(&dst_pr, sl, 0, i, width, 1);
     }
-    free(sl);
+    g_free(sl);
   } else
     printf("oops\n");
-  free(tmp);
+  g_free(tmp);
   gimp_drawable_flush(drawable);
   gimp_drawable_merge_shadow(drawable->id, TRUE);
   gimp_drawable_update(drawable->id, 0, 0, width, height);
 }
 
 
-static void close_callback(GtkWidget * widget, gpointer data)
+static void 
+close_callback (GtkWidget *widget, 
+		gpointer   data)
 {
   gtk_main_quit();
 }
 
-static void ok_callback(GtkWidget * widget, gpointer data)
+static void 
+ok_callback (GtkWidget *widget, 
+	     gpointer   data)
 {
   run_flag = 1;
   gtk_widget_destroy(GTK_WIDGET(data));
@@ -341,12 +350,15 @@ static void ok_callback(GtkWidget * widget, gpointer data)
     gtk_widget_destroy(edit_dlg);
 }
 
-static void file_ok_callback(GtkWidget * widget, gpointer data) {
+static void 
+file_ok_callback (GtkWidget *widget, 
+		  gpointer   data) 
+{
   GtkFileSelection *fs;
   char* filename;
   fs = GTK_FILE_SELECTION (data);
   filename = gtk_file_selection_get_filename (fs);
-  if (load_store) {
+  if (load_save) {
     FILE *f = fopen(filename, "r");
     int i, c;
     char *ss;
@@ -383,13 +395,17 @@ static void file_ok_callback(GtkWidget * widget, gpointer data) {
   gtk_widget_hide (file_dlg);  
 }
 
-static void file_cancel_callback(GtkWidget * widget, gpointer data) {
+static void 
+file_cancel_callback (GtkWidget *widget, 
+		      gpointer data) 
+{
   gtk_widget_hide (file_dlg);
 }
 
 static void
-make_file_dlg() {
-  file_dlg = gtk_file_selection_new ("Load/Store Flame");
+make_file_dlg (void) 
+{
+  file_dlg = gtk_file_selection_new ( _("Load/Save Flame"));
   gtk_window_position (GTK_WINDOW (file_dlg), GTK_WIN_POS_MOUSE);
   gtk_signal_connect(GTK_OBJECT (file_dlg),
 		     "delete_event",
@@ -402,27 +418,41 @@ make_file_dlg() {
 		     "clicked", (GtkSignalFunc) file_ok_callback, file_dlg);
 }
 
-static void randomize_callback(GtkWidget * widget, gpointer data) {
+static void 
+randomize_callback (GtkWidget *widget, 
+		    gpointer data) 
+{
   random_control_point(&edit_cp, config.variation);
   init_mutants();
   set_edit_preview();
 }
 
-static void edit_close_callback(GtkWidget * widget, gpointer data) {
+static void 
+edit_close_callback (GtkWidget *widget, 
+		     gpointer   data) 
+{
   gtk_widget_hide(edit_dlg);
 }
   
-static void edit_ok_callback(GtkWidget * widget, gpointer data) {
+static void 
+edit_ok_callback (GtkWidget *widget, 
+		  gpointer   data) 
+{
   gtk_widget_hide(edit_dlg);
   config.cp = edit_cp;
   set_flame_preview();  
 }
 
-static void edit_cancel_callback(GtkWidget * widget, gpointer data) {
+static void 
+edit_cancel_callback (GtkWidget *widget, 
+		      gpointer   data) 
+{
   gtk_widget_hide(edit_dlg);
 }
 
-static void init_mutants() {
+static void 
+init_mutants (void) 
+{
   int i;
   for (i = 0; i < nmutants; i++) {
     mutants[i] = edit_cp;
@@ -432,7 +462,9 @@ static void init_mutants() {
   }
 }
 
-static void my_mw_update_cb(gpointer data) {
+static void 
+my_mw_update_cb (gpointer data) 
+{
   double *fd = (double *) data;
   if (fd == &pick_speed)
     set_edit_preview();
@@ -445,7 +477,9 @@ static void my_mw_update_cb(gpointer data) {
     set_flame_preview();
 }   
 
-static void set_edit_preview() {
+static void 
+set_edit_preview (void) 
+{
   int y, i, j;
   guchar *b;
   control_point pcp;
@@ -454,7 +488,7 @@ static void set_edit_preview() {
 
   if (NULL == edit_previews[0]) return;
 
-  b = malloc(nbytes);
+  b = g_malloc(nbytes);
   maybe_init_cp();
   drawable_to_cmap(&edit_cp);
   for (i = 0; i < 3; i++)
@@ -490,10 +524,13 @@ static void set_edit_preview() {
 			     0, y, edit_preview_size);
       gtk_widget_draw (edit_previews[mut], NULL);  
     }
-  free(b);
+  g_free(b);
 }
 
-static void preview_clicked(GtkWidget * widget, gpointer data) {
+static void 
+preview_clicked (GtkWidget *widget, 
+		 gpointer data) 
+{
   int mut = (int) data;
   if (mut == 4) {
     control_point t = edit_cp;
@@ -512,17 +549,21 @@ static void preview_clicked(GtkWidget * widget, gpointer data) {
 
 
 static void
-edit_callback(GtkWidget * widget, gpointer data) {
+edit_callback (GtkWidget *widget, 
+	       gpointer   data) 
+{
   edit_cp = config.cp;
-  if (0 == edit_dlg) {
+
+  if (edit_dlg == 0) {
     GtkWidget *table;
+    GtkWidget *hbbox;
     GtkWidget *button;
     GtkWidget *box, *frame, *vbox;
     int i, j;
 
-    edit_dlg = gtk_dialog_new();
+    edit_dlg = gtk_dialog_new ();
   
-    gtk_window_set_title(GTK_WINDOW(edit_dlg), "Edit Flame");
+    gtk_window_set_title(GTK_WINDOW(edit_dlg), _("Edit Flame"));
     gtk_window_position(GTK_WINDOW(edit_dlg), GTK_WIN_POS_MOUSE);
     gtk_signal_connect(GTK_OBJECT(edit_dlg), "destroy",
 		       (GtkSignalFunc) edit_close_callback, NULL);
@@ -530,33 +571,41 @@ edit_callback(GtkWidget * widget, gpointer data) {
     gtk_signal_connect(GTK_OBJECT(edit_dlg), "delete_event",
 		       (GtkSignalFunc) gtk_widget_hide_on_delete, NULL);
 
-    button = gtk_button_new_with_label("Ok");
-    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		       (GtkSignalFunc) edit_ok_callback, 0);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(edit_dlg)->action_area),
-		       button, TRUE, TRUE, 0);
-    gtk_widget_grab_default(button);
-    gtk_widget_show(button);
+    /*  Action area  */
+    gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (edit_dlg)->action_area), 2);
+    gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (edit_dlg)->action_area), FALSE);
+    hbbox = gtk_hbutton_box_new ();
+    gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbbox), 4);
+    gtk_box_pack_end (GTK_BOX (GTK_DIALOG (edit_dlg)->action_area), hbbox, FALSE, FALSE, 0);
+    gtk_widget_show (hbbox);
+    
+    button = gtk_button_new_with_label ( _("OK"));
+    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+    gtk_signal_connect (GTK_OBJECT (button), "clicked",
+			(GtkSignalFunc) edit_ok_callback, 0);
+    gtk_box_pack_start (GTK_BOX (hbbox), button, FALSE, FALSE, 0);
+    gtk_widget_grab_default (button);
+    gtk_widget_show (button);
+    
+    button = gtk_button_new_with_label ( _("Cancel"));
+    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+    gtk_signal_connect (GTK_OBJECT (button), "clicked",
+			(GtkSignalFunc) edit_cancel_callback, 0);
+    gtk_box_pack_start (GTK_BOX (hbbox), button, FALSE, FALSE, 0);
+    gtk_widget_show (button);
 
-    button = gtk_button_new_with_label("Cancel");
-    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-    gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		       (GtkSignalFunc) edit_cancel_callback, 0);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(edit_dlg)->action_area),
-		       button, TRUE, TRUE, 0);
-    gtk_widget_grab_default(button);
-    gtk_widget_show(button);
-
-    frame = gtk_frame_new("Directions");
+    frame = gtk_frame_new( _("Directions"));
     gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
-    gtk_container_border_width(GTK_CONTAINER(frame), 10);
+    gtk_container_border_width(GTK_CONTAINER(frame), 4);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(edit_dlg)->vbox),
 		       frame, TRUE, TRUE, 0);
     gtk_widget_show(frame);
 
     table = gtk_table_new(3, 3, FALSE);
     gtk_container_add(GTK_CONTAINER(frame), table);
+    gtk_container_border_width(GTK_CONTAINER(table), 4);
+    gtk_table_set_row_spacings(GTK_TABLE(table), 4);
+    gtk_table_set_col_spacings(GTK_TABLE(table), 4);
     gtk_widget_show(table);
 
     for (i = 0; i < 3; i++)
@@ -576,36 +625,36 @@ edit_callback(GtkWidget * widget, gpointer data) {
 	gtk_widget_show (button);
       }
 
-    frame = gtk_frame_new("Controls");
-    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
-    gtk_container_border_width(GTK_CONTAINER(frame), 10);
+    frame = gtk_frame_new( _("Controls"));
+    gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
+    gtk_container_border_width (GTK_CONTAINER(frame), 4);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(edit_dlg)->vbox),
 		       frame, TRUE, TRUE, 0);
     gtk_widget_show(frame);
 
-    vbox = gtk_vbox_new (FALSE, 5);
+    vbox = gtk_vbox_new (FALSE, 4);
     gtk_container_add (GTK_CONTAINER (frame), vbox);
     gtk_widget_show(vbox);
 
     table = gtk_table_new(2, 2, FALSE);
-    gtk_box_pack_start(GTK_BOX(vbox),
-		       table, TRUE, FALSE, 10);
+    gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
+    gtk_container_border_width(GTK_CONTAINER(table), 4);
+    gtk_table_set_col_spacings(GTK_TABLE(table), 8);
     gtk_widget_show(table);
     
-    mw_fscale_entry_new(table, "Speed", 0.05, 0.5, 0.01, 0.1,
+    mw_fscale_entry_new(table, _("Speed"), 0.05, 0.5, 0.01, 0.1,
 			0.0, 0, 1, 1, 2, &pick_speed);
 
-    box = gtk_hbox_new (TRUE, 5);
-    gtk_box_pack_start(GTK_BOX(vbox),
-		       box, TRUE, FALSE, 10);
+    box = gtk_hbox_new (FALSE, 4);
+    gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 4);
     gtk_widget_show(box);
 
-    button = gtk_button_new_with_label("Randomize");
+    button = gtk_button_new_with_label( _("Randomize"));
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
 			      (GtkSignalFunc) randomize_callback,
 			      (gpointer) 0);
-    gtk_box_pack_start(GTK_BOX(box), button, TRUE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
     gtk_widget_show(button);
 
     {
@@ -613,15 +662,15 @@ edit_callback(GtkWidget * widget, gpointer data) {
 	char *name;
 	int value;
       } menu_items[] = {
-	{"Same", variation_same },
-	{"Random", variation_random },
-	{"Linear", 0},
-	{"Sinusoidal", 1},
-	{"Spherical", 2},
-	{"Swirl", 3},
-	{"Horseshoe", 4},
-	{"Polar", 5},
-	{"Bent", 6},
+	{ N_("Same"), variation_same },
+	{ N_("Random"), variation_random },
+	{ N_("Linear"), 0},
+	{ N_("Sinusoidal"), 1},
+	{ N_("Spherical"), 2},
+	{ N_("Swirl"), 3},
+	{ N_("Horseshoe"), 4},
+	{ N_("Polar"), 5},
+	{ N_("Bent"), 6},
 	{ NULL, 0},
       };
       GtkWidget *hbox;
@@ -629,11 +678,11 @@ edit_callback(GtkWidget * widget, gpointer data) {
       GtkWidget *menu, *w;
       int i;
 
-      hbox = gtk_hbox_new (FALSE, 5);
-      gtk_box_pack_start(GTK_BOX(box), hbox, TRUE, FALSE, 10);
+      hbox = gtk_hbox_new (FALSE, 4);
+      gtk_box_pack_end (GTK_BOX(box), hbox, FALSE, FALSE, 4);
       gtk_widget_show(hbox);
 
-      w = gtk_label_new("Variation:");
+      w = gtk_label_new( _("Variation:"));
       gtk_misc_set_alignment(GTK_MISC(w), 0.0, 0.5);
       gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
       gtk_widget_show(w);
@@ -644,7 +693,7 @@ edit_callback(GtkWidget * widget, gpointer data) {
       menu = gtk_menu_new ();
       while (menu_items[i].name) {
 	GtkWidget *menu_item;
-	menu_item = gtk_menu_item_new_with_label(menu_items[i].name);
+	menu_item = gtk_menu_item_new_with_label( gettext(menu_items[i].name));
 	gtk_container_add (GTK_CONTAINER (menu), menu_item);
 	gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
 			    (GtkSignalFunc) menu_cb,
@@ -664,31 +713,40 @@ edit_callback(GtkWidget * widget, gpointer data) {
     gtk_widget_show(edit_dlg);
 }
 
-static void load_callback(GtkWidget * widget, gpointer data) {
+static void 
+load_callback (GtkWidget *widget, 
+	       gpointer   data) 
+{
   if (!file_dlg) {
     make_file_dlg();
   } else {
     if (GTK_WIDGET_VISIBLE(file_dlg))
       return;
   }
-  gtk_window_set_title(GTK_WINDOW (file_dlg), "Load Flame");
-  load_store = 1;
+  gtk_window_set_title(GTK_WINDOW (file_dlg), _("Load Flame"));
+  load_save = 1;
   gtk_widget_show (file_dlg);
 }
 
-static void store_callback(GtkWidget * widget, gpointer data) {
+static void 
+save_callback (GtkWidget *widget, 
+	       gpointer   data) 
+{
   if (!file_dlg) {
     make_file_dlg();
   } else {
     if (GTK_WIDGET_VISIBLE(file_dlg))
       return;
   }
-  gtk_window_set_title(GTK_WINDOW (file_dlg), "Store Flame");
-  load_store = 0;
+  gtk_window_set_title(GTK_WINDOW (file_dlg), _("Save Flame"));
+  load_save = 0;
   gtk_widget_show (file_dlg);
 }
 
-static void menu_cb(GtkWidget * widget, gpointer data) {
+static void 
+menu_cb (GtkWidget *widget, 
+	 gpointer  data) 
+{
   config.variation = (int) data;
   if (variation_same != config.variation)
     random_control_point(&edit_cp, config.variation);
@@ -696,7 +754,9 @@ static void menu_cb(GtkWidget * widget, gpointer data) {
   set_edit_preview();
 }
 
-static void set_flame_preview() {
+static void 
+set_flame_preview (void) 
+{
   int y;
   guchar *b;
   control_point pcp;
@@ -705,7 +765,7 @@ static void set_flame_preview() {
   if (NULL == flame_preview)
     return;
 
-  b = malloc(preview_width * preview_height * 3);
+  b = g_malloc(preview_width * preview_height * 3);
 
   maybe_init_cp();
   drawable_to_cmap(&config.cp);
@@ -724,12 +784,14 @@ static void set_flame_preview() {
   for (y = 0; y < preview_size; y++)
     gtk_preview_draw_row(GTK_PREVIEW (flame_preview),
 			 b+y*preview_width*3, 0, y, preview_width);
-  free(b);
+  g_free(b);
 
   gtk_widget_draw (flame_preview, NULL);
 }
 
-static void set_cmap_preview() {
+static void 
+set_cmap_preview (void) 
+{
   int i, x, y;
   guchar b[96];
 
@@ -754,14 +816,20 @@ static void set_cmap_preview() {
   gtk_widget_draw (cmap_preview, NULL);  
 }
 
-static void gradient_cb(GtkWidget * widget, gpointer data) {
+static void 
+gradient_cb (GtkWidget *widget, 
+	     gpointer   data) 
+{
   config.cmap_drawable = (int)data;
   set_cmap_preview();
   set_flame_preview();
   /*  set_edit_preview(); */
 }
 
-static void cmap_callback(gint32 id, gpointer data) {
+static void 
+cmap_callback (gint32   id, 
+	       gpointer data) 
+{
   config.cmap_drawable = id;
   set_cmap_preview();
   set_flame_preview();
@@ -770,13 +838,18 @@ static void cmap_callback(gint32 id, gpointer data) {
 
 
 static gint
-cmap_constrain (gint32 image_id, gint32 drawable_id, gpointer data) {
-  
-  return ! gimp_drawable_indexed (drawable_id);
+cmap_constrain (gint32   image_id, 
+		gint32   drawable_id, 
+		gpointer data) 
+{  
+  return ! gimp_drawable_is_indexed (drawable_id);
 }
 
 
-static gint dialog() {
+static gint 
+dialog (void) 
+{
+  GtkWidget *hbbox;
   GtkWidget *button;
   GtkWidget *table;
   GtkWidget *box;
@@ -805,62 +878,69 @@ static gint dialog() {
   gtk_widget_set_default_colormap (gtk_preview_get_cmap ());
 
   dlg = gtk_dialog_new();
-  gtk_window_set_title(GTK_WINDOW(dlg), "Flame");
+  gtk_window_set_title(GTK_WINDOW(dlg), _("Flame"));
   gtk_window_position(GTK_WINDOW(dlg), GTK_WIN_POS_MOUSE);
   gtk_signal_connect(GTK_OBJECT(dlg), "destroy",
 		     (GtkSignalFunc) close_callback, NULL);
 
-  button = gtk_button_new_with_label("Ok");
-  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		     (GtkSignalFunc) ok_callback,
-		     dlg);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->action_area), button, TRUE, TRUE, 0);
-  gtk_widget_grab_default(button);
-  gtk_widget_show(button);
+  /*  Action area  */
+  gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dlg)->action_area), 2);
+  gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (dlg)->action_area), FALSE);
+  hbbox = gtk_hbutton_box_new ();
+  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbbox), 4);
+  gtk_box_pack_end (GTK_BOX (GTK_DIALOG (dlg)->action_area), hbbox, FALSE, FALSE, 0);
+  gtk_widget_show (hbbox);
+ 
+  button = gtk_button_new_with_label ( _("OK"));
+  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		      (GtkSignalFunc) ok_callback,
+		      dlg);
+  gtk_box_pack_start (GTK_BOX (hbbox), button, FALSE, FALSE, 0);
+  gtk_widget_grab_default (button);
+  gtk_widget_show (button);
 
-  button = gtk_button_new_with_label("Cancel");
-  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
-			    (GtkSignalFunc) gtk_widget_destroy,
-			    GTK_OBJECT(dlg));
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->action_area), button, TRUE, TRUE, 0);
-  gtk_widget_show(button);
+  button = gtk_button_new_with_label ( _("Cancel"));
+  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+			     (GtkSignalFunc) gtk_widget_destroy,
+			     GTK_OBJECT (dlg));
+  gtk_box_pack_start (GTK_BOX (hbbox), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
 
-  frame = gtk_frame_new("Rendering");
+  frame = gtk_frame_new( _("Rendering"));
   gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
-  gtk_container_border_width(GTK_CONTAINER(frame), 10);
+  gtk_container_border_width(GTK_CONTAINER(frame), 4);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), frame, TRUE, TRUE, 0);
   gtk_widget_show(frame);
 
-  box = gtk_vbox_new (FALSE, 5);
+  box = gtk_vbox_new (FALSE, 4);
   gtk_container_add(GTK_CONTAINER(frame), box);
   gtk_widget_show(box);
 
   table = gtk_table_new(7, 2, FALSE);
   gtk_box_pack_start(GTK_BOX(box), table, FALSE, FALSE, 0);
 
-  gtk_container_border_width(GTK_CONTAINER(table), 10);
+  gtk_container_border_width(GTK_CONTAINER(table), 4);
   gtk_widget_show(table);
 
-  gtk_table_set_row_spacings(GTK_TABLE(table), 10);
-  gtk_table_set_col_spacings(GTK_TABLE(table), 10);
+  gtk_table_set_row_spacings(GTK_TABLE(table), 8);
+  gtk_table_set_col_spacings(GTK_TABLE(table), 8);
 
   row = 1;
 
   /* this zoom - gamma should redraw flame preview */
-  mw_fscale_entry_new(table, "Brightness", 0, 5, 1, 1, 0,
+  mw_fscale_entry_new(table, _("Brightness"), 0, 5, 1, 1, 0,
                       0, 1, row, row+1, &config.cp.brightness); row++;
-  mw_fscale_entry_new(table, "Contrast", 0, 5, 1, 1, 0,
+  mw_fscale_entry_new(table, _("Contrast"), 0, 5, 1, 1, 0,
                       0, 1, row, row+1, &config.cp.contrast); row++;
-  mw_fscale_entry_new(table, "Gamma", 1, 5, 1, 1, 0,
+  mw_fscale_entry_new(table, _("Gamma"), 1, 5, 1, 1, 0,
                       0, 1, row, row+1, &config.cp.gamma); row++;
-  mw_fscale_entry_new(table, "Sample Density", 0.1, 20, 1, 5, 0,
+  mw_fscale_entry_new(table, _("Sample Density"), 0.1, 20, 1, 5, 0,
                       0, 1, row, row+1, &config.cp.sample_density); row++;
-  mw_iscale_entry_new(table, "Spatial Oversample", 1, 4, 1, 1, 0,
+  mw_iscale_entry_new(table, _("Spatial Oversample"), 1, 4, 1, 1, 0,
                       0, 1, row, row+1, &config.cp.spatial_oversample); row++;
-
-  mw_fscale_entry_new(table, "Spatial Filter Radius", 0, 4, 1, 1, 0,
+  mw_fscale_entry_new(table, _("Spatial Filter Radius"), 0, 4, 1, 1, 0,
                       0, 1, row, row+1, &config.cp.spatial_filter_radius); row++;
 
 {
@@ -870,22 +950,22 @@ static gint dialog() {
     GtkWidget *option_menu = gtk_option_menu_new ();
     gint32 save_drawable = config.cmap_drawable;
 
-    hbox = gtk_hbox_new (FALSE, 5);
+    hbox = gtk_hbox_new (FALSE, 4);
     gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 10);
     gtk_widget_show(hbox);
 
-    w = gtk_label_new("Colormap:");
+    w = gtk_label_new( _("Colormap"));
     gtk_misc_set_alignment(GTK_MISC(w), 0.0, 0.5);
-    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 10);
+    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 4);
     gtk_widget_show(w);
 
-    gtk_box_pack_start(GTK_BOX(hbox), option_menu, TRUE, TRUE, 10);
+    gtk_box_pack_start(GTK_BOX(hbox), option_menu, TRUE, TRUE, 4);
     menu = gimp_drawable_menu_new(cmap_constrain, cmap_callback,
 				  0, config.cmap_drawable);
 
     config.cmap_drawable = save_drawable;
 #if 0
-    menuitem = gtk_menu_item_new_with_label("Black");
+    menuitem = gtk_menu_item_new_with_label( _("Black"));
     gtk_signal_connect(GTK_OBJECT (menuitem), "activate",
 		       (GtkSignalFunc) gradient_cb,
 		       (gpointer) black_drawable);
@@ -915,7 +995,7 @@ static gint dialog() {
     }
 
 
-    menuitem = gtk_menu_item_new_with_label("Custom Gradient");
+    menuitem = gtk_menu_item_new_with_label( _("Custom Gradient"));
     gtk_signal_connect(GTK_OBJECT (menuitem), "activate",
 		       (GtkSignalFunc) gradient_cb,
 		       (gpointer) gradient_drawable);
@@ -930,32 +1010,32 @@ static gint dialog() {
     cmap_preview = gtk_preview_new (GTK_PREVIEW_COLOR);
     gtk_preview_size (GTK_PREVIEW (cmap_preview), 32, 32);
 
-    gtk_box_pack_start(GTK_BOX(hbox), cmap_preview, TRUE, TRUE, 10);
+    gtk_box_pack_start(GTK_BOX(hbox), cmap_preview, TRUE, TRUE, 4);
     gtk_widget_show (cmap_preview);
     set_cmap_preview();
   }
 
-  frame = gtk_frame_new("Camera");
+  frame = gtk_frame_new( _("Camera"));
   gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
-  gtk_container_border_width(GTK_CONTAINER(frame), 10);
+  gtk_container_border_width(GTK_CONTAINER(frame), 4);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), frame, TRUE, TRUE, 0);
   gtk_widget_show(frame);
 
   table = gtk_table_new(4, 2, FALSE);
-  gtk_container_border_width(GTK_CONTAINER(table), 10);
+  gtk_container_border_width(GTK_CONTAINER(table), 4);
   gtk_container_add(GTK_CONTAINER(frame), table);
   gtk_widget_show(table);
 
-  gtk_table_set_row_spacings(GTK_TABLE(table), 10);
-  gtk_table_set_col_spacings(GTK_TABLE(table), 10);
+  gtk_table_set_row_spacings(GTK_TABLE(table), 8);
+  gtk_table_set_col_spacings(GTK_TABLE(table), 8);
 
   row = 1;
   
-  mw_fscale_entry_new(table, "Zoom", -4, 4, 1, 1, 0,
+  mw_fscale_entry_new(table, _("Zoom"), -4, 4, 1, 1, 0,
                       0, 1, row, row+1, &config.cp.zoom); row++;
-  mw_fscale_entry_new(table, "X", -2, 2, 0.5, 0.5, 0,
+  mw_fscale_entry_new(table, _("X"), -2, 2, 0.5, 0.5, 0,
                       0, 1, row, row+1, &config.cp.center[0]); row++;
-  mw_fscale_entry_new(table, "Y", -2, 2, 0.5, 0.5, 0,
+  mw_fscale_entry_new(table, _("Y"), -2, 2, 0.5, 0.5, 0,
                       0, 1, row, row+1, &config.cp.center[1]); row++;
   
 
@@ -972,14 +1052,14 @@ static gint dialog() {
   }
   gtk_preview_size (GTK_PREVIEW (flame_preview), preview_width, preview_height);
 
-  box = gtk_hbox_new (FALSE, 5);
+  box = gtk_hbox_new (FALSE, 4);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox),
 		     box, FALSE, FALSE, 0);
   gtk_widget_show(box);
   
-  frame = gtk_frame_new("Preview");
+  frame = gtk_frame_new( _("Preview"));
   gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
-  gtk_container_border_width(GTK_CONTAINER(frame), 10);
+  gtk_container_border_width(GTK_CONTAINER(frame), 4);
   gtk_container_add(GTK_CONTAINER(frame), flame_preview);
   gtk_box_pack_start(GTK_BOX(box), frame, TRUE, FALSE, 0);
   gtk_widget_show(frame);
@@ -988,35 +1068,40 @@ static gint dialog() {
   set_flame_preview();
 
   {
-    GtkWidget *vbox = gtk_vbox_new (TRUE, 5);
-    gtk_box_pack_start(GTK_BOX(box), vbox, TRUE, FALSE, 0);
-    gtk_widget_show(vbox);
+    GtkWidget *vbox;
+    GtkWidget *vbbox;
+
+    vbox = gtk_vbox_new (FALSE, 4);
+    gtk_box_pack_start (GTK_BOX (box), vbox, FALSE, FALSE, 4);
+    vbbox= gtk_vbutton_box_new ();
+    gtk_box_set_homogeneous (GTK_BOX (vbbox), FALSE);
+    gtk_button_box_set_spacing (GTK_BUTTON_BOX (vbbox), 4);
+    gtk_box_pack_start (GTK_BOX (vbox), vbbox, FALSE, FALSE, 0);
+    gtk_widget_show (vbbox);
+    gtk_widget_show (vbox);
   
-    button = gtk_button_new_with_label("Shape Edit");
+    button = gtk_button_new_with_label( _("Edit Flame"));
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
 			      (GtkSignalFunc) edit_callback,
-			      (gpointer) 0);
-  
-    gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, FALSE, 10);
+			      (gpointer) 0);  
+    gtk_box_pack_start(GTK_BOX(vbbox), button, FALSE, FALSE, 0);
     gtk_widget_show(button);
 
-    button = gtk_button_new_with_label("Load");
+    button = gtk_button_new_with_label( _("Load Flame"));
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
 			      (GtkSignalFunc) load_callback,
 			      (gpointer) 0);
-  
-    gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, FALSE, 10);
+    gtk_box_pack_start(GTK_BOX(vbbox), button, FALSE, FALSE, 0);
     gtk_widget_show(button);
 
-    button = gtk_button_new_with_label("Store");
+    button = gtk_button_new_with_label( _("Save Flame"));
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
-			      (GtkSignalFunc) store_callback,
-			      (gpointer) 0);
-  
-    gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, FALSE, 10);
+			      (GtkSignalFunc) save_callback,
+			      (gpointer) 0);  
+    gtk_box_pack_start(GTK_BOX(vbbox), button, FALSE, FALSE, 0);
     gtk_widget_show(button);
   }
 
