@@ -37,14 +37,22 @@
  *
  */
 
-#ifdef MAZE_DEBUG
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef MAZE_DEBUG
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#endif
 
 #include <time.h>  /* For random seeding */
+
+#include <gtk/gtk.h>
 #include "libgimp/gimp.h"
+#include "libgimp/stdplugins-intl.h"
+
 #include "maze.h"
 
 extern gint      maze_dialog (void);
@@ -139,18 +147,23 @@ query ()
   static GParamDef *return_vals = NULL;
   static int nargs = sizeof (args) / sizeof (args[0]);
   static int nreturn_vals = 0;
+  gchar *help;
 
+  INIT_I18N();
+
+  help = g_strdup_printf (_("Generates a maze using either the depth-first search method or Prim's algorithm.  Can make tileable mazes too.  See %s for more help."), MAZE_URL);
   gimp_install_procedure ("plug_in_maze",
 			  "Draws a maze.",
-			  "Generates a maze using either the depth-first search method or Prim's algorithm.  Can make tileable mazes too.  See " MAZE_URL " for more help.",
+			  help,
 			  "Kevin Turner <kevint@poboxes.com>",
 			  "Kevin Turner",
 			  "1997, 1998",
-			  "<Image>/Filters/Render/Maze",
+			  N_("<Image>/Filters/Render/Pattern/Maze..."),
 			  "RGB*, GRAY*, INDEXED*",
 			  PROC_PLUG_IN,
 			  nargs, nreturn_vals,
 			  args, return_vals);
+  g_free (help);
 }
 
 static void
@@ -174,6 +187,8 @@ run    (gchar    *name,
   *nreturn_vals = 1;
   *return_vals = values;
 
+  INIT_I18N_UI(); 
+
   values[0].type = PARAM_STATUS;
   values[0].data.d_status = status;
 
@@ -203,13 +218,13 @@ run    (gchar    *name,
 	}
       if (status == STATUS_SUCCESS)
 	{
-	  mvals.width = (gint16)    param[3].data.d_int32;
-	  mvals.height = (gint16)   param[4].data.d_int32;
-	  mvals.tile = (gint8)      param[5].data.d_int32;
-          mvals.algorithm = (gint8) param[6].data.d_int32;
+	  mvals.width = (gint16)    param[3].data.d_int16;
+	  mvals.height = (gint16)   param[4].data.d_int16;
+	  mvals.tile = (gint8)      param[5].data.d_int8;
+          mvals.algorithm = (gint8) param[6].data.d_int8;
 	  mvals.seed = (gint32)     param[7].data.d_int32;
-	  mvals.multiple = (gint16) param[8].data.d_int32;
-	  mvals.offset = (gint16)   param[9].data.d_int32;
+	  mvals.multiple = (gint16) param[8].data.d_int16;
+	  mvals.offset = (gint16)   param[9].data.d_int16;
 	}
       break;
     case RUN_WITH_LAST_VALS:
@@ -222,7 +237,7 @@ run    (gchar    *name,
   }
   
   /* color, gray, or indexed... hmm, miss anything?  ;)  */
-  if (gimp_drawable_color (drawable->id) || gimp_drawable_gray (drawable->id) || gimp_drawable_indexed (drawable->id)) {
+  if (gimp_drawable_is_rgb (drawable->id) || gimp_drawable_is_gray (drawable->id) || gimp_drawable_is_indexed (drawable->id)) {
 
       maze (drawable);
       
@@ -385,7 +400,7 @@ maze( GDrawable * drawable)
   /* Get the foreground and background colors */
   get_colors(drawable,fg,bg);
 
-  gimp_progress_init ("Drawing Maze...");
+  gimp_progress_init (_("Drawing Maze..."));
 
   for (pr = gimp_pixel_rgns_register (1, &dest_rgn); 
        pr != NULL; 
@@ -668,8 +683,8 @@ mask_maze(gint32 drawable_ID, guchar *maz, guint mw, guint mh,
 	  } /* next yy */	    
 
      } /* next pr sel_rgn tile thing */
-  /* #ifdef MAZE_DEBUG
-     maze_dump(maz,mw,mh);
-     #endif */
+#ifdef MAZE_DEBUG
+     /* maze_dump(maz,mw,mh); */
+#endif
 } /* mask_maze */
 #endif /* 0 */
