@@ -1,9 +1,10 @@
 /*
- * "$Id: print-ps.c,v 1.9.4.1 1998/08/28 01:58:15 yosh Exp $"
+ * "$Id: print-ps.c,v 1.19 2000/01/28 13:12:50 neo Exp $"
  *
  *   Print plug-in Adobe PostScript driver for the GIMP.
  *
- *   Copyright 1997-1998 Michael Sweet (mike@easysw.com)
+ *   Copyright 1997-2000 Michael Sweet (mike@easysw.com) and
+ *	Robert Krawitz (rlk@alum.mit.edu)
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the Free
@@ -31,82 +32,14 @@
  *
  * Revision History:
  *
- *   $Log: print-ps.c,v $
- *   Revision 1.9.4.1  1998/08/28 01:58:15  yosh
- *   s/strdup/g_strdup/ for portability
- *
- *   -Yosh
- *
- *   Revision 1.9  1998/05/17 07:16:47  yosh
- *   0.99.31 fun
- *
- *   updated print plugin
- *
- *   -Yosh
- *
- *   Revision 1.13  1998/05/15  21:01:51  mike
- *   Updated image positioning code (invert top and center left/top independently)
- *   Updated ps_imageable_area() to return a default imageable area when no PPD
- *   file is available.
- *
- *   Revision 1.12  1998/05/11  23:56:56  mike
- *   Removed unused outptr variable.
- *
- *   Revision 1.11  1998/05/08  19:20:50  mike
- *   Updated to support PPD files, media size, imageable area, and parameter
- *   functions.
- *   Added support for scaling modes - scale by percent or scale by PPI.
- *   Updated Ascii85 output - some Level 2 printers are buggy and won't accept
- *   whitespace in the data stream.
- *   Now use image dictionaries with Level 2 printers - allows interpolation
- *   flag to be sent (not all printers use this flag).
- *
- *   Revision 1.10  1998/01/22  15:38:46  mike
- *   Updated copyright notice.
- *   Whoops - wasn't encoding correctly for portrait output to level 2 printers!
- *
- *   Revision 1.9  1998/01/21  21:33:47  mike
- *   Added support for Level 2 filters; images are now sent in hex or
- *   base-85 ASCII as necessary (faster printing).
- *
- *   Revision 1.8  1997/11/12  15:57:48  mike
- *   Minor changes for clean compiles under Digital UNIX.
- *
- *   Revision 1.8  1997/11/12  15:57:48  mike
- *   Minor changes for clean compiles under Digital UNIX.
- *
- *   Revision 1.7  1997/07/30  20:33:05  mike
- *   Final changes for 1.1 release.
- *
- *   Revision 1.7  1997/07/30  20:33:05  mike
- *   Final changes for 1.1 release.
- *
- *   Revision 1.6  1997/07/30  18:47:39  mike
- *   Added scaling, orientation, and offset options.
- *
- *   Revision 1.5  1997/07/26  18:38:55  mike
- *   Bug - was using asctime instead of ctime...  D'oh!
- *
- *   Revision 1.4  1997/07/26  18:19:54  mike
- *   Fixed positioning/scaling bug.
- *
- *   Revision 1.3  1997/07/03  13:26:46  mike
- *   Updated documentation for 1.0 release.
- *
- *   Revision 1.2  1997/07/02  18:49:36  mike
- *   Forgot to free memory buffers...
- *
- *   Revision 1.2  1997/07/02  18:49:36  mike
- *   Forgot to free memory buffers...
- *
- *   Revision 1.1  1997/07/02  13:51:53  mike
- *   Initial revision
+ *   See ChangeLog
  */
 
 #include "print.h"
 #include <time.h>
-/*#define DEBUG*/
+#include <glib.h>
 
+/*#define DEBUG*/
 
 /*
  * Local variables...
@@ -120,8 +53,8 @@ static char	*ps_ppd_file = NULL;
  * Local functions...
  */
 
-static void	ps_hex(FILE *, guchar *, int);
-static void	ps_ascii85(FILE *, guchar *, int, int);
+static void	ps_hex(FILE *, unsigned short *, int);
+static void	ps_ascii85(FILE *, unsigned short *, int, int);
 static char	*ppd_find(char *, char *, char *, int *);
 
 
@@ -142,12 +75,12 @@ ps_parameters(int  model,	/* I - Printer model */
   char		**valptrs;
   static char	*media_sizes[] =
 		{
-		  "Letter",
-		  "Legal",
-		  "A4",
-		  "Tabloid",
-		  "A3",
-		  "12x18"
+		  ("Letter"),
+		  ("Legal"),
+		  ("A4"),
+		  ("Tabloid"),
+		  ("A3"),
+		  ("12x18")
 		};
 
 
@@ -170,7 +103,7 @@ ps_parameters(int  model,	/* I - Printer model */
       ps_ppd_file = NULL;
     else
       ps_ppd_file = ppd_file;
-  };
+  }
 
   if (ps_ppd == NULL)
   {
@@ -178,20 +111,24 @@ ps_parameters(int  model,	/* I - Printer model */
     {
       *count = 6;
 
-      valptrs = g_new(char *, 6);
-      for (i = 0; i < 6; i ++)
-        valptrs[i] = g_strdup(media_sizes[i]);
+      valptrs = malloc(*count * sizeof(char *));
+      for (i = 0; i < *count; i ++)
+	{
+	  /* strdup doesn't appear to be POSIX... */
+	  valptrs[i] = malloc(strlen(media_sizes[i]) + 1);
+	  strcpy(valptrs[i], media_sizes[i]);
+	}
 
       return (valptrs);
     }
     else
       return (NULL);
-  };
+  }
 
   rewind(ps_ppd);
   *count = 0;
 
-  valptrs = g_new(char *, 100);
+  valptrs = malloc(100 * sizeof(char *));
 
   while (fgets(line, sizeof(line), ps_ppd) != NULL)
   {
@@ -201,16 +138,17 @@ ps_parameters(int  model,	/* I - Printer model */
     if (sscanf(line, "*%s %[^/:]", lname, loption) != 2)
       continue;
 
-    if (strcasecmp(lname, name) == 0)
+    if (g_strcasecmp(lname, name) == 0)
     {
-      valptrs[*count] = g_strdup(loption);
+      valptrs[(*count)] = malloc(strlen(loption) + 1);
+      strcpy(valptrs[(*count)], loption);
       (*count) ++;
-    };
-  };
+    }
+  }
 
   if (*count == 0)
   {
-    g_free(valptrs);
+    free(valptrs);
     return (NULL);
   }
   else
@@ -286,7 +224,7 @@ ps_imageable_area(int  model,		/* I - Printer model */
     *right  -= 18;
     *top    -= 36;
     *bottom = 36;
-  };
+  }
 }
 
 
@@ -296,27 +234,27 @@ ps_imageable_area(int  model,		/* I - Printer model */
 
 void
 ps_print(int       model,		/* I - Model (Level 1 or 2) */
-         char      *ppd_file,		/* I - PPD file */
-         char      *resolution,		/* I - Resolution */
-         char      *media_size,		/* I - Media size */
-         char      *media_type,		/* I - Media type */
-         char      *media_source,	/* I - Media source */
-         int       output_type,		/* I - Output type (color/grayscale) */
-         int       orientation,		/* I - Orientation of image */
-         float     scaling,		/* I - Scaling of image */
-         int       left,		/* I - Left offset of image (points) */
-         int       top,			/* I - Top offset of image (points) */
          int       copies,		/* I - Number of copies */
          FILE      *prn,		/* I - File to print to */
-         GDrawable *drawable,		/* I - Image to print */
-         guchar    *lut,		/* I - Brightness lookup table */
-         guchar    *cmap)		/* I - Colormap (for indexed images) */
+         Image     image,		/* I - Image to print */
+	 unsigned char    *cmap,	/* I - Colormap (for indexed images) */
+	 lut_t     *lut,		/* I - Brightness lookup table */
+	 vars_t    *v)
 {
+  char 		*ppd_file = v->ppd_file;
+  char 		*resolution = v->resolution;
+  char 		*media_size = v->media_size;
+  char 		*media_type = v->media_type;
+  char 		*media_source = v->media_source;
+  int 		output_type = v->output_type;
+  int		orientation = v->orientation;
+  float 	scaling = v->scaling;
+  int		top = v->top;
+  int		left = v->left;
   int		i, j;		/* Looping vars */
   int		x, y;		/* Looping vars */
-  GPixelRgn	rgn;		/* Image region */
-  guchar	*in,		/* Input pixels from image */
-		*out;		/* Output pixels for printer */
+  unsigned char	*in;		/* Input pixels from image */
+  unsigned short	*out;		/* Output pixels for printer */
   int		page_left,	/* Left margin of page */
 		page_right,	/* Right margin of page */
 		page_top,	/* Top of page */
@@ -341,27 +279,32 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
     char	*command;
     int		order;
   }		commands[4];
+  int           image_height,
+                image_width,
+                image_bpp;
 
 
  /*
   * Setup a read-only pixel region for the entire image...
   */
 
-  gimp_pixel_rgn_init(&rgn, drawable, 0, 0, drawable->width, drawable->height,
-                      FALSE, FALSE);
+  Image_init(image);
+  image_height = Image_height(image);
+  image_width = Image_width(image);
+  image_bpp = Image_bpp(image);
 
  /*
   * Choose the correct color conversion function...
   */
 
-  if (drawable->bpp < 3 && cmap == NULL)
+  if (image_bpp < 3 && cmap == NULL)
     output_type = OUTPUT_GRAY;		/* Force grayscale output */
 
   if (output_type == OUTPUT_COLOR)
   {
     out_bpp = 3;
 
-    if (drawable->bpp >= 3)
+    if (image_bpp >= 3)
       colorfunc = rgb_to_rgb;
     else
       colorfunc = indexed_to_rgb;
@@ -370,13 +313,13 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
   {
     out_bpp = 1;
 
-    if (drawable->bpp >= 3)
+    if (image_bpp >= 3)
       colorfunc = rgb_to_gray;
     else if (cmap == NULL)
       colorfunc = gray_to_gray;
     else
       colorfunc = indexed_to_gray;
-  };
+  }
 
  /*
   * Compute the output size...
@@ -391,7 +334,7 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
 
 #ifdef DEBUG
   printf("page_width = %d, page_height = %d\n", page_width, page_height);
-  printf("drawable->width = %d, drawable->height = %d\n", drawable->width, drawable->height);
+  printf("image_width = %d, image_height = %d\n", image_width, image_height);
   printf("scaling = %.1f\n", scaling);
 #endif /* DEBUG */
 
@@ -405,8 +348,8 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
     * Scale to pixels per inch...
     */
 
-    out_width  = drawable->width * -72.0 / scaling;
-    out_height = drawable->height * -72.0 / scaling;
+    out_width  = image_width * -72.0 / scaling;
+    out_height = image_height * -72.0 / scaling;
   }
   else
   {
@@ -415,13 +358,13 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
     */
 
     out_width  = page_width * scaling / 100.0;
-    out_height = out_width * drawable->height / drawable->width;
+    out_height = out_width * image_height / image_width;
     if (out_height > page_height)
     {
       out_height = page_height * scaling / 100.0;
-      out_width  = out_height * drawable->width / drawable->height;
-    };
-  };
+      out_width  = out_height * image_width / image_height;
+    }
+  }
 
  /*
   * Landscape width/height...
@@ -433,8 +376,8 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
     * Scale to pixels per inch...
     */
 
-    temp_width  = drawable->height * -72.0 / scaling;
-    temp_height = drawable->width * -72.0 / scaling;
+    temp_width  = image_height * -72.0 / scaling;
+    temp_height = image_width * -72.0 / scaling;
   }
   else
   {
@@ -443,13 +386,13 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
     */
 
     temp_width  = page_width * scaling / 100.0;
-    temp_height = temp_width * drawable->width / drawable->height;
+    temp_height = temp_width * image_width / image_height;
     if (temp_height > page_height)
     {
       temp_height = page_height;
-      temp_width  = temp_height * drawable->height / drawable->width;
-    };
-  };
+      temp_width  = temp_height * image_height / image_width;
+    }
+  }
 
  /*
   * See which orientation has the greatest area (or if we need to rotate the
@@ -472,8 +415,8 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
 	orientation = ORIENT_LANDSCAPE;
       else
 	orientation = ORIENT_PORTRAIT;
-    };
-  };
+    }
+  }
 
   if (orientation == ORIENT_LANDSCAPE)
   {
@@ -488,13 +431,13 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
     x    = top;
     top  = left;
     left = x;
-  };
+  }
 
  /*
   * Let the user know what we're doing...
   */
 
-  gimp_progress_init("Printing...");
+  Image_progress_init(image);
 
  /*
   * Output a standard PostScript header with DSC comments...
@@ -517,10 +460,13 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
   printf("left = %d, top = %d\n", left, top);
 #endif /* DEBUG */
 
+#ifdef __EMX__
+  _fsetmode(prn, "t");
+#endif
   fputs("%!PS-Adobe-3.0\n", prn);
-  fputs("%%Creator: " PLUG_IN_NAME " plug-in V" PLUG_IN_VERSION " for GIMP.\n", prn);
+  fprintf(prn, "%%Creator: %s\n", Image_get_pluginname(image));
   fprintf(prn, "%%%%CreationDate: %s", ctime(&curtime));
-  fputs("%%Copyright: 1997-1998 by Michael Sweet (mike@easysw.com)\n", prn);
+  fputs("%%Copyright: 1997-1999 by Michael Sweet (mike@easysw.com) and Robert Krawitz (rlk@alum.mit.edu)\n", prn);
   fprintf(prn, "%%%%BoundingBox: %d %d %d %d\n",
           left, top - out_height, left + out_width, top);
   fputs("%%DocumentData: Clean7Bit\n", prn);
@@ -537,31 +483,35 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
 
   if ((command = ppd_find(ppd_file, "PageSize", media_size, &order)) != NULL)
   {
-    commands[num_commands].command = g_strdup(command);
+    commands[num_commands].command = malloc(strlen(command) + 1);
+    strcpy(commands[num_commands].command, command);
     commands[num_commands].order   = order;
     num_commands ++;
-  };
+  }
 
   if ((command = ppd_find(ppd_file, "InputSlot", media_source, &order)) != NULL)
   {
-    commands[num_commands].command = g_strdup(command);
+    commands[num_commands].command = malloc(strlen(command) + 1);
+    strcpy(commands[num_commands].command, command);
     commands[num_commands].order   = order;
     num_commands ++;
-  };
+  }
 
   if ((command = ppd_find(ppd_file, "MediaType", media_type, &order)) != NULL)
   {
-    commands[num_commands].command = g_strdup(command);
+    commands[num_commands].command = malloc(strlen(command) + 1);
+    strcpy(commands[num_commands].command, command);
     commands[num_commands].order   = order;
     num_commands ++;
-  };
+  }
 
   if ((command = ppd_find(ppd_file, "Resolution", resolution, &order)) != NULL)
   {
-    commands[num_commands].command = g_strdup(command);
+    commands[num_commands].command = malloc(strlen(command) + 1);
+    strcpy(commands[num_commands].command, command);
     commands[num_commands].order   = order;
     num_commands ++;
-  };
+  }
 
  /*
   * Sort the commands using the OrderDependency value...
@@ -577,7 +527,7 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
         commands[i].order   = commands[j].order;
         commands[j].command = command;
         commands[j].order   = order;
-      };
+      }
 
  /*
   * Send the commands...
@@ -591,10 +541,10 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
     {
       fputs(commands[i].command, prn);
       free(commands[i].command);
-    };
+    }
 
     fputs("%%EndProlog\n", prn);
-  };
+  }
 
  /*
   * Output the page, rotating as necessary...
@@ -607,25 +557,25 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
   {
     fprintf(prn, "%d %d translate\n", left, top - out_height);
     fprintf(prn, "%.3f %.3f scale\n",
-            (float)out_width / ((float)drawable->height),
-            (float)out_height / ((float)drawable->width));
+            (float)out_width / ((float)image_height),
+            (float)out_height / ((float)image_width));
   }
   else
   {
     fprintf(prn, "%d %d translate\n", left, top);
     fprintf(prn, "%.3f %.3f scale\n",
-            (float)out_width / ((float)drawable->width),
-            (float)out_height / ((float)drawable->height));
-  };
+            (float)out_width / ((float)image_width),
+            (float)out_height / ((float)image_height));
+  }
 
-  in  = g_malloc(drawable->width * drawable->bpp);
-  out = g_malloc(drawable->width * out_bpp + 3);
+  in  = malloc(image_width * image_bpp);
+  out = malloc((image_width * out_bpp + 3) * 2);
 
   if (model == 0)
   {
-    fprintf(prn, "/picture %d string def\n", drawable->width * out_bpp);
+    fprintf(prn, "/picture %d string def\n", image_width * out_bpp);
 
-    fprintf(prn, "%d %d 8\n", drawable->width, drawable->height);
+    fprintf(prn, "%d %d 8\n", image_width, image_height);
 
     if (landscape)
       fputs("[ 0 1 1 0 0 0 ]\n", prn);
@@ -637,16 +587,16 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
     else
       fputs("{currentfile picture readhexstring pop} false 3 colorimage\n", prn);
 
-    for (y = 0; y < drawable->height; y ++)
+    for (y = 0; y < image_height; y ++)
     {
       if ((y & 15) == 0)
-        gimp_progress_update((double)y / (double)drawable->height);
+	Image_note_progress(image, y, image_height);
 
-      gimp_pixel_rgn_get_row(&rgn, in, 0, y, drawable->width);
-      (*colorfunc)(in, out, drawable->width, drawable->bpp, lut, cmap);
+      Image_get_row(image, in, y);
+      (*colorfunc)(in, out, image_width, image_bpp, lut, cmap, v);
 
-      ps_hex(prn, out, drawable->width * out_bpp);
-    };
+      ps_hex(prn, out, image_width * out_bpp);
+    }
   }
   else
   {
@@ -658,8 +608,8 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
     fputs("<<\n", prn);
     fputs("\t/ImageType 1\n", prn);
 
-    fprintf(prn, "\t/Width %d\n", drawable->width);
-    fprintf(prn, "\t/Height %d\n", drawable->height);
+    fprintf(prn, "\t/Width %d\n", image_width);
+    fprintf(prn, "\t/Height %d\n", image_height);
     fputs("\t/BitsPerComponent 8\n", prn);
 
     if (output_type == OUTPUT_GRAY)
@@ -669,7 +619,7 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
 
     fputs("\t/DataSource currentfile /ASCII85Decode filter\n", prn);
 
-    if ((drawable->width * 72 / out_width) < 100)
+    if ((image_width * 72 / out_width) < 100)
       fputs("\t/Interpolate true\n", prn);
 
     if (landscape)
@@ -680,17 +630,17 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
     fputs(">>\n", prn);
     fputs("image\n", prn);
 
-    for (y = 0, out_offset = 0; y < drawable->height; y ++)
+    for (y = 0, out_offset = 0; y < image_height; y ++)
     {
       if ((y & 15) == 0)
-        gimp_progress_update((double)y / (double)drawable->height);
+	Image_note_progress(image, y, image_height);
 
-      gimp_pixel_rgn_get_row(&rgn, in, 0, y, drawable->width);
-      (*colorfunc)(in, out + out_offset, drawable->width, drawable->bpp, lut, cmap);
+      Image_get_row(image, in, y);
+      (*colorfunc)(in, out + out_offset, image_width, image_bpp, lut, cmap, v);
 
-      out_length = out_offset + drawable->width * out_bpp;
+      out_length = out_offset + image_width * out_bpp;
 
-      if (y < (drawable->height - 1))
+      if (y < (image_height - 1))
       {
         ps_ascii85(prn, out, out_length & ~3, 0);
         out_offset = out_length & 3;
@@ -699,15 +649,15 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
       {
         ps_ascii85(prn, out, out_length, 1);
         out_offset = 0;
-      };
+      }
 
       if (out_offset > 0)
         memcpy(out, out + out_length - out_offset, out_offset);
-    };
-  };
+    }
+  }
 
-  g_free(in);
-  g_free(out);
+  free(in);
+  free(out);
 
   fputs("grestore\n", prn);
   fputs("showpage\n", prn);
@@ -722,7 +672,7 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
 
 static void
 ps_hex(FILE   *prn,	/* I - File to print to */
-       guchar *data,	/* I - Data to print */
+       unsigned short *data,	/* I - Data to print */
        int    length)	/* I - Number of bytes to print */
 {
   int		col;	/* Current column */
@@ -732,13 +682,14 @@ ps_hex(FILE   *prn,	/* I - File to print to */
   col = 0;
   while (length > 0)
   {
+    unsigned char pixel = (*data & 0xff00) >> 8;
    /*
     * Put the hex chars out to the file; note that we don't use fprintf()
     * for speed reasons...
     */
 
-    putc(hex[*data >> 4], prn);
-    putc(hex[*data & 15], prn);
+    putc(hex[pixel >> 4], prn);
+    putc(hex[pixel & 15], prn);
 
     data ++;
     length --;
@@ -746,7 +697,7 @@ ps_hex(FILE   *prn,	/* I - File to print to */
     col = (col + 1) & 31;
     if (col == 0)
       putc('\n', prn);
-  };
+  }
 
   if (col > 0)
     putc('\n', prn);
@@ -759,7 +710,7 @@ ps_hex(FILE   *prn,	/* I - File to print to */
 
 static void
 ps_ascii85(FILE   *prn,		/* I - File to print to */
-	   guchar *data,	/* I - Data to print */
+	   unsigned short *data,	/* I - Data to print */
 	   int    length,	/* I - Number of bytes to print */
 	   int    last_line)	/* I - Last line of raster data? */
 {
@@ -770,7 +721,11 @@ ps_ascii85(FILE   *prn,		/* I - File to print to */
 
   while (length > 3)
   {
-    b = (((((data[0] << 8) | data[1]) << 8) | data[2]) << 8) | data[3];
+    unsigned char d0 = (data[0] & 0xff00) >> 8;
+    unsigned char d1 = (data[1] & 0xff00) >> 8;
+    unsigned char d2 = (data[2] & 0xff00) >> 8;
+    unsigned char d3 = (data[3] & 0xff00) >> 8;
+    b = (((((d0 << 8) | d1) << 8) | d2) << 8) | d3;
 
     if (b == 0)
       putc('z', prn);
@@ -787,11 +742,11 @@ ps_ascii85(FILE   *prn,		/* I - File to print to */
       c[0] = b + '!';
 
       fwrite(c, 5, 1, prn);
-    };
+    }
 
     data += 4;
     length -= 4;
-  };
+  }
 
   if (last_line)
   {
@@ -810,10 +765,10 @@ ps_ascii85(FILE   *prn,		/* I - File to print to */
       c[0] = b + '!';
 
       fwrite(c, length + 1, 1, prn);
-    };
+    }
 
     fputs("~>\n", prn);
-  };
+  }
 }
 
 
@@ -848,7 +803,7 @@ ppd_find(char *ppd_file,	/* I - Name of PPD file */
       ps_ppd_file = NULL;
     else
       ps_ppd_file = ppd_file;
-  };
+  }
 
   if (ps_ppd == NULL)
     return (NULL);
@@ -862,7 +817,7 @@ ppd_find(char *ppd_file,	/* I - Name of PPD file */
     if (line[0] != '*')
       continue;
 
-    if (strncasecmp(line, "*OrderDependency:", 17) == 0 && order != NULL)
+    if (g_strncasecmp(line, "*OrderDependency:", 17) == 0 && order != NULL)
     {
       sscanf(line, "%*s%d", order);
       continue;
@@ -870,8 +825,8 @@ ppd_find(char *ppd_file,	/* I - Name of PPD file */
     else if (sscanf(line, "*%s %[^/:]", lname, loption) != 2)
       continue;
 
-    if (strcasecmp(lname, name) == 0 &&
-        strcasecmp(loption, option) == 0)
+    if (g_strcasecmp(lname, name) == 0 &&
+        g_strcasecmp(loption, option) == 0)
     {
       opt = strchr(line, ':') + 1;
       while (*opt == ' ' || *opt == '\t')
@@ -889,20 +844,20 @@ ppd_find(char *ppd_file,	/* I - Name of PPD file */
           {
             strcpy(strchr(value, '\"'), "\n");
             break;
-          };
-        };
+          }
+        }
       }
       else
         *opt = '\0';
 
       return (value);
-    };
-  };
+    }
+  }
 
   return (NULL);
 }
 
 
 /*
- * End of "$Id: print-ps.c,v 1.9.4.1 1998/08/28 01:58:15 yosh Exp $".
+ * End of "$Id: print-ps.c,v 1.19 2000/01/28 13:12:50 neo Exp $".
  */
