@@ -16,20 +16,39 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/types.h>
+#if HAVE_DIRENT_H
 #include <dirent.h>
+#endif
+#if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <sys/stat.h>
+
 #include "gdk/gdkkeysyms.h"
 #include "gtk/gtk.h"
+
 #include "libgimp/gimp.h"
 #include "libgimp/gimpui.h"
+
+#include "libgimp/stdplugins-intl.h"
+
 #include "siod.h"
 #include "script-fu-console.h"
 #include <plug-ins/dbbrowser/dbbrowser.h>
 
+#ifdef G_OS_WIN32
+#include <fcntl.h>
+#include <io.h>
+#endif
+
+#ifndef G_OS_WIN32
+
 #define TEXT_WIDTH  400
 #define TEXT_HEIGHT 400
 #define ENTRY_WIDTH 400
@@ -100,6 +119,8 @@ extern char  siod_err_msg[];
 extern FILE *siod_output;
 
 
+#define message(string) printf("(%s): %d ::: %s\n", __PRETTY_FUNCTION__, __LINE__, string)
+
 /*
  *  Function definitions
  */
@@ -133,7 +154,7 @@ script_fu_console_run (char     *name,
     case RUN_WITH_LAST_VALS:
     case RUN_NONINTERACTIVE:
       status = STATUS_CALLING_ERROR;
-      gimp_message ("Script-Fu console mode allows only interactive invocation");
+      gimp_message (_("Script-Fu console mode allows only interactive invocation"));
       break;
 
     default:
@@ -148,7 +169,7 @@ script_fu_console_run (char     *name,
 }
 
 static void
-script_fu_console_interface ()
+script_fu_console_interface (void)
 {
   GtkWidget *dlg;
   GtkWidget *button;
@@ -166,8 +187,10 @@ script_fu_console_interface ()
   gtk_init (&argc, &argv);
   gtk_rc_parse (gimp_gtkrc ());
 
+  INIT_I18N_UI();
+
   dlg = gtk_dialog_new ();
-  gtk_window_set_title (GTK_WINDOW (dlg), "Script-Fu Console");
+  gtk_window_set_title (GTK_WINDOW (dlg), _("Script-Fu Console"));
   gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
 		      (GtkSignalFunc) script_fu_close_callback,
 		      NULL);
@@ -179,7 +202,7 @@ script_fu_console_interface ()
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dlg)->action_area), 2);
 
   /*  Action area  */
-  button = gtk_button_new_with_label ("Close");
+  button = gtk_button_new_with_label (_("Close"));
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
                       (GtkSignalFunc) script_fu_close_callback,
                       NULL);
@@ -187,7 +210,7 @@ script_fu_console_interface ()
   gtk_widget_show (button);
 
   /*  The info vbox  */
-  label = gtk_label_new ("SIOD Output");
+  label = gtk_label_new (_("SIOD Output"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), label, FALSE, TRUE, 0);
   gtk_widget_show (label);
@@ -264,7 +287,7 @@ script_fu_console_interface ()
   gtk_widget_show (table);
 
   /*  The current command  */
-  label = gtk_label_new ("Current Command");
+  label = gtk_label_new (_("Current Command"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), label, FALSE, TRUE, 0);
   gtk_widget_show (label);
@@ -285,7 +308,7 @@ script_fu_console_interface ()
 		      (GtkSignalFunc) script_fu_cc_key_function,
 		      NULL);
 
-  button = gtk_button_new_with_label ("Browse...");
+  button = gtk_button_new_with_label (_("Browse..."));
   gtk_widget_set_usize (button, (ENTRY_WIDTH)/6, 0);
   gtk_box_pack_start (GTK_BOX (hbox), button, 
 		      FALSE, TRUE, 0);
@@ -321,37 +344,42 @@ script_fu_close_callback (GtkWidget *widget,
   gtk_main_quit ();
 }
 
-void apply_callback( gchar *proc_name,
-			   gchar *scheme_proc_name,
-			   gchar *proc_blurb,
-			   gchar *proc_help,
-			   gchar *proc_author,
-			   gchar *proc_copyright,
-			   gchar *proc_date,
-			   int proc_type,
-			   int nparams,
-			   int nreturn_vals,
-			   GParamDef *params,
-			   GParamDef *return_vals )
+void 
+apply_callback (gchar     *proc_name,
+		gchar     *scheme_proc_name,
+		gchar     *proc_blurb,
+		gchar     *proc_help,
+		gchar     *proc_author,
+		gchar     *proc_copyright,
+		gchar     *proc_date,
+		int        proc_type,
+		int        nparams,
+		int        nreturn_vals,
+		GParamDef *params,
+		GParamDef *return_vals )
 {
   gint i;
+  GString *text;
 
-  if (proc_name==NULL) return;
-  gtk_widget_hide(cint.cc);
-  gtk_entry_set_text( GTK_ENTRY(cint.cc), "(" );
-  gtk_entry_append_text( GTK_ENTRY(cint.cc), scheme_proc_name );
-  if ((nparams==0) || (params==NULL)) return;
-  for (i=0;i<nparams;i++) {
-    gtk_entry_append_text( GTK_ENTRY(cint.cc), " " );
-    gtk_entry_append_text( GTK_ENTRY(cint.cc), params[i].name);
-  }
-  gtk_entry_append_text( GTK_ENTRY(cint.cc), ")" );
-  gtk_widget_show(cint.cc);
+  if (proc_name == NULL) 
+    return;
+  
+  text = g_string_new ("(");
+  text = g_string_append (text, scheme_proc_name);
+  for (i=0; i<nparams; i++) 
+    {
+      text = g_string_append_c (text, ' ');
+      text = g_string_append (text, params[i].name);
+    }
+  text = g_string_append_c (text, ')');
+
+  gtk_entry_set_text (GTK_ENTRY (cint.cc), text->str);
+  g_string_free (text, TRUE);
 }
 
 static void
-script_fu_browse_callback(GtkWidget *widget,
-			  gpointer   data)
+script_fu_browse_callback (GtkWidget *widget,
+			   gpointer   data)
 {
   gtk_quit_add_destroy (1, (GtkObject*) gimp_db_browser (apply_callback));
 }
@@ -380,12 +408,26 @@ script_fu_siod_read (gpointer          data,
 		     GdkInputCondition cond)
 {
   int count;
+  static int hack = 0;
 
   if ((count = read (id, read_buffer, BUFSIZE - 1)) != 0)
     {
+      if (!hack) /* this is a stupid hack, but as of 10/27/98
+		 * the script-fu-console will hang on my system without it.
+		 * the real cause of this needs to be tracked down.
+		 * posibly a problem with the text widget, or the
+		 * signal handlers not getting fully initialized or...
+		 * no reports of hangs on other platforms, could just be a
+		 * problem with my system.
+		 */
+      {
+	hack = 1;
+	return;
+      }
       read_buffer[count] = '\0';
       gtk_text_freeze (GTK_TEXT (cint.console));
-      gtk_text_insert (GTK_TEXT (cint.console), cint.font_weak, NULL, NULL, read_buffer, -1);
+      gtk_text_insert (GTK_TEXT (cint.console), cint.font_weak, NULL, NULL,
+		       read_buffer, -1);
       gtk_text_thaw (GTK_TEXT (cint.console));
 
       script_fu_console_scroll_end (NULL);
@@ -393,7 +435,7 @@ script_fu_siod_read (gpointer          data,
 }
 
 static gint
-script_fu_cc_is_empty ()
+script_fu_cc_is_empty (void)
 {
   char *str;
 
@@ -523,17 +565,17 @@ script_fu_cc_key_function (GtkWidget   *widget,
 
 
 static FILE *
-script_fu_open_siod_console ()
+script_fu_open_siod_console (void)
 {
   if (siod_output == stdout)
     {
       if (pipe (siod_output_pipe))
 	{
-	  gimp_message ("Unable to open SIOD output pipe");
+	  gimp_message (_("Unable to open SIOD output pipe"));
 	}
       else if ((siod_output = fdopen (siod_output_pipe [1], "w")) == NULL)
 	{
-	  gimp_message ("Unable to open a stream on the SIOD output pipe");
+	  gimp_message (_("Unable to open a stream on the SIOD output pipe"));
 	  siod_output = stdout;
 	}
       else
@@ -547,7 +589,7 @@ script_fu_open_siod_console ()
 }
 
 static void
-script_fu_close_siod_console ()
+script_fu_close_siod_console (void)
 {
   if (siod_output != stdout)
     fclose (siod_output);
@@ -555,6 +597,8 @@ script_fu_close_siod_console ()
   close (siod_output_pipe[1]);
 }
 
+#endif /* !G_OS_WIN32 */
+
 void
 script_fu_eval_run (char     *name,
 		    int       nparams,
@@ -578,7 +622,7 @@ script_fu_eval_run (char     *name,
     case RUN_INTERACTIVE:
     case RUN_WITH_LAST_VALS:
       status = STATUS_CALLING_ERROR;
-      gimp_message ("Script-Fu evaluate mode allows only noninteractive invocation");
+      gimp_message (_("Script-Fu evaluate mode allows only noninteractive invocation"));
       break;
 
     default:
