@@ -18,6 +18,12 @@
 ; You should have received a copy of the GNU General Public License
 ; along with this program; if not, write to the Free Software
 ; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; ************************************************************************
+; Changed on Feb 4, 1999 by Piet van Oostrum <piet@cs.uu.nl>
+; For use with GIMP 1.1.
+; All calls to gimp-text-* have been converted to use the *-fontname form.
+; The corresponding parameters have been replaced by an SF-FONT parameter.
+; ************************************************************************
 
 
 (define (text-width extents)
@@ -33,8 +39,7 @@
   (cadr (cddr extents)))
 
 (define (blend-bumpmap img drawable x1 y1 x2 y2)
-  (gimp-blend img
-	      drawable
+  (gimp-blend drawable
 	      FG-BG-RGB
 	      DARKEN-ONLY
 	      LINEAR
@@ -51,12 +56,7 @@
 
 (define (script-fu-button00 text
 			    size
-			    foundry
-			    family
-			    weight
-			    slant
-			    set-width
-			    spacing
+			    font
 			    ul-color
 			    lr-color
 			    text-color
@@ -66,29 +66,19 @@
   (let* ((old-fg-color (car (gimp-palette-get-foreground)))
 	 (old-bg-color (car (gimp-palette-get-background)))
 	 
-	 (text-extents (gimp-text-get-extents text
+	 (text-extents (gimp-text-get-extents-fontname text
 					      size
 					      PIXELS
-					      foundry
-					      family
-					      weight
-					      slant
-					      set-width
-					      spacing))
+					      font))
 	 (ascent (text-ascent text-extents))
 	 (descent (text-descent text-extents))
 	 
 	 (img-width (+ (* 2 (+ padding bevel-width))
 		       (- (text-width text-extents)
-			  (text-width (gimp-text-get-extents " "
+			  (text-width (gimp-text-get-extents-fontname " "
 							     size
 							     PIXELS
-							     foundry
-							     family
-							     weight
-							     slant
-							     set-width
-							     spacing)))))
+							     font)))))
 	 (img-height (+ (* 2 (+ padding bevel-width))
 			(+ ascent descent)))
 
@@ -97,14 +87,14 @@
 	 (bumpmap (car (gimp-layer-new img img-width img-height RGBA_IMAGE "Bumpmap" 100 NORMAL)))
 	 (gradient (car (gimp-layer-new img img-width img-height RGBA_IMAGE "Gradient" 100 NORMAL))))
 
-    (gimp-image-disable-undo img)
+    (gimp-image-undo-disable img)
 
     ; Create bumpmap layer
     
     (gimp-image-add-layer img bumpmap -1)
     (gimp-palette-set-foreground '(0 0 0))
     (gimp-palette-set-background '(255 255 255))
-    (gimp-edit-fill img bumpmap)
+    (gimp-edit-fill bumpmap)
 
     (gimp-rect-select img 0 0 bevel-width img-height REPLACE FALSE 0)
     (blend-bumpmap img bumpmap 0 0 (- bevel-width 1) 0)
@@ -125,8 +115,7 @@
     (gimp-image-add-layer img gradient -1)
     (gimp-palette-set-foreground ul-color)
     (gimp-palette-set-background lr-color)
-    (gimp-blend img
-		gradient
+    (gimp-blend gradient
 		FG-BG-RGB
 		NORMAL
 		LINEAR
@@ -146,8 +135,8 @@
     ; Create text layer
 
     (gimp-palette-set-foreground text-color)
-    (let ((textl (car (gimp-text
-		       img -1 0 0 text 0 TRUE size PIXELS foundry family weight slant set-width spacing))))
+    (let ((textl (car (gimp-text-fontname
+		       img -1 0 0 text 0 TRUE size PIXELS font))))
       (gimp-layer-set-offsets textl
 			      (+ bevel-width padding)
 			      (+ bevel-width padding descent)))
@@ -157,29 +146,24 @@
     (gimp-selection-none img)
     (gimp-palette-set-foreground old-fg-color)
     (gimp-palette-set-background old-bg-color)
-    (gimp-image-enable-undo img)
+    (gimp-image-undo-enable img)
     (gimp-display-new img)))
 
 ; Register!
 
 (script-fu-register "script-fu-button00"
-		    "<Toolbox>/Xtns/Script-Fu/Buttons/Simple beveled button"
+		    "<Toolbox>/Xtns/Script-Fu/Buttons/Simple Beveled Button..."
 		    "Simple beveled button"
 		    "Federico Mena Quintero"
 		    "Federico Mena Quintero"
 		    "June 1997"
 		    ""
-		    SF-VALUE "Text" "\"Hello world!\""
-		    SF-VALUE "Size" "16"
-		    SF-VALUE "Foundry" "\"adobe\""
-		    SF-VALUE "Family" "\"helvetica\""
-		    SF-VALUE "Weight" "\"bold\""
-		    SF-VALUE "Slant" "\"r\""
-		    SF-VALUE "Set width" "\"normal\""
-		    SF-VALUE "Spacing" "\"p\""
+		    SF-STRING "Text" "Hello world!"
+		    SF-ADJUSTMENT "Font Size (pixels)" '(16 2 100 1 1 0 1)
+		    SF-FONT "Font" "-*-helvetica-*-r-*-*-16-*-*-*-p-*-*-*"
 		    SF-COLOR "Upper-left color" '(0 255 127)
 		    SF-COLOR "Lower-right color" '(0 127 255)
 		    SF-COLOR "Text color" '(0 0 0)
 		    SF-VALUE "Padding" "2"
 		    SF-VALUE "Bevel width" "4"
-		    SF-VALUE "Pressed?" "FALSE")
+		    SF-TOGGLE "Pressed?" FALSE)
