@@ -52,11 +52,11 @@
 
 (define (copy-layer-crystal dest-image dest-drawable source-image source-drawable)
   (gimp-selection-all dest-image)
-  (gimp-edit-clear dest-image dest-drawable)
+  (gimp-edit-clear dest-drawable)
   (gimp-selection-none dest-image)
   (gimp-selection-all source-image)
-  (gimp-edit-copy source-image source-drawable)
-      (let ((floating-sel (car (gimp-edit-paste dest-image dest-drawable FALSE))))
+  (gimp-edit-copy source-drawable)
+      (let ((floating-sel (car (gimp-edit-paste dest-drawable FALSE))))
 	(gimp-floating-sel-anchor floating-sel)))
 
 (define (script-fu-crystal-logo chrome-factor text size font bg-img env-map)
@@ -77,7 +77,7 @@
 	 (blur (sota-scale size 0.5 chrome-factor))
 	 (displace (sota-scale size 0.25 chrome-factor))
 	 (brush-size (sota-scale size 0.5 chrome-factor))
-	 (text-layer (car (gimp-text img -1 0 0 text b-size TRUE size PIXELS "*" font "*" "*" "*" "*")))
+	 (text-layer (car (gimp-text-fontname img -1 0 0 text b-size TRUE size PIXELS font)))
 	 (width (car (gimp-drawable-width text-layer)))
 	 (height (car (gimp-drawable-height text-layer)))
 	 (tile-ret (plug-in-tile 1 back-img back-layer width height TRUE))
@@ -97,26 +97,26 @@
 	 (old-bg (car (gimp-palette-get-background)))
 	 (old-brush (car (gimp-brushes-get-brush))))
     (gimp-image-delete back-img)
-    (gimp-image-disable-undo img)
+    (gimp-image-undo-disable img)
     (gimp-image-resize img width height 0 0)
     (gimp-image-add-layer img layer3 0)
     (gimp-image-add-layer img layer2 0)
     (gimp-palette-set-background '(255 255 255))
     (gimp-selection-none img)
-    (gimp-edit-fill img layer2)
-    (gimp-edit-fill img layer3)
+    (gimp-edit-fill layer2)
+    (gimp-edit-fill layer3)
     (gimp-layer-set-visible text-layer FALSE)
 
-    (gimp-selection-layer-alpha img text-layer)
+    (gimp-selection-layer-alpha text-layer)
     (gimp-palette-set-background '(0 0 0))
     (gimp-selection-translate img offx1 offy1)
     (gimp-selection-feather img feather)
-    (gimp-edit-fill img layer2)
+    (gimp-edit-fill layer2)
     (gimp-selection-translate img (* 2 offx2) (* 2 offy2))
-    (gimp-edit-fill img layer3)
+    (gimp-edit-fill layer3)
     (gimp-selection-none img)
     (set! layer2 (car (gimp-image-merge-visible-layers img CLIP-TO-IMAGE)))
-    (gimp-invert img layer2)
+    (gimp-invert layer2)
 
     (copy-layer-crystal img layer1 banding-img banding-layer)
     (gimp-image-delete banding-img)
@@ -125,24 +125,24 @@
     (plug-in-gauss-iir 1 img layer1 10 TRUE TRUE)
     (gimp-layer-set-opacity layer1 50)
     (set! layer1 (car (gimp-image-merge-visible-layers img CLIP-TO-IMAGE)))
-    (gimp-curves-spline img layer1 0 18 (spline1))
+    (gimp-curves-spline layer1 0 18 (spline1))
 
     (set! layer-mask (car (gimp-layer-create-mask layer1 BLACK-MASK)))
     (gimp-image-add-layer-mask img layer1 layer-mask)
-    (gimp-selection-layer-alpha img text-layer)
+    (gimp-selection-layer-alpha text-layer)
     (gimp-palette-set-background '(255 255 255))
-    (gimp-edit-fill img layer-mask)
+    (gimp-edit-fill layer-mask)
 
     (set! disp-map (car (gimp-selection-save img)))
     (gimp-brushes-set-brush (crystal-brush brush-size))
     (gimp-palette-set-foreground '(0 0 0))
-    (gimp-edit-stroke img disp-map)
+    (gimp-edit-stroke disp-map)
     (gimp-selection-none img)
 
     (plug-in-gauss-rle 1 img disp-map blur TRUE TRUE)
-    (gimp-levels img disp-map 0 0 255 1.0 96 255)
+    (gimp-levels disp-map 0 0 255 1.0 96 255)
 
-    (if (= (car (gimp-drawable-color bg-layer)) 1)
+    (if (= (car (gimp-drawable-is-rgb bg-layer)) 1)
 	(gimp-convert-rgb img))
 
    
@@ -156,18 +156,18 @@
     (plug-in-displace 1 img layer2 displace displace TRUE TRUE disp-map disp-map 0)
     (set! layer-mask2 (car (gimp-layer-create-mask layer2 BLACK-MASK)))
     (gimp-image-add-layer-mask img layer2 layer-mask2)
-    (gimp-selection-layer-alpha img text-layer)
+    (gimp-selection-layer-alpha text-layer)
     (gimp-palette-set-background '(255 255 255))
-    (gimp-edit-fill img layer-mask2)
+    (gimp-edit-fill layer-mask2)
 
     (gimp-selection-none img)
-    (gimp-levels img layer2 0 0 200 1.5 50 255)
+    (gimp-levels layer2 0 0 200 1.5 50 255)
     (gimp-layer-set-mode layer1 OVERLAY)
 
     (plug-in-gauss-rle 1 img text-layer blur TRUE TRUE)
     (gimp-layer-set-preserve-trans text-layer TRUE)
     (gimp-palette-set-background '(0 0 0))
-    (gimp-edit-fill img text-layer)
+    (gimp-edit-fill text-layer)
     (gimp-layer-set-mode text-layer OVERLAY)
     (gimp-layer-translate text-layer offx1 offy1)
 
@@ -182,20 +182,22 @@
     (gimp-palette-set-foreground old-fg)
     (gimp-palette-set-background old-bg)
     (gimp-brushes-set-brush old-brush)
-    (gimp-image-enable-undo img)
+    (gimp-image-undo-enable img)
     (gimp-display-new img)))
 
 
 (script-fu-register "script-fu-crystal-logo"
-		    "<Toolbox>/Xtns/Script-Fu/Logos/Crystal"
+		    "<Toolbox>/Xtns/Script-Fu/Logos/Crystal..."
 		    "Crystal/Gel effect which displaces the image underneath"
 		    "Spencer Kimball"
 		    "Spencer Kimball"
 		    "1997"
 		    ""
 		    SF-VALUE "Chrome Factor" "1.0"
-		    SF-VALUE "Text String" "\"Crystal\""
-		    SF-VALUE "Font Size (in pixels)" "150"
-		    SF-VALUE "Font" "\"Engraver\""
-		    SF-VALUE "Background Img" (string-append "\"" gimp-data-dir "/scripts/texture1.jpg\"")
-		    SF-VALUE "Environment Map" (string-append "\"" gimp-data-dir "/scripts/beavis.jpg\""))
+		    SF-STRING "Text String" "Crystal"
+		    SF-ADJUSTMENT "Font Size (pixels)" '(150 2 1000 1 10 0 1)
+		    SF-FONT "Font" "-*-Engraver-*-r-*-*-24-*-*-*-p-*-*-*"
+;		    SF-STRING "Background Img" (string-append "" gimp-data-dir "/scripts/texture1.jpg")
+		    SF-FILENAME "Background Img" (string-append "" gimp-data-dir "/scripts/texture1.jpg")
+;		    SF-STRING "Environment Map" (string-append "" gimp-data-dir "/scripts/beavis.jpg"))
+		    SF-FILENAME "Environment Map" (string-append "" gimp-data-dir "/scripts/beavis.jpg"))
