@@ -16,22 +16,23 @@
 ; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ;
 ;
-; perspective-shadow.scm   version 1.02   12/13/97
+; perspective-shadow.scm   version 1.03   1999/12/21
 ;
 ; CHANGE-LOG:
 ; 1.00 - initial release
 ; 1.01 - fixed the problem with a remaining copy of the selection
 ; 1.02 - some code cleanup, no real changes
+; 1.03 - only call blur plugin when blut-radius >= 1.0
 ;
 ;
-; Copyright (C) 1997 Sven Neumann (neumanns@uni-duesseldorf.de)
+; Copyright (C) 1997-1999 Sven Neumann <sven@gimp.org>
 ; 
 ;  
 ; Adds a perspective shadow of the current selection or alpha-channel 
 ; as a layer below the active layer
 ;
   
-(define (script-fu-perspective-shadow image 
+(define (script-fu-perspective-shadow image
 				      drawable
 				      alpha
 				      rel-distance
@@ -55,15 +56,15 @@
 	 (shadow-layer 0))
 
     
-  (if (= rel-distance 0) (set! rel-distance 999999)) 
-  (gimp-image-disable-undo image)
+  (if (= rel-distance 0) (set! rel-distance 999999))
+  (gimp-image-undo-disable image)
   
   (gimp-layer-add-alpha drawable)
   (if (= (car (gimp-selection-is-empty image)) TRUE)
       (begin
-	(gimp-selection-layer-alpha image drawable)
+	(gimp-selection-layer-alpha drawable)
 	(set! from-selection FALSE))
-      (begin 
+      (begin
 	(set! from-selection TRUE)
 	(set! active-selection (car (gimp-selection-save image)))))
   
@@ -75,8 +76,8 @@
  
 	 (abs-length (* rel-length select-height))
 	 (abs-distance (* rel-distance select-height))
-	 (half-bottom-width (/ select-width 2)) 
-	 (half-top-width (* half-bottom-width 
+	 (half-bottom-width (/ select-width 2))
+	 (half-top-width (* half-bottom-width
 			  (/ (- rel-distance rel-length) rel-distance)))
   
 	 (x0 (+ select-offset-x (+ (- half-bottom-width half-top-width)
@@ -96,17 +97,17 @@
 	 (shadow-offset-y (- (min y0 y2) shadow-blur)))
 	 
   
-    (set! shadow-layer (car (gimp-layer-new image 
-					    select-width 
-					    select-height 
+    (set! shadow-layer (car (gimp-layer-new image
+					    select-width
+					    select-height
 					    type
-					    "Perspective Shadow" 
+					    "Perspective Shadow"
 					    shadow-opacity
 					    NORMAL)))
     (gimp-layer-set-offsets shadow-layer select-offset-x select-offset-y)
     (gimp-drawable-fill shadow-layer TRANS-IMAGE-FILL)
     (gimp-palette-set-background shadow-color)
-    (gimp-edit-fill image shadow-layer)
+    (gimp-edit-fill shadow-layer)
     (gimp-selection-none image)
 
     (if (= allow-resize TRUE)
@@ -131,91 +132,65 @@
 	  (if (> (+ shadow-height shadow-offset-y) new-image-height)
 	      (set! new-image-height (+ shadow-height shadow-offset-y)))
 	  (gimp-image-resize image
-			     new-image-width 
-			     new-image-height 
-			     image-offset-x 
+			     new-image-width
+			     new-image-height
+			     image-offset-x
 			     image-offset-y)))
 
     (gimp-image-add-layer image shadow-layer -1)
   
-    (gimp-perspective image
-		      shadow-layer
+    (gimp-perspective shadow-layer
 		      interpolate
 		      x0 y0
 		      x1 y1
 		      x2 y2
 		      x3 y3)
 
-    (if (> shadow-blur 0) 
+    (if (>= shadow-blur 1.0)
 	(begin
 	  (gimp-layer-set-preserve-trans shadow-layer FALSE)
-	  (gimp-layer-resize shadow-layer 
+	  (gimp-layer-resize shadow-layer
 			     shadow-width
 			     shadow-height
 			     shadow-blur
 			     shadow-blur)
-	  (plug-in-gauss-rle 1 
-			     image 
-			     shadow-layer 
-			     shadow-blur 
-			     TRUE 
+	  (plug-in-gauss-rle 1
+			     image
+			     shadow-layer
+			     shadow-blur
+			     TRUE
 			     TRUE))))
 
   (if (= from-selection TRUE)
       (begin
-	(gimp-selection-load image active-selection)
-	(gimp-edit-clear image shadow-layer)
+	(gimp-selection-load active-selection)
+	(gimp-edit-clear shadow-layer)
 	(gimp-image-remove-channel image active-selection)))
 
-  (if (and 
+  (if (and
        (= (car (gimp-layer-is-floating-sel drawable)) 0)
        (= from-selection FALSE))
       (gimp-image-raise-layer image drawable))
 
   (gimp-image-set-active-layer image drawable)
   (gimp-palette-set-background old-bg)
-  (gimp-image-enable-undo image)
+  (gimp-image-undo-enable image)
   (gimp-displays-flush)))
 
-(script-fu-register "script-fu-perspective-shadow" 
-		    "<Image>/Script-Fu/Shadow/Perspective"
+(script-fu-register "script-fu-perspective-shadow"
+		    "<Image>/Script-Fu/Shadow/Perspective..."
 		    "Add a perspective shadow"
-		    "Sven Neumann (neumanns@uni-duesseldorf.de)"
+		    "Sven Neumann <sven@gimp.org>"
 		    "Sven Neumann"
-		    "12/13/1997"
+		    "1999/12/21"
 		    "RGB RGBA GRAY GRAYA"
 		    SF-IMAGE "Image" 0
 		    SF-DRAWABLE "Drawable" 0
-		    SF-VALUE "Angle" "45"
-		    SF-VALUE "Relative horizon distance" "5.0"
-		    SF-VALUE "Relative shadow length" "1.0"
-		    SF-VALUE "Blur Radius" "3"
+		    SF-ADJUSTMENT "Angle" '(45 0 180 1 10 1 0)
+		    SF-ADJUSTMENT "Relative horizon distance" '(5 0 24 .1 1 1 1)
+		    SF-ADJUSTMENT "Relative shadow length" '(1 0 24 .1 1 1 1)
+		    SF-ADJUSTMENT "Blur Radius" '(3 0 1024 1 10 0 0)
 		    SF-COLOR "Color" '(0 0 0)
-		    SF-VALUE "Opacity" "80"
-		    SF-TOGGLE "Interpolate" TRUE
-		    SF-TOGGLE "Allow Resizing" FALSE) 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+		    SF-ADJUSTMENT "Opacity" '(80 0 100 1 10 0 0)
+		    SF-TOGGLE "Interpolate?" TRUE
+		    SF-TOGGLE "Allow Resizing?" FALSE)
